set echo on
set feedback on
set serveroutput on
var start_date  varchar2(80);
var end_date    varchar2(80);
var sid         varchar2(10);
var host        varchar2(80);
var user        varchar2(30);


spool 3.3.0_Update_Script_Backout.log;

begin
  select to_char(sysdate, 'mm-dd-yyyy hh24:mi:ss') into :start_date from dual;
  select instance_name into :sid from v$instance;
  select host_name into :host from v$instance;
  select user into :user from dual;

  dbms_output.put_line('Script executed on SID:  '||:sid);
  dbms_output.put_line('Script executed on host: '||:host);
  dbms_output.put_line('Script start date/time:  '||:start_date);
  dbms_output.put_line('Script executed by:      '||:user);
end;
/
SET DEFINE OFF;

DROP TABLE VAP.PERMISSION;

DROP INDEX VAP.ANNOUNCEMENT_BATCH_ORG_ID_IDX;

DROP INDEX VAP.AC_ID_BATCH_ID_CREATED_TS_IDX;

DROP INDEX VAP.MN_DELAYED_CONSENT_ID_IDX; 

DROP INDEX VAP.MN_CONSENT_DIR_ID_IDX; 

DROP INDEX VAP.PATIENT_CONSENT_DIR_EXP_IDX;

DROP INDEX VAP.CONSENT_AUDIT_PAT_TYPE_IDX;

CREATE OR REPLACE FUNCTION VAP.ISTESTPATIENT(ssn IN VARCHAR2)
  RETURN NUMBER 
IS
  BEGIN
    IF SUBSTR(ssn,1,3) = '000' THEN
      RETURN 1;
    END IF;
    
    IF SUBSTR(ssn,1,3) = '666' THEN
      RETURN 1;
    END IF;
    
    IF SUBSTR(ssn,1,1) = '9' THEN
      RETURN 1;
    END IF; 
    
    IF SUBSTR(ssn,4,2) = '00' THEN
      RETURN 1;
    END IF;
    
    IF SUBSTR(ssn,6,4) = '0000' THEN
      RETURN 1;
    END IF;
    
    RETURN 0;
  END;
/

CREATE OR REPLACE FORCE VIEW "VAP"."CONSENT_AUDIT_EX" ("CONSENT_AUDIT_ID", "ACTION_TYPE", "CONSENT_TYPE", "CREATED_DATE", "FACILITY", "FACILITY_NAME", "OPTOUT_REASON", "PATIENT_GIVEN_NAME", "PATIENT_ID", "PATIENT_LAST_NAME", "PATIENT_SSN", "POU_VALUE", "TIME_OF_EVENT", "USER_ID", "PATIENT_MIDDLE_NAME", "VISN_NAME", "VISN_NUMBER", "PARENT_STATION_NUMBER", "IS_TEST_PATIENT", "PARENT_FACILITY_NAME") AS 
SELECT c.CONSENT_AUDIT_ID, c.ACTION_TYPE, c.CONSENT_TYPE, c.CREATED_DATE,
     NVL(fa.FACILITY_STATION, c.FACILITY) AS FACILITY,
     NVL(fa.FACILITY_NAME, 'Unknown') AS FACILITY_NAME,
     c.OPTOUT_REASON, c.PATIENT_GIVEN_NAME, c.PATIENT_ID,
     c.PATIENT_LAST_NAME, c.PATIENT_SSN, c.POU_VALUE, c.TIME_OF_EVENT,
     LOWER(c.USER_ID), c.PATIENT_MIDDLE_NAME, v.VISN_NAME, v.VISN_NUMBER,
     CASE WHEN fa.FACILITY_PARENT IS NULL THEN fa.FACILITY_STATION ELSE fa.FACILITY_PARENT END AS PARENT_STATION_NUMBER,
     ISTESTPATIENT(c.PATIENT_SSN),
     NVL(fb.FACILITY_NAME, 'Unknown') PARENT_FACILITY_NAME
FROM (
  SELECT ci.CONSENT_AUDIT_ID, ci.ACTION_TYPE, ci.CONSENT_TYPE, ci.CREATED_DATE,
         ci.FACILITY, ci.FACILITY_NAME, ci.OPTOUT_REASON, ci.PATIENT_GIVEN_NAME, ci.PATIENT_ID,
         ci.PATIENT_LAST_NAME, ci.PATIENT_SSN, ci.POU_VALUE, ci.TIME_OF_EVENT,
         ci.USER_ID, ci.PATIENT_MIDDLE_NAME, 
         (SELECT fi.FACILITY_ID FROM FACILITY fi WHERE ci.FACILITY = fi.FACILITY_STATION) STATION_LOOKUP_TYPE,
         (SELECT fi.FACILITY_ID FROM FACILITY fi WHERE substr(ci.FACILITY,1,3) = fi.FACILITY_STATION) FACILITY_LOOKUP_TYPE
  FROM CONSENT_AUDIT ci
) c
LEFT OUTER JOIN FACILITY fa ON fa.FACILITY_ID =
     CASE
       WHEN c.STATION_LOOKUP_TYPE IS NOT NULL THEN c.STATION_LOOKUP_TYPE
       WHEN c.FACILITY_LOOKUP_TYPE IS NOT NULL THEN c.FACILITY_LOOKUP_TYPE
     END
LEFT OUTER JOIN FACILITY fb ON CASE WHEN fa.FACILITY_PARENT IS NULL THEN fa.FACILITY_STATION ELSE fa.FACILITY_PARENT END = fb.FACILITY_STATION
LEFT OUTER JOIN VISN v ON fa.VISN_ID = v.ID;

--run the create/replace script for this view because we swapped out some dependencies it has and most likely will invalidate it if we don't rerun it
CREATE OR REPLACE FORCE VIEW "VAP"."DELAYED_CONSENT_VIEW" ("DELAYED_CONSENT_ID", "PATIENT_IEN", "PATIENT_SSN", "PATIENT_LAST_NAME", "PATIENT_FIRST_NAME", "PATIENT_MIDDLE_NAME", "DATE_ADDED", "USER_ID", "CONSENT_TYPE_ID", "PARENT_STATION_NUMBER", "STATION_NUMBER", "FACILITY_NAME", "STATUS", "RESOLUTION_DATE", "IS_TEST_PATIENT", "PARENT_FACILITY_NAME", "VISN_NUMBER") AS 
SELECT DELAYED_CONSENT_ID, PATIENT_IEN, PATIENT_SSN, PATIENT_LAST_NAME, PATIENT_FIRST_NAME, PATIENT_MIDDLE_NAME, DATE_ADDED, USER_ID,
       CONSENT_TYPE_ID, SUBSTR(STATION_NUMBER,1,3), STATION_NUMBER, fa.FACILITY_NAME, STATUS, RESOLUTION_DATE, ISTESTPATIENT(PATIENT_SSN),
       fb.FACILITY_NAME AS PARENT_FACILITY_NAME, VISN_NUMBER
FROM DELAYED_CONSENT d 
LEFT OUTER JOIN FACILITY fa ON d.STATION_NUMBER = fa.FACILITY_STATION
LEFT OUTER JOIN FACILITY fb ON SUBSTR(d.STATION_NUMBER,1,3) = fb.FACILITY_STATION
LEFT OUTER JOIN VISN v ON fa.VISN_ID = v.ID;

ALTER TABLE VAP.CONSENT_AUDIT
DROP COLUMN IS_TEST_PATIENT;

DELETE FROM VAP.DESCRIPTOR
WHERE NAME IN ('manageBatchesFailedHeader','batchAnnounceDetailsCompletedColumn');

COMMIT;

SET DEFINE ON;

Begin
  select to_char(sysdate, 'mm-dd-yyyy hh24:mi:ss') into :end_date from dual;
  dbms_output.put_line('Script end date/time:    '||:end_date);
  dbms_output.put_line('Script executed in '||(to_date(:end_date, 'mm_dd_yyyy hh24:mi:ss') - to_date(:start_date, 'mm-dd-yyyy hh24:mi:ss')) *24*60*60||' seconds');
end;
/

Spool off;
