/**
 *
 */
package gov.va.nvap.common.spring;

import gov.va.nvap.common.validation.NullChecker;
import java.io.ByteArrayOutputStream;

import java.io.IOException;

import org.springframework.ws.client.support.interceptor.ClientInterceptor;
import org.springframework.ws.context.MessageContext;
import org.springframework.ws.client.WebServiceClientException;
import org.springframework.ws.WebServiceMessage;

import org.slf4j.LoggerFactory;
import org.slf4j.Logger;



/**
 *
 * 
 */
public class WSClientLoggingInterceptor implements ClientInterceptor {
    
    private static final Logger LOG =
        LoggerFactory.getLogger(WSClientLoggingInterceptor.class);
    
    private String webserviceName = "";    
    
    
    @Override
    public boolean handleRequest(MessageContext messageContext) throws WebServiceClientException {
        //return true;
         try {
            WebServiceMessage request = messageContext.getRequest();
            LOG.debug(this.webserviceName + " Request : " + this.getMessage(request));      
        } catch (IOException e) {
            LOG.error(this.webserviceName+" Request from handleResponse: ",e);
        }
        return true;          
    }
 
    @Override
    public boolean handleResponse(MessageContext messageContext) throws WebServiceClientException {
        //return true;
         try {
            WebServiceMessage response = messageContext.getResponse();
            LOG.debug("\n" + this.webserviceName + " Response : " + this.getMessage(response));    
        } catch (IOException e) {
            LOG.error(this.webserviceName+" Response from handleResponse: ",e);
        }
        return true;        
    }
 
    @Override
    public boolean handleFault(MessageContext messageContext) throws WebServiceClientException {
        //return true;
         try {
            WebServiceMessage response = messageContext.getResponse();
            LOG.info("\n" + this.webserviceName + " Fault Response : " + this.getMessage(response));
        } catch (IOException e) {
            LOG.error(this.webserviceName+" Response from handleFault: ",e);
        } 
        return true; 
    }
 
    @Override
    public void afterCompletion(MessageContext messageContext, Exception ex) throws WebServiceClientException {
//        try {
            //if(LOG.isDebugEnabled()) {
//                WebServiceMessage request = messageContext.getRequest();
//                LOG.info(this.webserviceName + " Request : " + this.getMessage(request));           
                //messageContext.getRequest().writeTo(System.out);
//                WebServiceMessage response = messageContext.getResponse();
//                LOG.info("\n" + this.webserviceName + " Response : " + this.getMessage(response));
                //messageContext.getResponse().writeTo(System.out);
            //}
            if(null != ex) { 
                LOG.error("\n" + this.webserviceName + " afterCompletion printing Exception: " + ex.getMessage(), ex);
            }
//        } catch (IOException e) {
//            LOG.error(this.webserviceName+" Exception:",e);
//        }
    }  
    
    /**
     * Get the value of webserviceName
     *
     * @return the value of webserviceName
     */
    public String getWebserviceName() {
        return webserviceName;
    }

    /**
     * Set the value of webserviceName
     *
     * @param webserviceName new value of webserviceName
     */
    public void setWebserviceName(String webserviceName) {
        this.webserviceName = webserviceName;
    }
    
    private String getMessage(WebServiceMessage message) throws IOException {
        String msgStr = null;
        if(NullChecker.isNullOrEmpty(msgStr)) {
            ByteArrayOutputStream buffer = new ByteArrayOutputStream();
            message.writeTo(buffer);
            msgStr = buffer.toString(java.nio.charset.StandardCharsets.UTF_8.name());
        }        
        return msgStr;
    }    
    
}
