/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package gov.va.med.nhin.vap.server.service.permission.contexthandler;

import gov.va.nvap.server.auth.ServiceAuthentication;
import gov.va.nvap.service.auth.ServiceAudit;
import gov.va.nvap.service.auth.ServiceAuthenticationException;
import gov.va.nvap.svc.consenteval.intf.PolicyEvaluationService;
import javax.annotation.Resource;
import javax.ejb.Stateless;
import javax.interceptor.Interceptors;
import javax.jws.WebMethod;
import javax.jws.WebService;
import javax.jws.soap.SOAPBinding;
import javax.xml.ws.WebServiceContext;

import oasis.names.tc.xacml._2_0.context.schema.os.RequestType;
import oasis.names.tc.xacml._2_0.context.schema.os.ResponseType;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.ApplicationContext;
import org.springframework.ejb.interceptor.SpringBeanAutowiringInterceptor;

import org.slf4j.LoggerFactory;
import org.slf4j.Logger;


/**
 *
 * @author Anand Sastry
 */
@Stateless(name = "XACMLContextHandlerService", mappedName = "XACMLContextHandlerService")
//@SOAPBinding(parameterStyle = SOAPBinding.ParameterStyle.BARE)
@WebService(serviceName = "XACMLContextHandlerService",
            portName = "XACMLContextHandlerPort",
            endpointInterface = "gov.va.med.nhin.vap.server.service.permission.contexthandler.XACMLContextHandlerPortType"/*,
            name = "XACMLContextHandlerPortType",
            wsdlLocation = "META-INF/xml-resources/web-services/XACMLContextHandlerService/wsdl/XACMLContextHandlerService.wsdl"*/)
@Interceptors(SpringBeanAutowiringInterceptor.class)
public class XACMLContextHandlerServiceBean implements XACMLContextHandlerPortType
{
    
    private static final Logger LOG =
        LoggerFactory.getLogger(XACMLContextHandlerServiceBean.class);
    /*
	* Inject Spring bean, so that we have all dependencies injected.
     */
    @Autowired
    @Qualifier(value = "PolicyEvaluationService")
    private PolicyEvaluationService policyEvaluationService;
    @Autowired
    private ApplicationContext applicationContext;
    @Resource
    private WebServiceContext context;
    private final ServiceAuthentication auth = new ServiceAuthentication();

    /**
     * @param request
     * @return
     */
    @Override
    public ResponseType checkPolicy(final RequestType request)
    {
        ServiceAudit sa;
        Integer success = 1;

        try {
            //System.out.println("Calling checkAuth...");
            sa = auth.checkAuth(context, applicationContext, "checkPolicy");
            //System.out.println("Returning from checkAuth...");
        }
        catch (ServiceAuthenticationException ex) {
            //ex.printStackTrace(System.out);
            LOG.error("checkAuth exception!", ex); 
            return null;
        }

        ResponseType response = null;
        long startTime = System.currentTimeMillis();

        try {            
            //System.out.println("calling checkPolicy...");            
            response = policyEvaluationService.checkPolicy(request);            
        }
        catch (Exception e) {
           //e.printStackTrace(System.out);
            LOG.error("checkPolicy exception!", e); 
            success = 0;
        }
        finally {
            if (sa != null) {
                sa.setSuccess(success);
                sa.setDuration((int) (System.currentTimeMillis() - startTime));
                auth.persist(applicationContext, sa);
            }
        }

        return response;
    }
}
