package gov.va.nvap.server.endpoint.adapter.announce;

import gov.va.med.nhin.adapter.announce.AnnouncePatient;
//import gov.va.med.nhin.adapter.announce.AnnouncePatient.Facilities;
import gov.va.med.nhin.adapter.announce.UserInfoType;

import gov.va.nvap.common.jaxb.JaxbUtil;
import gov.va.nvap.common.validation.Assert;
import gov.va.nvap.common.validation.NullChecker;
import gov.va.nvap.svc.consentmgmt.PIPInterface;
import gov.va.nvap.svc.consentmgmt.stub.adapter.announce.data.Announcement;
import gov.va.nvap.svc.consentmgmt.stub.adapter.announce.data.AnnouncerInterface;
import gov.va.nvap.svc.consentmgmt.stub.adapter.announce.data.PatientAnnouncer;
import gov.va.nvap.svc.consentmgmt.stub.adapter.announce.data.PatientAnnouncerException;
import gov.va.nvap.svc.consentmgmt.stub.adapter.announce.jaxb.AnnouncePatientRequest;
import gov.va.nvap.svc.consentmgmt.stub.adapter.announce.jaxb.AnnouncePatientResponse;
import gov.va.nvap.svc.consentmgmt.stub.adapter.announce.jaxb.AuthorizedOrganizationType;
import gov.va.nvap.svc.consentmgmt.stub.data.ConsentDirective;
import gov.va.nvap.svc.consentmgmt.stub.data.Organization;
import gov.va.nvap.svc.consentmgmt.stub.adapter.announce.data.AnnouncementOrg;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang.time.DateFormatUtils;
import org.apache.commons.lang.exception.ExceptionUtils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.springframework.beans.factory.annotation.Required;
import org.springframework.jms.core.JmsTemplate;
import org.springframework.ws.client.WebServiceIOException;
import org.springframework.ws.client.core.WebServiceTemplate;
import org.springframework.ws.soap.client.SoapFaultClientException;

/**
 * Announce patient on the NwHIN through the NwHIN Adapter.
 * 
 * @author Asha Amritraj
 * @author Irakli Kakushadze
 * @author Zack Peterson
 */
public class PatientAnnouncerEndpoint implements PatientAnnouncer {

        private static final Logger LOG =
            LoggerFactory.getLogger(PatientAnnouncerEndpoint.class);
        
        
	/**
	 * The announcer interface for creating batches of announcements and doing
	 * asynchronous announcements.
	 */
	AnnouncerInterface announcer;
	/**
	 * Spring JMS Template to send messages to the
	 * PatientAnnouncerForwardingQueue.
	 */
	JmsTemplate jmsTemplate;
	/**
	 * An interceptor to audit the announce.
	 * 
	 * PatientAnnouncerAuditInterceptor auditInterceptor;
	 */
	/**
	 * Convert to XML for JMS message.
	 */
	JaxbUtil patientAnnouncerJaxbHelper;
	/**
	 * The Adapter AnnouncePatient webservice template.
	 */
	WebServiceTemplate patientAnnounceWebserviceTemplate;
	/**
	 * The Policy information point, to be able to store announcements
	 * associated with a consent directive id.
	 */
	PIPInterface pipInterface;

	@Override
	public AnnouncePatientResponse announce(
			final AnnouncePatientRequest announcePatientRequest)
			throws PatientAnnouncerException {
		Assert.assertNotEmpty(announcePatientRequest, "AnnoucePatientRequest cannot be empty!");
		Assert.assertNotEmpty(announcePatientRequest.getAuthorizedOrganizations(), "Authorized organizations cannot be empty!");
		Assert.assertNotEmpty(announcePatientRequest.getPatient(), "Patient cannot be empty!");
		Assert.assertNotEmpty(announcePatientRequest.getPatient().getPatientId(), "Patient ID cannot be empty!");
		Assert.assertNotEmpty(announcePatientRequest.getUser(), "User cannot be empty!");
		Assert.assertNotEmpty(announcePatientRequest.getUser().getUserId(), "User ID cannot be empty!");
        
		// Announce for authorized organizations
		final AnnouncePatientResponse response = new AnnouncePatientResponse();

                final AnnouncePatient announcePatient = this
                        .constructAnnouncePatientMessage(announcePatientRequest);
                /*
                 * try { this.auditInterceptor.intercept(announcePatientRequest); }
                 * catch (final InterceptorException ex) { throw new
                 * PatientAnnouncerException(ex); }
                 */

                /* Commented out for testing Batch Announcements - 10/01/2012 - Anand Srivastava */
                
                // make the announcement request to PatientDiscovery
                int responseCode = this.makePatientDiscoveryRequest(announcePatient);
                response.setResult(responseCode);
                // Get the associated announcement
                final Long announcementId = announcePatientRequest.getAnnouncementId();
                // Store the announcement
                final Announcement announcement = this.announcer.getAnnouncementById(announcementId);
                // Get associated orgs if using ANNOUNCEMENT_ORGS table
                Collection<AnnouncementOrg> announceOrgs = null;
                if (announcement != null && announcement.getTargetOrganization() == null) {
                    announceOrgs = pipInterface.getAnnouncementOrgsByAnnouncement(announcement);
                }
                if (NullChecker.isNotEmpty(announcement)) {
                    announcement.setCompletedTs(new Date());
                    announcement.setScheduledTs(null);
                    /*
                     * The community response will be added response by the NwHIN
                     * adapter
                     * communityResponse.setPRPAIN201306UV02(resultFromNhin);
                     * response.getCommunityResponse().add(communityResponse);
                     */
                    // TODO: Verify that the code below works for multiple assigning
                    // authorities & Request adapter to send proper response. The
                    // communityResponse from NwHIN could be error.

                    // If the response is zero, then its success
                    /* Commented out for testing Batch Announcements - 10/01/2012 - Anand Srivastava */
                    announcement.setAnnouncementResult(Integer
                            .toString(responseCode));
                    announcement.setHieTransactionId(announcePatient.getHieTransactionId());

                    this.announcer.storeAnnouncement(announcement);

                    // Set result for each associated organization
                    // Set them all to the overall result for now
                    if (announceOrgs != null) {
                        for (AnnouncementOrg org : announceOrgs) {
                            org.setAnnounceResult(Integer.toString(responseCode));
                            pipInterface.updateAnnouncementOrg(org);
                        }
                    }
                }
                // Construct the AnnouncePatient root element for the webservice
                return response;
	}

	@Override
	public void announceAsync(
			final AnnouncePatientRequest announcePatientRequest)
			throws PatientAnnouncerException {
		Assert.assertNotEmpty(announcePatientRequest,
				"AnnoucePatientRequest cannot be empty!");
		Assert.assertNotEmpty(
				announcePatientRequest.getAuthorizedOrganizations(),
				"Authorized organizations cannot be empty!");
		Assert.assertNotEmpty(announcePatientRequest.getPatient(),
				"Patient cannot be empty!");
		Assert.assertNotEmpty(announcePatientRequest.getPatient()
				.getPatientId(), "Patient ID cannot be empty!");
		Assert.assertNotEmpty(announcePatientRequest.getUser(),
				"User cannot be empty!");
		Assert.assertNotEmpty(announcePatientRequest.getUser().getUserId(),
				"User ID cannot be empty!");
		// TODO: Write to ANNOUNCEMENTS table
		final AnnouncePatientMessageCreator messageCreator = new AnnouncePatientMessageCreator(
				announcePatientRequest, this.patientAnnouncerJaxbHelper);
		this.jmsTemplate.send(messageCreator);
	}

    @Override
    public AnnouncePatientResponse announceAll(
            AnnouncePatientRequest announcePatientRequest)
            throws PatientAnnouncerException {

        Assert.assertNotEmpty(announcePatientRequest, "AnnoucePatientRequest cannot be empty!");
        Assert.assertNotEmpty(announcePatientRequest.getPatient(), "Patient cannot be empty!");
        Assert.assertNotEmpty(announcePatientRequest.getPatient().getPatientId(), "Patient ID cannot be empty!");
        Assert.assertNotEmpty(announcePatientRequest.getUser(), "User cannot be empty!");
        Assert.assertNotEmpty(announcePatientRequest.getUser().getUserId(), "User ID cannot be empty!");

        final Date createdTs = new Date();

        final AnnouncePatientResponse response = new AnnouncePatientResponse();
        final AnnouncePatient announcePatient = this.constructAnnouncePatientMessage(announcePatientRequest);
        // make the announcement request to PatientDiscovery
        int responseCode = this.makePatientDiscoveryRequest(announcePatient);
        response.setResult(responseCode);
        // create a new announcement and persist it in the table.
        try {
            // Get consent directives for this patient
            List<String> patientIens = new ArrayList<String>();
            String patientIen = announcePatientRequest.getPatient().getPatientId();
            patientIens.add(patientIen);
            Collection<ConsentDirective> allConsentDirectives = this.pipInterface.getConsentHistory(patientIens);
            // Use the latest directive to store the announcement
            if (allConsentDirectives != null && !allConsentDirectives.isEmpty()) {
                final ConsentDirective cd = allConsentDirectives.iterator().next();
                final Announcement announcement = new Announcement();
                announcement.setAnnouncementResult(Integer.toString(responseCode));
                announcement.setCompletedTs(new Date());
                announcement.setCreatedTs(createdTs);
                announcement.setPatientConsentDir(cd);
                announcement.setPatientIen(patientIen);
                announcement.setUserId(announcePatientRequest.getUser().getUserId());
                announcement.setHieTransactionId(announcePatient.getHieTransactionId());
                announcer.storeAnnouncement(announcement);
                announcePatientRequest.setAnnouncementId(announcement.getAnnouncementId());
                
                // Insert orgs for announcement
                for (final AuthorizedOrganizationType authorizedOrgType : announcePatientRequest.getAuthorizedOrganizations()) {
                    Organization org = pipInterface.getOrganizationByOid(authorizedOrgType.getOrgOid());

                    String result = String.valueOf(responseCode);
                    
                    AnnouncementOrg announceOrg = new AnnouncementOrg();
                    announceOrg.setAnnouncement(announcement);
                    announceOrg.setTargetOrganization(org);
                    announceOrg.setAnnounceResult(result);
                    pipInterface.storeAnnouncementOrg(announceOrg);
                }
            }
        } catch (final Exception ex) {
            String msg = ex.getMessage();
            LOG.error("PatientAnnouncerEndpoint announceAll caught an Exception: "+msg, ex);
        }
        return response;
    }

    /**
     * Finds all announcements for a particular user.
     * 
     * @param userId User ID
     * @return List of objects of type Announcement found for this user
     */  
    @Override
    public List<Announcement> getAnnouncementsByUserId(String userId) {
        return this.announcer.getAnnouncementsByUserId(userId);
    }
    
    /**
     * Finds all announcements for a list of consent directive ID numbers
     * 
     * @param consentDirectiveIds list of consent directive ID numbers
     * @return List of objects of type Announcement found for this patient
     */
    @Override
    public List<Announcement> getAnnouncementsByConsentDirectiveId(List<Long> consentDirectiveIds) {
        return this.announcer.getAnnouncementsByConsentDirectiveId(consentDirectiveIds);
    }
    
    private AnnouncePatient constructAnnouncePatientMessage(final AnnouncePatientRequest announcePatientRequest) {
            // Get the list of assigning authorities
            final List<String> assigningAuthorities = new ArrayList<String>();
            for (final AuthorizedOrganizationType authorizedOrgType : announcePatientRequest.getAuthorizedOrganizations()) {
                    assigningAuthorities.add(authorizedOrgType.getOrgNumber());
            }
            // Create the announce patient message
            final AnnouncePatient announcePatient = new AnnouncePatient();
            // If authorized organizations are in request add to AnnouncePatient
            if (assigningAuthorities.size() > 0) {
                AnnouncePatient.Facilities allFacilities = new AnnouncePatient.Facilities();
                allFacilities.getFacility().addAll(assigningAuthorities);
                announcePatient.setFacilities(allFacilities);
            }
            announcePatient.setPatientId(announcePatientRequest.getPatient()
                            .getPatientId());
            final UserInfoType userInfoType = new UserInfoType();
            userInfoType.setUserId(announcePatientRequest.getUser().getUserId());
            userInfoType
                            .setFullName(announcePatientRequest.getUser().getFullName());
            userInfoType.setAuthenticationMethod(announcePatientRequest.getUser()
                            .getAuthenticationMethod());
            userInfoType.setRole(announcePatientRequest.getUser().getRole());
            announcePatient.setUserInfo(userInfoType);
            String hieTransactionId = DigestUtils.md5Hex(DateFormatUtils.format(new Date(), "yyyy-MM-dd-HH-mm-ss-SSS"));
            hieTransactionId = hieTransactionId.substring(24);
            announcePatient.setHieTransactionId(hieTransactionId);
            return announcePatient;
    }
        
    private int makePatientDiscoveryRequest(final AnnouncePatient announcePatientRequest) {
            // make the announcement request to PatientDiscovery
        int responseCode = 0;
        gov.va.med.nhin.adapter.announce.AnnouncePatientResponse adapterResponse = null;
        try {
            adapterResponse = 
                (gov.va.med.nhin.adapter.announce.AnnouncePatientResponse) this.patientAnnounceWebserviceTemplate
                                                                            .marshalSendAndReceive(announcePatientRequest);
            // set positive integer, 0, or -1 value (or possibly also the -100 value 
            // for the "System Busy" notice) from Adapter PatientDiscovery response as result value
            responseCode = adapterResponse.getReturn();
        } catch (WebServiceIOException e) {
            // Exceptions already logged in the Interceptor class
            //LOG.error("PatientAnnouncerEndpoint caught an WebServiceIOException: ", e);
            Throwable eRoot = e.getMostSpecificCause();
            if(eRoot.getClass() == org.apache.http.conn.ConnectTimeoutException.class || eRoot.getClass() == org.apache.http.conn.ConnectionPoolTimeoutException.class) {
                responseCode = AnnouncerInterface.ENDPOINTERROR_CONN_TIMEOUT;
            } else if(eRoot.getClass() == java.net.ConnectException.class) {                        
                if(eRoot.getMessage().contains("time")) {
                    // assume it's a connection time-out exception
                    responseCode = AnnouncerInterface.ENDPOINTERROR_CONN_TIMEOUT;
                } else {
                    // some other connection exception
                    responseCode = AnnouncerInterface.ENDPOINTERROR_CONN_OTHERERROR;
                }                        
            } else if(eRoot.getClass() == java.net.UnknownHostException.class) {
                responseCode = AnnouncerInterface.ENDPOINTERROR_CONN_UNKNOWNHOST;
            } else if(eRoot.getClass() == java.net.SocketTimeoutException.class) { 
                responseCode = AnnouncerInterface.ENDPOINTERROR_SOCKET_TIMEOUT;
            } else if(eRoot instanceof java.io.IOException || eRoot instanceof org.apache.http.HttpException) {
                responseCode = AnnouncerInterface.ENDPOINTERROR_IO_OTHERERROR;
            } else {
                responseCode = AnnouncerInterface.ENDPOINTERROR_OTHER;
            }                   
        } catch (SoapFaultClientException e) {
            // Exceptions already logged in the Interceptor class
            //LOG.error("PatientAnnouncerEndpoint received a SoapFault response from the PatientDiscovery webservice: ", e.getFaultStringOrReason());
            responseCode = AnnouncerInterface.ENDPOINTERROR_SOAPFAULT;
        } catch (Exception e) { 
            // Exceptions already logged in the Interceptor class
            //LOG.error("PatientAnnouncerEndpoint caught an default Exception: ", e);
            Throwable eRoot = ExceptionUtils.getRootCause(e); 
            if(eRoot instanceof org.apache.http.conn.ConnectTimeoutException) {
                responseCode = AnnouncerInterface.ENDPOINTERROR_CONN_TIMEOUT;
            } else if(eRoot instanceof java.net.ConnectException) {
                if(eRoot.getMessage().contains("time")) {
                    // assume it's a connection time-out exception
                    responseCode = AnnouncerInterface.ENDPOINTERROR_CONN_TIMEOUT;
                } else {
                    // some other connection exception
                    responseCode = AnnouncerInterface.ENDPOINTERROR_CONN_OTHERERROR;
                }                        
            } else if(eRoot instanceof java.net.UnknownHostException) {
                responseCode = AnnouncerInterface.ENDPOINTERROR_CONN_UNKNOWNHOST;
            } else if(eRoot instanceof java.net.SocketTimeoutException) { 
                responseCode = AnnouncerInterface.ENDPOINTERROR_SOCKET_TIMEOUT;
            } else if(eRoot instanceof java.io.IOException) {
                responseCode = AnnouncerInterface.ENDPOINTERROR_IO_OTHERERROR;
            } else {
                responseCode = AnnouncerInterface.ENDPOINTERROR_OTHER;
            }
        }                
        return responseCode;
    }

    /*
     * @Required public void setAuditInterceptor( final
     * PatientAnnouncerAuditInterceptor auditInterceptor) {
     * this.auditInterceptor = auditInterceptor; }
     */

    @Required
    public void setAnnouncer(final AnnouncerInterface announcer) {
            this.announcer = announcer;
    }

    @Required
    public void setJmsTemplate(final JmsTemplate jmsTemplate) {
            this.jmsTemplate = jmsTemplate;
    }

    @Required
    public void setPatientAnnouncerJaxbHelper(
                    final JaxbUtil patientAnnouncerJaxbHelper) {
            this.patientAnnouncerJaxbHelper = patientAnnouncerJaxbHelper;
    }

    @Required
    public void setPatientAnnounceWebserviceTemplate(
                    final WebServiceTemplate patientAnnounceWebserviceTemplate) {
            this.patientAnnounceWebserviceTemplate = patientAnnounceWebserviceTemplate;
    }

    @Required
    public void setPipInterface(final PIPInterface pipInterface) {
            this.pipInterface = pipInterface;
    }
}
