package gov.va.nvap.server.propertylookup;


import java.io.InputStream;
import java.io.IOException;
import java.util.Properties;
import javax.annotation.PostConstruct;
import javax.ejb.TransactionAttribute;
import javax.ejb.Singleton;
import javax.ejb.TransactionAttributeType;

import org.slf4j.LoggerFactory;
import org.slf4j.Logger;

import gov.va.nvap.common.file.FileUtil;
import gov.va.nvap.common.validation.NullChecker;
import gov.va.nvap.propertylookup.PropertyLookupLocal;
import gov.va.nvap.propertylookup.PropertyLookupRemote;


/**
 *
 * @author David Vazquez
 */
@TransactionAttribute(value = TransactionAttributeType.SUPPORTS)
@Singleton(name = "ServerPropertyFileLookup", mappedName = "ServerPropertyFileLookup")
public class ServerPropertyFileLookupBean implements PropertyLookupLocal, PropertyLookupRemote
{
    private static final Logger LOG = LoggerFactory.getLogger(ServerPropertyFileLookupBean.class);
    
    private static final String SERVER_PROPERTIES_LOCATION = "config/vap/server.properties";    
    
    Properties properties;
    
    @PostConstruct
    public void init()
    {
        InputStream in = null;
        Properties p;
        
        try {
            p = new Properties();
            in = FileUtil.getResourceAsStream(SERVER_PROPERTIES_LOCATION);
            if(null == in) {
                throw new IOException("InputStream returned null from FileUtil.getResourceAsStream()!");
            }            
            p.load(in);
        }
        catch (IOException ioe) {
            LOG.warn("Error loading server.properties.  Returning null.", ioe);
            p = null;
        }
        finally {
            try {
                if (in != null) {
                    in.close();
                }
            }
            catch (IOException ioe) {
                LOG.warn("Error closing input stream.", ioe);
            }
        }
        
        properties = p;
    }
    
    @Override
    public String getProperty(String key)
    {
        String ret = null;

        if (properties != null) {
            ret = properties.getProperty(key);
        }        
        
        if(null == ret) {
            throw new RuntimeException("Referenced a property named "+key+" which is not found in properties file!: "+SERVER_PROPERTIES_LOCATION);
        }        
        
        return ret;
    }
}
