package gov.va.nvap.svc.consentmgmt.stub.dao;

import gov.va.nvap.privacy.ConsentType;
import gov.va.nvap.svc.consentmgmt.stub.data.ConsentDirective;
import gov.va.nvap.svc.consentmgmt.stub.data.DetailedConsentDirective;
import java.text.SimpleDateFormat;  
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import javax.persistence.TypedQuery;
import org.apache.commons.lang.time.DateUtils;

/**
 *
 * @author Irakli Kakushadze
 * @since 05/28/2016
 */
public class PatientConsentDirDAO {

    @PersistenceContext
    private EntityManager em;

    // Public methods

    public SearchResponse find(final SearchRequest searchRequest) {
        final HashMap<String, Object> queryParams = new HashMap<String, Object>();
       
        SimpleDateFormat dateFormat = new SimpleDateFormat("MM/dd/YYYY HH:mm:ss", Locale.ENGLISH);
        String queryString = DetailedConsentDirective.constructQuery(searchRequest, queryParams);
        Query q = this.em.createNativeQuery(queryString, DetailedConsentDirective.class);
        SearchResponse searchResponse = new SearchResponse();
        
        //leave start and end date like it was
        if( !searchRequest.dateType.equalsIgnoreCase("entryDate") || !searchRequest.dateType.equalsIgnoreCase("optInDate")){
            if (searchRequest.startDate == null) {
                q.setParameter("start_date", null);
            } else {
                q.setParameter("start_date", dateFormat.format(searchRequest.startDate));
            }
            if (searchRequest.endDate == null) {
                q.setParameter("end_date", null);
            } else {
                q.setParameter("end_date", dateFormat.format(searchRequest.endDate));
            }
        }else {
            q.setParameter("start_date", null);
            q.setParameter("end_date", null);
        }
        
        //handle entry date (otherwise known as TIME_OF_EVENT)
        if (searchRequest.dateType.equalsIgnoreCase("entryDate")){
            q.setParameter("startTimeOfEvent", searchRequest.startTimeOfEvent == null ? null : dateFormat.format(searchRequest.startTimeOfEvent) );
            q.setParameter("endTimeOfEvent", searchRequest.endTimeOfEvent == null ? null : dateFormat.format(searchRequest.endTimeOfEvent) );
        }
        
        //handle opt-in date (otherwise known as OPTIN_DATE)
        if (searchRequest.dateType.equalsIgnoreCase("optInDate")){
            q.setParameter("startOptInDate", searchRequest.startTimeOfEvent == null ? null : dateFormat.format(searchRequest.startTimeOfEvent) );
            q.setParameter("endOptInDate", searchRequest.endTimeOfEvent == null ? null : dateFormat.format(searchRequest.endTimeOfEvent) );
        }
       
        if (searchRequest.consentTypeName.equals("NwHIN Authorization") || searchRequest.consentTypeName.isEmpty()) {
            q.setParameter("eheath_consent", "100");
        } else {
            q.setParameter("eheath_consent", "");
        }

        if (searchRequest.consentTypeName.equals("SSA Authorization") || searchRequest.consentTypeName.isEmpty()) {
            q.setParameter("ssa_consent", "102");
        } else {
            q.setParameter("ssa_consent", "");
        }

        if (searchRequest.userId.isEmpty()) {
            q.setParameter("user_id", null);
        } else {
              q.setParameter("user_id", searchRequest.userId + "%");
        }

        q.setParameter("from_record", searchRequest.fromPage * searchRequest.recordsPerPage + 1);

        int toPage = searchRequest.recordsPerPage;
        if (searchRequest.fromPage > 0) {
            toPage = (searchRequest.fromPage + 1) * searchRequest.recordsPerPage;
        }
        q.setParameter("to_record", toPage);

        // Set query parameters
        DAOUtil.setQueryParams(q, queryParams);
        
        searchResponse.consents = q.getResultList();
        searchResponse.count = new Long(searchResponse.consents.size());
        
        return searchResponse;
    }

    /**
     * gets the count of all entries meeting the search requirement, not just the displayed page count
     * @param searchRequest
     * @return 
     */
    public SearchResponse findCount(final SearchRequest searchRequest) {
        final HashMap<String, Object> queryParams = new HashMap<String, Object>();
         SimpleDateFormat dateFormat = new SimpleDateFormat("MM/dd/YYYY HH:mm:ss", Locale.ENGLISH);
        String queryString = DetailedConsentDirective.constructCountQuery(searchRequest, queryParams);
        Query q = this.em.createNativeQuery(queryString);
        SearchResponse searchResponse = new SearchResponse();
        String dateType = searchRequest.dateType != null ? searchRequest.dateType : "";
        
        //leave start and end date like it was
//        if (searchRequest.startDate == null) {
//            q.setParameter("start_date", null);
//        } else {
//            q.setParameter("start_date", dateFormat.format(searchRequest.startDate));
//        }
//        if (searchRequest.endDate == null) {
//            q.setParameter("end_date", null);
//        } else {
//            //q.setParameter("end_date", dateFormat.format(addTime(searchRequest.endDate, Calendar.HOUR, 24)));
//            q.setParameter("end_date", dateFormat.format(searchRequest.endDate));
//        }
        if( !dateType.equalsIgnoreCase("entryDate") || !dateType.equalsIgnoreCase("optInDate")){
            if (searchRequest.startDate == null) {
                q.setParameter("start_date", null);
            } else {
                q.setParameter("start_date", dateFormat.format(searchRequest.startDate));
            }
            if (searchRequest.endDate == null) {
                q.setParameter("end_date", null);
            } else {
                q.setParameter("end_date", dateFormat.format(searchRequest.endDate));
            }
        }else {
            q.setParameter("start_date", null);
            q.setParameter("end_date", null);
        }
        //handle entry date (otherwise known as TIME_OF_EVENT)
        if (dateType.equalsIgnoreCase("entryDate")){
            q.setParameter("startTimeOfEvent", searchRequest.startTimeOfEvent == null ? null : dateFormat.format(searchRequest.startTimeOfEvent) );
            q.setParameter("endTimeOfEvent", searchRequest.endTimeOfEvent == null ? null : dateFormat.format(searchRequest.endTimeOfEvent) );
        }
        
        //handle opt-in date (otherwise known as OPTIN_DATE)
        if (dateType.equalsIgnoreCase("optInDate")){
            q.setParameter("startOptInDate", searchRequest.startTimeOfEvent == null ? null : dateFormat.format(searchRequest.startTimeOfEvent) );
            q.setParameter("endOptInDate", searchRequest.endTimeOfEvent == null ? null : dateFormat.format(searchRequest.endTimeOfEvent) );
        }
        

        if (searchRequest.consentTypeName.equals("NwHIN Authorization") || searchRequest.consentTypeName.isEmpty()) {
            q.setParameter("eheath_consent", "100");
        } else {
            q.setParameter("eheath_consent", "");
        }

        if (searchRequest.consentTypeName.equals("SSA Authorization") || searchRequest.consentTypeName.isEmpty()) {
            q.setParameter("ssa_consent", "102");
        } else {
            q.setParameter("ssa_consent", "");
        }

        if (searchRequest.userId.isEmpty()) {
            q.setParameter("user_id", null);
        } else {
              q.setParameter("user_id", searchRequest.userId + "%");
        }

        // Set query parameters
        DAOUtil.setQueryParams(q, queryParams);

        searchResponse.consents = null;
        //the count should be the only thing returned so its entry 0.
        Object count = q.getResultList().get(0);
        searchResponse.count = Long.parseLong(count.toString());
        
        return searchResponse;
    }
    
    public ConsentDirective findByConsentDirId(final Long consentDirId) {
        
        final Query q = this.em.createNamedQuery("ConsentDirective.findByConsentDirId");
        q.setParameter("consentDirId", consentDirId);
        
        ConsentDirective cd = (ConsentDirective) q.getSingleResult();
        
        return cd;
    }
    
    public ConsentDirective findRestrictionAuthorizationFromConsentAudit(final String ien, final Date optInDate) {
        final String consentType = ConsentType.NW_HIN_ORGANIZATION_RESTRICTION_AUTHORIZATION.getValue();
        final Date optInDateLessAMinute = DateUtils.addMinutes(optInDate, -1);
        
        final TypedQuery<ConsentDirective> query = this.em.createQuery("SELECT p FROM ConsentDirective p WHERE p.optinConsentType.name = :consentType AND p.patientIen = :ien AND p.optinTS >= :optInDateLessAMinute AND p.optinTS <= :optInDate ORDER BY p.optinTS DESC", ConsentDirective.class);
        query.setParameter("consentType", consentType);
        query.setParameter("ien", ien);
        query.setParameter("optInDateLessAMinute", optInDateLessAMinute);
        query.setParameter("optInDate", optInDate);
        
        // Return the first one found
        List<ConsentDirective> list = query.getResultList();
        if (list.size() > 0) {
            return list.get(0);
        } else {
            return null;
        }
    }

    public void setEntityManager(final EntityManager entityManager) {
        this.em = entityManager;
    }

    // Private methods
    
	public static Date addTime(final Date date, final int field, final int amount) {
		final Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		cal.add(field, amount);
		final Date newDate = cal.getTime();
		return newDate;
	}
    
    // Inner classes

    public class SearchRequest {

        public String consentTypeName;
        public String dateType;
        public Date endDate;
        public Date endTimeOfEvent;
        public int fromPage;
        public boolean includeUnknownVisn;
        public int patientTypes;
        public int recordsPerPage;
        public String sortBy;
        public String sortOrder;
        public Date startDate;
        public Date startTimeOfEvent;
        public String stationNumbers;
        public String toPage;
        public String userId;
        public boolean isOptInReport;

    }

    public class SearchResponse {

        public List<DetailedConsentDirective> consents = null;
        public long count = 0;

    }
}
