/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package gov.va.nvap.web.dao;

import gov.va.nvap.web.user.document.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import org.springframework.transaction.annotation.Transactional;

/**
 *
 * @author 565114
 */
public class UserDocumentScheduleDAO {
    
    @PersistenceContext
    private EntityManager em;

    /**
     * Set the entity manager (Injected by Spring).
     * @param em
     */
    public void setEntityManager(final EntityManager em) {
        this.em = em;
    }
    
    // get current status for all scheduled exports for a user
    /*@Transactional
    public HashMap<String, Object> getSummary(String userId) {
        Query q = this.em.createQuery("SELECT NEW gov.va.nvap.web.user.document.UserDocumentScheduleSummary (uds.scheduleId, uds.reportType, uds.userDefinedReportName, uds.frequency, uds.format, uds.creationDate, uds.lastUpdatedDate, " + 
                                      "udsa.runStartDate, udsa.runEndDate, udsa.status, udsa.errorMessage, uds.frequencyText)" +
                                      "FROM UserDocumentSchedule uds RIGHT OUTER JOIN UserDocumentScheduleAudit udsa ON uds.scheduleId = udsa.scheduleId " +
                                      "WHERE uds.userId = :userId " +
                                      "AND udsa.isCurrent = 1");
        
        q.setParameter("userId", userId);
        
        Query cq = em.createQuery("SELECT COUNT(uds.scheduleId) FROM UserDocumentSchedule uds LEFT OUTER JOIN UserDocumentScheduleAudit udsa ON uds.scheduleId = udsa.scheduleId WHERE uds.userId = :userId and udsa.isCurrent = 1");
        cq.setParameter("userId", userId);
        
        HashMap<String, Object> hm = new HashMap<String, Object>();
        hm.put("results", (List<UserDocumentScheduleSummary>) q.getResultList());
        hm.put("count", (Long) cq.getSingleResult());
        
        return hm;
    }*/
    
    public UserDocumentSchedule getUserDocumentSchedule(final Long scheduleId) {
        return this.em.find(UserDocumentSchedule.class, scheduleId);
    }
    
    // get scheduled exports for a user
    @Transactional
    public HashMap<String, Object> getUserDocumentScheduledForUser(String userId, final String sortBy, String sortOrder, final Integer maxRows, final Integer startRow) {
        String query = "SELECT u FROM UserDocumentSchedule u WHERE u.userId = :userId ";
        Integer sort = sortBy.isEmpty() ? 4 : Integer.parseInt(sortBy);
        String queryParams = "";
        
        switch (sort) {
            case 1:
                queryParams += " ORDER BY u.userDefinedReportName";
                break;
            case 2:
                queryParams += " ORDER BY u.reportType";
                break;
            case 3: 
                queryParams += " ORDER BY u.format";
                break; 
            case 4: 
                queryParams += " ORDER BY u.creationDate";
                break; 
            case 5: 
                queryParams += " ORDER BY u.startDate";
                break; 
            case 6: 
                queryParams += " ORDER BY u.endDate";
                break; 
            default:
                queryParams += " ORDER BY u.creationDate";
                break;
        }

        switch(sortOrder) {
            case "asc":
                sortOrder = " ASC";
                break;
            case "desc":
                sortOrder = " DESC";
                break;
            default: 
                sortOrder = " DESC";
                break;
        }
        
        queryParams = queryParams + sortOrder;

        query = query + queryParams;
        Query q = em.createQuery(query).setFirstResult(startRow);

        if( maxRows != -1 ) //set the max rows if we have it
        {
            q.setMaxResults(maxRows);
        } 
        
        q.setParameter("userId", userId);
        
        Query cq = em.createQuery("SELECT COUNT(uds.scheduleId) FROM UserDocumentSchedule uds WHERE uds.userId = :userId");
        cq.setParameter("userId", userId);
        
        HashMap<String, Object> hm = new HashMap<String, Object>();
        hm.put("results", (List<UserDocumentScheduleSummary>) q.getResultList());
        hm.put("count", (Long) cq.getSingleResult());
        
        return hm;
    }
    
    @Transactional
    public Long getUserDocumentScheduleCount(String userId) {
        Query cq = em.createQuery("SELECT COUNT(uds.scheduleId) FROM UserDocumentSchedule uds WHERE uds.userId = :userId");
        cq.setParameter("userId", userId);
        
        return (Long) cq.getSingleResult();
    }
    
    @Transactional
    public UserDocumentSchedule create(UserDocumentSchedule userDocumentSchedule) throws Exception {
        try {
            this.em.persist(userDocumentSchedule);
        } catch (final Exception ex) {
            throw ex;
        }
        
        return userDocumentSchedule;
    }
    
    @Transactional
    public void delete(final Long id) throws Exception {
        try {
            UserDocumentSchedule userDocumentSchedule;

            userDocumentSchedule = this.em.getReference(UserDocumentSchedule.class, id);

            this.em.remove(userDocumentSchedule);
        } catch (final Exception ex) {
            throw ex;
        }
    }
    
    @Transactional
    public UserDocumentSchedule update(UserDocumentSchedule userDocumentSchedule) throws Exception {
        try {
            this.em.merge(userDocumentSchedule);
        } catch (final Exception ex) {
            throw ex;
        }
        
        return userDocumentSchedule;
    }
    
    @Transactional
    public void deleteByIds(final String userId, final String ids) {
        List<String> removalIds = Arrays.asList(ids.split(","));
        List<Long> removalIdsLong  = new ArrayList<Long>();
        
        for(String s : removalIds) removalIdsLong.add(Long.parseLong(s));
        
        final Query q = this.em.createQuery("DELETE FROM UserDocumentSchedule uds WHERE lower(uds.userId) = lower(:userId) AND uds.scheduleId IN (:ids)");

        q.setParameter("userId", userId);
        q.setParameter("ids", removalIdsLong);
        
        q.executeUpdate();
    }
    
    @Transactional
    public UserDocumentSchedule getNextScheduledTask() throws Exception {
       UserDocumentSchedule nextToRun = null;
       
       String queryString = ""
            +   "SELECT uds.SCHEDULE_ID, uds.USER_ID, uds.CREATION_DATE, uds.LAST_UPDATED_DATE, uds.REPORT_TYPE, uds.USER_DEFINED_REPORT_NAME, uds.FORMAT, uds.PERIOD_OF_REPORTING, uds.REPORT_PARAMETERS, uds.REPORT_VIEW_PARAMETERS, uds.FREQUENCY, uds.WEEKLY_RECURRENCE_WEEK, "
               + "uds.WEEKLY_RECURRENCE_DAY_OF_WEEK, uds.MONTHLY_RECURRENCE_MONTH, uds.MONTHLY_RECURRENCE_DAY, uds.START_DATE, uds.END_DATE, uds.FREQUENCY_TEXT \n" +
                "FROM USER_DOCUMENT_SCHEDULE uds\n" +
                "WHERE (uds.END_DATE >= TRUNC(SYSDATE + 1) - 1/(24*60*60) OR uds.END_DATE IS NULL)\n" +
                "AND (SELECT COUNT('x') FROM USER_DOCUMENT_SCHEDULE_AUDIT a WHERE a.SCHEDULE_ID = uds.SCHEDULE_ID AND TRUNC(a.RUN_START_DATE) = TRUNC(SYSDATE)) = 0\n" +
                "AND (\n" +
                "     uds.FREQUENCY = 'DAILY' --always run the DAILY ones\n" +
                "     OR \n" +
                "     (uds.FREQUENCY = 'WEEKLY' AND TO_CHAR(SYSDATE,'D') = TO_CHAR(uds.WEEKLY_RECURRENCE_DAY_OF_WEEK) AND MOD(TO_CHAR(SYSDATE,'WW') - TO_CHAR(uds.START_DATE, 'WW'), uds.WEEKLY_RECURRENCE_WEEK) = 0) -- logic for WEEKLY\n" +
                "     OR \n" +
                "     (1 = \n" +
                "      CASE WHEN uds.MONTHLY_RECURRENCE_DAY = 'first' AND TO_CHAR(SYSDATE, 'D') = 1 AND MOD(TO_CHAR(SYSDATE, 'MM') - TO_CHAR(uds.START_DATE, 'MM'), uds.MONTHLY_RECURRENCE_MONTH) = 0 THEN\n" +
                "        1\n" +
                "      WHEN uds.MONTHLY_RECURRENCE_DAY = 'fifteenth' AND TO_CHAR(SYSDATE, 'D') = 15 AND MOD(TO_CHAR(SYSDATE, 'MM') - TO_CHAR(uds.START_DATE, 'MM'), uds.MONTHLY_RECURRENCE_MONTH) = 0 THEN\n" +
                "        1\n" +
                "      WHEN uds.MONTHLY_RECURRENCE_DAY = 'last' AND TO_CHAR(SYSDATE, 'D') = TO_CHAR(LAST_DAY(SYSDATE), 'DD') AND MOD(TO_CHAR(SYSDATE, 'MM') - TO_CHAR(uds.START_DATE, 'MM'), uds.MONTHLY_RECURRENCE_MONTH) = 0 THEN\n" +
                "        1\n" +
                "      ELSE \n" +
                "        0\n" +
                "      END \n" +
                "     ) -- logic for MONTHLY\n" +
                "    )\n" +
                "AND ROWNUM = 1\n" +
                "ORDER BY uds.CREATION_DATE DESC";
       
       Query q = this.em.createNativeQuery(queryString, UserDocumentSchedule.class);
       
       try {
           nextToRun = (UserDocumentSchedule) q.getSingleResult();
       } catch(Exception e) {
           //do nothing
       }
       
       return nextToRun;
    }
}
