package gov.va.nvap.web.helper.report;

import java.io.UnsupportedEncodingException;
import java.security.GeneralSecurityException;
import java.util.HashMap;
import java.util.Map;

import org.identityconnectors.common.security.GuardedString;

import gov.va.nvap.common.validation.NullChecker;

public class Ssn {

	private String ssn = null;
	private String ssnMasked = null;
	private boolean isTestPatient = false;
	private boolean isEmpty = true;

    /**
     * Adds the given SSN to the result map that is passed to detailed report JSPs.
     * If SSN is valid, "ssn" parameter will contain the SSN and "ssnMasked" parameter will contain the masked SSN;
     * Otherwise, "ssn" parameter will be an empty strings, and "ssnMasked" will contain the relevant error information.
     * 
     * @param resultMap Result map to add SSN to
     * @param ssn SSN to be added to the result map
     */
	public static void addSsnToResultMap(Map<String, Object> resultMap, GuardedString ssn) {
			if (NullChecker.isNullOrEmpty(ssn)) {
				resultMap.put("ssn", "");
				resultMap.put("ssnMasked", "Unavailable");
			} else if (decryptToString(ssn).matches("[0-9]{9}")) {
				resultMap.put("ssn", ssn);
				resultMap.put("ssnMasked", maskSsn(ssn));
			} else if (decryptToString(ssn).matches("[0-9]{3}-[0-9]{2}-[0-9]{4}")) {
				ssn = new GuardedString(decryptToString(ssn).replaceAll("-", "").toCharArray());
				resultMap.put("ssn", ssn);
				resultMap.put("ssnMasked", maskSsn(ssn));
			} else {
				resultMap.put("ssn", "");
				resultMap.put("ssnMasked", "Unavailable");
			}
	}
    public static String decryptToMaskSsn(GuardedString string) {
        final StringBuilder buf = new StringBuilder();
        string.access(
                new GuardedString.Accessor() {

                    public void access(char [] chars) {
                        buf.append(chars);
                    }
                });
        return buf.toString();
    }
    
    public static String decryptToString(GuardedString string) {
        final StringBuilder buf = new StringBuilder();
        string.access(
                new GuardedString.Accessor() {

                    public void access(char [] chars) {
                        buf.append(chars);
                    }
                });
        return buf.toString();
    }
//	private boolean checkPatient(GuardedString ssn) {
//		return NullChecker.isNotEmpty(ssn) && // Ignore null entries
//				decryptToString(ssn).length() == 9 && ( // Ignore entries that aren't exactly 9
//										// digits
//						decryptToString(ssn).startsWith("000") || // Area number 000 denotes a test
//											// patient
//						decryptToString(ssn).startsWith("666") || // Area number 666 denotes a
//													// test patient
//						decryptToString(ssn).startsWith("9") || // Area number in 900-999 range
//												// denotes a test patient
//						decryptToString(ssn).substring(3, 5).equals("00") || // Group number 00
//															// denotes a test
//															// patient
//						decryptToString(ssn).substring(5, 9).equals("0000")); // Serial number
//																// 0000 denotes
//																// a test
//																// patient
//	}

	/**
	 * Masks the given SSN so that only the last 4 digits are visible.
	 * 
	 * @param ssn SSN to be masked
	 * @return masked SSN
	 */
    public static GuardedString maskSsn(GuardedString ssn) {
        // Temporarily disable masking for testing purposes.
        if (NullChecker.isNullOrEmpty(ssn)){
            return new GuardedString();
        } else if (decryptToString(ssn).matches("[0-9]{9}")) {
            return new GuardedString((decryptToString(ssn).substring(0, 3) + "-" + decryptToString(ssn).substring(3, 5) + "-" + decryptToString(ssn).substring(5)).toCharArray());
        } else if (decryptToString(ssn).matches("[0-9]{3}-[0-9]{2}-[0-9]{4}")) {
            return ssn;
        } else {
            return ssn;
        }
    }

    public static String formulaInjection(String txt) {
        if (txt.startsWith("=") ||
        	txt.startsWith("+") || 
        	txt.startsWith("-")) {
        	txt = "\'"+txt;
        }
        return txt;
    }
    
    public static void main(String[] args){
    	Map<String, Object> resultMap = new HashMap<String, Object>();
    	//Ssn ss = new Ssn();
    	GuardedString ssn = new GuardedString("123-45-6789".toCharArray());
    	Ssn.addSsnToResultMap(resultMap,ssn);
    	
    } 
    
}
