package gov.va.nvap.web.user.document;

import java.io.Serializable;
import java.util.Date;
import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Lob;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

/**
 *
 * @author Johann P. Sonnenberg
 * 
 * This bean represents a scheduled export audit record/log. One is created every time managed scheduled export (UserDocumentSchedule) is run
 */

@Entity
@Table(name = "USER_DOCUMENT_SCHEDULE_AUDIT")
public class UserDocumentScheduleAudit implements Serializable {
    private static final long serialVersionUID = 1L;
    
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Basic(optional = false)
    @Column(name = "AUDIT_ID")
    private Long auditId;
    
    @Basic(optional = false)
    @Column(name = "SCHEDULE_ID")
    private Long scheduleId;
    
    @Basic(optional = false)
    @Column(name = "RUN_START_DATE")
    @Temporal(TemporalType.TIMESTAMP)
    private Date runStartDate;
    
    @Basic(optional = false)
    @Column(name = "RUN_END_DATE")
    @Temporal(TemporalType.TIMESTAMP)
    private Date runEndDate;
    
    @Basic(optional = false)
    @Column(name = "STATUS")
    private String status;
    
    @Lob
    @Column(name = "ERROR_MESSAGE")
    private String errorMessage;
    
    @Basic(optional = false)
    @Column(name = "FORMAT")
    private String format;
    
    @Column(insertable = false, updatable = false, name = "CHECK_DATE")
    private String checkDate;

    public UserDocumentScheduleAudit() {
    }

    public UserDocumentScheduleAudit(Long auditId) {
        this.auditId = auditId;
    }
    
    public UserDocumentScheduleAudit(Long scheduleId, Date runStartDate, Date runEndDate, String status, String errorMessage, String format, String checkDate) {
        this.scheduleId = scheduleId;
        this.runStartDate = runStartDate;
        this.runEndDate = runEndDate;
        this.status = status;
        this.errorMessage = errorMessage;
        this.format = format;
        this.checkDate = checkDate;
    }

    public UserDocumentScheduleAudit(Long auditId, Long scheduleId, Date runStartDate, Date runEndDate, String status, String errorMessage, String format, String checkDate) {
        this.auditId = auditId;
        this.scheduleId = scheduleId;
        this.runStartDate = runStartDate;
        this.runEndDate = runEndDate;
        this.status = status;
        this.errorMessage = errorMessage;
        this.format = format;
        this.checkDate = checkDate;
    }

    public Long getAuditId() {
        return auditId;
    }

    public void setAuditId(Long auditId) {
        this.auditId = auditId;
    }

    public Long getScheduleId() {
        return scheduleId;
    }

    public void setScheduleId(Long scheduleId) {
        this.scheduleId = scheduleId;
    }

    public Date getRunStartDate() {
        return runStartDate;
    }

    public void setRunStartDate(Date runStartDate) {
        this.runStartDate = runStartDate;
    }

    public Date getRunEndDate() {
        return runEndDate;
    }

    public void setRunEndDate(Date runEndDate) {
        this.runEndDate = runEndDate;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public String getErrorMessage() {
        return errorMessage;
    }

    public void setErrorMessage(String errorMessage) {
        this.errorMessage = errorMessage;
    }

    public String getFormat() {
        return format;
    }

    public void setFormat(String format) {
        this.format = format;
    }

    public String getCheckDate() {
        return checkDate;
    }

    public void setCheckDate(String checkDate) {
        this.checkDate = checkDate;
    }
}