package gov.va.nvap.web.util.status;

import gov.va.med.nhin.adapter.audit.ActionType;
import gov.va.med.nhin.adapter.audit.ActionValuesType;
import gov.va.med.nhin.adapter.audit.GetAudits;
import gov.va.med.nhin.adapter.audit.StringValuesType;
import gov.va.nvap.audit.BulkDownloadDocs;
import gov.va.nvap.common.exception.StackTraceUtil;
import gov.va.nvap.common.transformer.xml.XMLToString;
import gov.va.nvap.common.util.DocumentBuilderFactoryUtil;
import gov.va.nvap.common.validation.NullChecker;
import gov.va.nvap.service.adapter.audit.AdapterAuditManager;
import gov.va.nvap.service.adapter.audit.DirectAuditManager;
import gov.va.nvap.service.pdq.PatientDemographicsQuery;
import gov.va.nvap.service.pdq.PatientMatchQuery;
import gov.va.nvap.service.pdq.PatientProfile;
import gov.va.nvap.service.pdq.PdqService;
import gov.va.nvap.svc.consentmgmt.stub.FacilityResolver;
import gov.va.nvap.web.report.ReportDataProcessor;
import gov.va.nvap.propertylookup.PropertyLookup;
import gov.va.nvap.propertylookup.PropertyLookupLocal;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.StringWriter;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.Month;
import java.util.Calendar;
import java.util.Date;
import java.util.Properties;
import javax.ejb.EJB;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.io.IOUtils;
import org.apache.http.HttpResponse;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.identityconnectors.common.security.GuardedString;
import org.json.JSONException;
import org.json.JSONObject;
import weblogic.security.internal.SerializedSystemIni;
import weblogic.security.internal.encryption.ClearOrEncryptedService;
import weblogic.security.internal.encryption.EncryptionService;
import org.slf4j.LoggerFactory;
import org.slf4j.Logger;
import org.w3c.dom.Document;


/**
 *
 * @author Johann P. Sonnenberg
 */

public class StatusChecker extends gov.va.nvap.web.app.ResponseDispatcherHttpServlet {
    
    private static final long serialVersionUID = 1L;
    
    private static final Logger LOG = LoggerFactory.getLogger(StatusChecker.class);    
    
    //comment out the field - unused field
    //@Autowired
    //private ResourceLoader resourceLoader;  
    
    private PropertyLookup propertyLookup; 
    
    private PropertyLookup serverPropertyLookup;   
    
    //used for calls to the Exchange Service
    private AdapterAuditManager adapterAuditManager;

    //used for calls to MVI Service and VistA Service
    private PdqService pdqService;
    
    //Direct Service    
    private DirectAuditManager directAuditManager;  

    //used for calls to the ESR Service
    private FacilityResolver facilityResolver;    
    
    
    @Override
    public void unspecified(final HttpServletRequest request, final HttpServletResponse response) throws ServletException, IOException {
        //default page
        this.forward(request, response, "show");
    }
    
    public void checkExchangeStatus(final HttpServletRequest request, final HttpServletResponse response) throws ServletException, IOException {
        Boolean pass = true;
        String error = "";
        
        //do the call to the service
        try {
            final GetAudits getAuditsRequest = new GetAudits();

            final StringValuesType patientSSNs = new StringValuesType();
            patientSSNs.getValue().add("000000000");
            patientSSNs.setNotIn(false);
            getAuditsRequest.setPatientSSNs(patientSSNs);

            final ActionValuesType actionsType = new ActionValuesType();
            actionsType.getValue().add(ActionType.RETRIEVE_DOCUMENT_OUT);
            getAuditsRequest.setActions(actionsType);

            this.adapterAuditManager.getAudits(getAuditsRequest);
        } catch(Exception e) {
            pass = false;
            error = StackTraceUtil.getStackTraceUtilAsString(e);
        } 
        //package it up into JSON and send it back to the caller
        request.setAttribute("results", translateToJSON("Exchange", pass, error));

        this.forward(request, response, "resultsJSON");
    }
    
    public void checkMVIStatus(final HttpServletRequest request, final HttpServletResponse response) throws ServletException, IOException {
        Boolean pass = true;
        String error = "";
        
        //do the call to the service
        try{
            PatientProfile testPatient = new PatientProfile();
            // java.util.Date was deprecated in JEE 8
            //testPatient.setDob(new Date("01/01/1970"));
            LocalDate birthDay = LocalDate.of(1970, Month.JANUARY, 01);
            Date birthDate = java.sql.Date.valueOf(birthDay);
            testPatient.setDob(birthDate);             
            testPatient.setFirstName("CHDRONE");
            testPatient.setLastName("CHDRONE");
            testPatient.setSsn("000000000");
            
            PatientMatchQuery pmq = new PatientMatchQuery();
            
            pmq.setPatientProfile(testPatient);
            
            this.pdqService.searchPatient(pmq);
        } catch(Exception e) {
            pass = false;
            error = StackTraceUtil.getStackTraceUtilAsString(e);
        }
        
        //package it up into JSON and send it back to the caller
        request.setAttribute("results", translateToJSON("MVI", pass, error));

        this.forward(request, response, "resultsJSON");
    }
    
    public void checkDirectStatus(final HttpServletRequest request, final HttpServletResponse response) throws ServletException, IOException {
        Boolean pass = true;
        String error = "";
        
        //do the call to the service
        try{
            final GetAudits getAuditsRequest = new GetAudits();

            final StringValuesType patientSSNs = new StringValuesType();
            patientSSNs.getValue().add("000000000");
            patientSSNs.setNotIn(false);
            getAuditsRequest.setPatientSSNs(patientSSNs);

            final ActionValuesType actionsType = new ActionValuesType();
            actionsType.getValue().add(ActionType.RETRIEVE_DOCUMENT_OUT);
            getAuditsRequest.setActions(actionsType);
            
            this.directAuditManager.getAudits(getAuditsRequest);
        } catch(Exception e) {
            pass = false;
            error = StackTraceUtil.getStackTraceUtilAsString(e);
        }
        
        //package it up into JSON and send it back to the caller
        request.setAttribute("results", translateToJSON("Direct", pass, error));

        this.forward(request, response, "resultsJSON");
    }
    
    public void checkVistAStatus(final HttpServletRequest request, final HttpServletResponse response) throws ServletException, IOException {
        Boolean pass = true;
        String error = "";
        
        //do the call to the service
        try{
            PatientDemographicsQuery query = new PatientDemographicsQuery();
            query.setPatientId("1012581676V377802");
            
            pdqService.getPatientDemographics(query);
        } catch(Exception e) {
            pass = false;
            error = StackTraceUtil.getStackTraceUtilAsString(e);
        }
        
        //package it up into JSON and send it back to the caller
        request.setAttribute("results", translateToJSON("VistA", pass, error));

        this.forward(request, response, "resultsJSON");
    }
    
    public void checkESRStatus(final HttpServletRequest request, final HttpServletResponse response) throws ServletException, IOException {
        Boolean pass = true;
        String error = "";
        
        //do the call to the service
        try {
            // using the CHDRONE ICN because it works everywhere
            this.facilityResolver.getPatientPreferredFacility("1012581676V377802");
        } catch (Exception e) {
            pass = false;
            error = StackTraceUtil.getStackTraceUtilAsString(e);
        }
        
        //package it up into JSON and send it back to the caller
        request.setAttribute("results", translateToJSON("ESR", pass, error));
        
        this.forward(request, response, "resultsJSON");
    }
    
    public void checkBulkDownloadStatus(final HttpServletRequest request, final HttpServletResponse response) throws ServletException, IOException {
        Boolean pass = true;
        String error = "";
        
        //do the call to the service
        try {
            SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
            String today = formatter.format(new Date());
            ClearOrEncryptedService encSvc;
            String domainHome = new File("dummy").getAbsoluteFile().getParent();
            EncryptionService encryptSrv = SerializedSystemIni.getEncryptionService(domainHome);
            encSvc = new ClearOrEncryptedService(encryptSrv);            
            
            GuardedString vapIdClear = ReportDataProcessor.toGuardedString(encSvc.decrypt(this.loadVAPId()));
            
            getbulkDownloadDocs().downloadRetrievedDocs(ReportDataProcessor.decryptToString(vapIdClear), today, today, "1", "0000000", "000000000", "TestPatient", "TestPatient");
        } catch (Exception e) {
            pass = false;
            error = StackTraceUtil.getStackTraceUtilAsString(e);
        }
        
        //package it up into JSON and send it back to the caller
        request.setAttribute("results", translateToJSON("BulkDownload", pass, error));
        
        this.forward(request, response, "resultsJSON");
    }
    
    private String translateToJSON(String serviceChecked, Boolean pass, String error) {
        String jsonString = "";
        
        DateFormat dateFormat = new SimpleDateFormat("MM/dd/yyyy h:mm:ss a");
        
        Calendar todayCal = Calendar.getInstance();
        String today = dateFormat.format(todayCal.getTime());
        
        try {
            StringWriter json = new StringWriter();
            JSONObject obj = new JSONObject();
            
            obj.put("serviceChecked", serviceChecked);
            obj.put("pass", pass);
            obj.put("error", error);
            obj.put("dateTimeChecked", today);
            
            obj.write(json);

            jsonString = json.toString();
        } catch (JSONException ex) {
            LOG.error("translateToJSON", ex);
        }
        
        return jsonString;
    }
        
    //used to make calls to the bulk document download
    public BulkDownloadDocs getbulkDownloadDocs()
    {
    	String useMock =""; 
    	useMock = System.getProperty("mock.bulk.download")!=null?System.getProperty("mock.bulk.download"):"";
    	String mockClass="";    
        
    	 if(useMock.equalsIgnoreCase("true"))
         {
    	      mockClass = "bulkDownloadDocs-mock";     		 
    	 }    	   
    	 else
    	 {
    	      mockClass = "bulkDownloadDocs";    		
    	 }
         
        final BulkDownloadDocs bulkDownloadBean = getBean(mockClass, BulkDownloadDocs.class);
        
    	return bulkDownloadBean;    	
    }
    
    //used to get the VAP_ID from a properties file so we can use it to make the call to check on the bulk document download service
    private String loadVAPId() throws IOException
    {
        File file;
        
        if (System.getProperty("vap.bulkDownload.config") != null) {
            final String path = FilenameUtils.normalize(System.getProperty("vap.bulkDownload.config"));
            file = new File(path);
        } else {
            file = new File("config/vap/bulkDownload.properties");
        }

        InputStream stream = null;
        Properties props = new Properties();
        
        try {
            stream = new FileInputStream(file);
            props.load(stream);
        } finally {
            IOUtils.closeQuietly(stream);
        }
        
        return props.getProperty("vap.identifier");
    }
    
    public void checkPdpSacStatus(final HttpServletRequest request, final HttpServletResponse response) throws ServletException, IOException {
        Boolean pass = true;
        String error = "";        
        
        // do the call to the service
        try {
            this.makePDPSACSoapRequest();
        } catch (Exception e) {
            pass = false;
            error = StackTraceUtil.getStackTraceUtilAsString(e);            
        }     
        
        // package it up into JSON and send it back to the caller
        request.setAttribute("results", translateToJSON("PDPSAC", pass, error));

        this.forward(request, response, "resultsJSON");         
    }

    private void makePDPSACSoapRequest() throws Exception {
        String soapResponseBodyStr = null;
        
        final String REQUEST_XML_PATH = "gov/va/nvap/web/util/status/testPDPSACSOAPRequest.xml";          

        final String SOAP_ACTION="";
        
        ClassLoader classLoader = getClass().getClassLoader(); 
        try (InputStream requestXMLIStream = classLoader.getResourceAsStream(REQUEST_XML_PATH);) {
            
            String soapRequestBodyStr = IOUtils.toString(requestXMLIStream, StandardCharsets.UTF_8.name()); 
            requestXMLIStream.close();
            
            // Get pdpWebService.serviceLocation property in server.properties here
            final String SAC_SERVER_URL = serverPropertyLookup.getProperty("pdpWebService.serviceLocation"); 

            soapResponseBodyStr = this.callSOAPServer(soapRequestBodyStr, SAC_SERVER_URL, SOAP_ACTION);            
        } // end try-with-resource 
        
        // if made it this far, then test for "Permit" in response which 
        // is always supposed to be there for Patient Discovery IN request to PDP/SAC from VAP
        if(NullChecker.isNullOrEmpty(soapResponseBodyStr) || !soapResponseBodyStr.contains("Permit")) {
            throw new Exception("VAP request to PDP/SAC Service did not return Permit response! \n");
        }
    }

    /**
     * Note: Wrote this method using the java.net.HttpURLConnection library so 
     * that it would work with the existing WebLogic Server SSL library implementation and certs.
     * 
     * @param soapRequestBody
     * @param serverURL
     * @param soapAction
     * @return
     * @throws Exception 
     */
    private String callSOAPServer(String soapRequestBody, String serverURL, String soapAction) throws Exception {

        String result = null;
        
        HttpURLConnection conn = null;        

        // Set the timeout in milliseconds until a connection is established.
        int timeoutConnectionMS = 15000;
        // Set the default socket timeout (SO_TIMEOUT)
        // in milliseconds which is the timeout for waiting for data.
        int timeoutSocketMS = 35000;
        
        //InputStream is = null;
        OutputStream reqStream = null;
        try {
            URL requestUrl = new URL(serverURL);
            
            conn = (HttpURLConnection) requestUrl.openConnection();
            
            conn.setDoOutput(true); 
            conn.setDoInput(true);

            conn.setConnectTimeout(timeoutConnectionMS);
            conn.setReadTimeout(timeoutSocketMS);
            
            conn.setRequestMethod("POST");
           
            conn.setRequestProperty("Content-type", "text/xml; charset=utf-8");
            conn.setRequestProperty("SOAPAction", soapAction);
                      
            //conn.connect();                       
            
            reqStream = conn.getOutputStream();
            reqStream.write(soapRequestBody.getBytes());
            reqStream.flush();
   
            if(!NullChecker.isNullOrEmpty(conn) && conn.getResponseCode() > 299) {
                throw new RuntimeException("\nCall to PDP/SAC Failed : HTTP error code : " + conn.getResponseCode() + "\nHTTP response : " + conn.getResponseMessage());
            } 
            
            DocumentBuilderFactory dbFactory = DocumentBuilderFactoryUtil.getDocumentBuilderFactory(null, true, false, false);
            dbFactory.setFeature("http://apache.org/xml/features/disallow-doctype-decl", true); // external entity injection fortify fix
            DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
            
            try (InputStream is = conn.getInputStream();) {
                Document doc = dBuilder.parse(is);
                result = new XMLToString().transform(doc);
            } catch (IOException e) {
                LOG.error("callSOAPServer", e);
            }

        } catch (Exception e) {
            throw new Exception(e);
        } finally {
            try {
//                if(!NullChecker.isNullOrEmpty(is)) { 
//                    is.close();
//                }                
                if(!NullChecker.isNullOrEmpty(reqStream)) { 
                    reqStream.close();
                }
                // shut down the connection
                if(!NullChecker.isNullOrEmpty(conn)) { 
                    conn.disconnect();                    
                }
            } catch (IOException ex) {
                LOG.error("callSOAPServer", ex);
            }
        }
        return result;
    }
    
    public void checkDASCRUDStatus(final HttpServletRequest request, final HttpServletResponse response) throws ServletException, IOException {
        Boolean pass = true;
        String error = "";        
        
        // do the call to the service
        try {
            // check the status by making a simple collection count request, and use das.retrieveDocument property with "count" appended to the URL value
            String dasRetrieveDocumentEndPoint =  propertyLookup.getProperty("das.retrieveDocument");  
            String requestURL = dasRetrieveDocumentEndPoint+"count";  
            
            String restResponseBodyStr = this.makeRESTGETRequest(requestURL, "application/json");

            // if made it this far, then test for "count" in response which 
            // is always supposed to be there for a successful count GET request to the DAS CRUD service
            if(NullChecker.isNullOrEmpty(restResponseBodyStr) || !restResponseBodyStr.contains("count")) {
                throw new Exception("VAP request to DAS CRUD Service did not return count response! \n");
            } 
        } catch (Exception e) {
            pass = false;
            error = StackTraceUtil.getStackTraceUtilAsString(e);            
        }     
        
        // package it up into JSON and send it back to the caller
        request.setAttribute("results", translateToJSON("DAS CRUD", pass, error));

        this.forward(request, response, "resultsJSON");  
    }  

/*    
    private String makeRESTGETRequest(String requestURL, String acceptMimeType) throws Exception {
        
        String result = null;
        CloseableHttpClient httpClient = null;
        try {        
            // verify requestURL is a valid URL      
            URL url = new URL(requestURL);             

            // Set the timeout in milliseconds until a connection is established.
            int timeoutConnectionMS = 15000;
            // Set the default socket timeout (SO_TIMEOUT)
            // in milliseconds which is the timeout for waiting for data.
            int timeoutSocketMS = 35000;
            RequestConfig config = RequestConfig.custom().setConnectTimeout(timeoutConnectionMS).setSocketTimeout(timeoutSocketMS).build();

            httpClient = HttpClientBuilder.create().setDefaultRequestConfig(config).build();   

            HttpGet httpGet = new HttpGet(url.toString());
            httpGet.setHeader("Accept", acceptMimeType);

            HttpResponse clientResponse = httpClient.execute(httpGet);  

            result = EntityUtils.toString(clientResponse.getEntity());            
                        
            if(!NullChecker.isNullOrEmpty(clientResponse) && clientResponse.getStatusLine().getStatusCode() > 299) {
                // you've received an error response! 
                // throw something with the error response status and message in it?
                throw new Exception("REST GET Call returned HTTP Response code of: "+clientResponse.getStatusLine().getStatusCode()+" and response body: \n"+result);                
            } else {
                // do nothing
            }
        } catch (Exception e) {
            //System.out.println("Exception While Connecting: "+e.getMessage()+"\n");
            //e.printStackTrace();
            throw new Exception(e);
        } finally {
            try {
                if(!NullChecker.isNullOrEmpty(httpClient)) {
                    // shut down the connection
                    httpClient.close();
                }
            } catch (IOException ex) {
                LOG.error("makeRESTGETRequest", ex);
            }
        }
        return result;
    }   
*/

    
    private String makeRESTGETRequest(String requestURL, String acceptMimeType) throws Exception {
        
        String result = null;
        
        HttpURLConnection conn = null;   
        
        // Set the timeout in milliseconds until a connection is established.
        int timeoutConnectionMS = 15000;
        // Set the default socket timeout (SO_TIMEOUT)
        // in milliseconds which is the timeout for waiting for data.
        int timeoutSocketMS = 35000;
            
        //InputStream is = null;
        try {        
            // verify requestURL is a valid URL      
            URL rUrl = new URL(requestURL);        
            conn = (HttpURLConnection) rUrl.openConnection();
                        
            conn.setDoInput(true);

            conn.setConnectTimeout(timeoutConnectionMS);
            conn.setReadTimeout(timeoutSocketMS);
            
            conn.setRequestMethod("GET");
           
            conn.setRequestProperty("Accept", acceptMimeType);               
            
            conn.connect();
            
            try (InputStream is = conn.getInputStream();) {

                result = IOUtils.toString(is, StandardCharsets.UTF_8.name());

                if (!NullChecker.isNullOrEmpty(conn) && conn.getResponseCode() > 299) {
                    // you've received an error response! 
                    // throw something with the error response status and message in it?
                    throw new Exception("REST GET Call returned HTTP Response code of: " + conn.getResponseCode() + " and response body: \n" + result);
                } else {
                    // do nothing
                }
            } catch (IOException e) {
                LOG.error("makeRESTGETRequest", e);
            }
        } catch (Exception e) {
            throw new Exception(e);
        } finally {
            try {
//                if(!NullChecker.isNullOrEmpty(is)) { 
//                    is.close();
//                } 
                // shut down the connection
                if(!NullChecker.isNullOrEmpty(conn)) { 
                    conn.disconnect();                    
                }
            } catch (Exception ex) {
                LOG.error("makeRESTGETRequest", ex);
            }
        }
        return result;
    }           

    public void checkDQUploadTransmissionLogStatus(final HttpServletRequest request, final HttpServletResponse response) throws ServletException, IOException {
        Boolean pass = true;
        String error = "";        
        
        // do the call to the service
        try {
            // check the status by making a REST request to get JSON data from the service

            // Get dqTransmissionLogService.serviceLocation property in server.properties here
            final String DQUTRANSMLOG_SERVICE_URL = serverPropertyLookup.getProperty("dqTransmissionLogService.serviceLocation");             
            
            // get the date range parameter values
            DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd.HH:mm:ss");        
            
            Calendar todayCal = Calendar.getInstance();
            String runFinishStartDate = dateFormat.format(todayCal.getTime());
            
            // set the start date to an hour ago
            int currentHour = todayCal.get(Calendar.HOUR_OF_DAY);
            todayCal.set(Calendar.HOUR_OF_DAY, currentHour - 1);
            String runBeginStartDate = dateFormat.format(todayCal.getTime());            
            
            //runBeginStartDate={runBeginStartDate}&runFinishStartDate={runFinishStartDate}
            // with date format: yyyy-MM-dd.HH:mm:ss
            StringBuilder requestURLSB = new StringBuilder(DQUTRANSMLOG_SERVICE_URL).append("?runBeginStartDate=").append(runBeginStartDate).append("&runFinishStartDate=").append(runFinishStartDate);
            
            String requestURL = requestURLSB.toString();
            
            // GET a JSON response
            String restResponseBodyStr = this.makeRESTGETRequest(requestURL, "application/json");
            //System.out.println("restResponseBodyStr"+restResponseBodyStr);
            
            // if made it this far, then test for data array contents in response which 
            // is always supposed to be there for a successful GET request for data to the DQ Upload Transmission Log Service
            if(NullChecker.isNullOrEmpty(restResponseBodyStr) || !restResponseBodyStr.contains("data")) {
                throw new Exception("VAP request to DQ Upload Transmission Log Service did not return data response! \n");
            } 
        } catch (Exception e) {
            pass = false;
            error = StackTraceUtil.getStackTraceUtilAsString(e);            
        }     
        
        // package it up into JSON and send it back to the caller
        request.setAttribute("results", translateToJSON("TransmLog", pass, error));

        this.forward(request, response, "resultsJSON");
    }

    @EJB(beanInterface = PropertyLookupLocal.class, beanName = "WebPropertyFileLookup")
    public void setPropertyLookup(PropertyLookup propertyLookup)
    {
        this.propertyLookup = propertyLookup;
    } 

    @EJB(beanInterface = PropertyLookupLocal.class, beanName = "ServerPropertyFileLookup")
    public void setServerPropertyLookup(PropertyLookup serverPropertyLookup)
    {
        this.serverPropertyLookup = serverPropertyLookup;
    }  
    
    @EJB(beanInterface = AdapterAuditManager.class, mappedName = "AdapterAuditManager")    
    public void setAdapterAuditManager(final AdapterAuditManager adapterAuditManager) {
        this.adapterAuditManager = adapterAuditManager;
    } 
    
    @EJB(beanInterface = PdqService.class, mappedName = "PdqService")    
    public void setPdqService(final PdqService pdqService) {
        this.pdqService = pdqService;
    }
        
    @EJB(beanInterface = DirectAuditManager.class, mappedName = "DirectAuditManager")    
    public void setDirectAuditManager(final DirectAuditManager directAuditManager) {
        this.directAuditManager = directAuditManager;
    }   
    
    @EJB(beanInterface = FacilityResolver.class, mappedName = "FacilityResolver")    
    public void setFacilityResolver(final FacilityResolver facilityResolver) {
        this.facilityResolver = facilityResolver;
    }        
}