/*
 * The brains behind running the Advanced Scheduling tasks at second :00, every minute starting at minute :00, every hour between 02am and 06am, every day starting on the 1st, every month.
 */
package gov.va.nvap.web.util.xls;

import gov.va.med.nhin.adapter.audit.ActionType;
import gov.va.med.nhin.adapter.audit.ActionValuesType;
import gov.va.nvap.common.exception.StackTraceUtil;
import gov.va.nvap.common.validation.NullChecker;
import gov.va.nvap.service.adapter.audit.AdapterAuditManager;
import gov.va.nvap.service.adapter.audit.DirectAuditManager;
import gov.va.nvap.service.audit.ConsentAuditType;
import gov.va.nvap.service.dataquality.DqTransmissionLogManager;
import gov.va.nvap.svc.consentmgmt.stub.dao.DelayReasonDAO;
import gov.va.nvap.svc.consentmgmt.stub.dao.DelayedConsentReportDAO;
import gov.va.nvap.svc.consentmgmt.stub.dao.MailNotificationDAO;
import gov.va.nvap.svc.consentmgmt.stub.dao.PatientConsentDirDAO;
import gov.va.nvap.web.consent.audit.dao.AuditedConsentDAO;
import gov.va.nvap.web.dao.FacilityDAO;
import gov.va.nvap.web.dao.UserDocumentDAO;
import gov.va.nvap.web.dao.UserDocumentScheduleAuditDAO;
import gov.va.nvap.web.dao.UserDocumentScheduleDAO;
import gov.va.nvap.web.helper.document.DocumentHelper;
import gov.va.nvap.web.helper.facility.FacilityHelper;
import gov.va.nvap.web.helper.privacy.ConsentManagementHelper;
import gov.va.nvap.web.helper.report.DqTransmissionLogReportHelper;
import gov.va.nvap.web.helper.report.ReportHelper;
import gov.va.nvap.web.patient.ExcelGeneratorThread;
import gov.va.nvap.web.user.document.UserDocumentSchedule;
import gov.va.nvap.web.user.document.UserDocumentScheduleAudit;
import gov.va.nvap.web.util.date.DateUtil;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.ejb.EJB;
import org.springframework.beans.factory.annotation.Required;

/**
 *
 * @author Johann P. Sonnenberg
 */
public class AdvancedSchedulingRunner {
    
    private static final Logger logger = Logger.getLogger(AdvancedSchedulingRunner.class.getName());
    
    private UserDocumentScheduleDAO userDocumentScheduleDAO;
    private UserDocumentScheduleAuditDAO userDocumentScheduleAuditDAO;
    private UserDocumentDAO userDocumentDAO;
    private ReportHelper reportHelper;
    private FacilityHelper facilityHelper;
    private ExcelExporter excelExporter;
    private FacilityDAO facilityDAO;
    private AdapterAuditManager adapterAuditManager;
    private DirectAuditManager directAuditManager;
    private ConsentManagementHelper consentManagementHelper;
    private DocumentHelper documentHelper;
    private AuditedConsentDAO auditedConsentDAO;
    private PatientConsentDirDAO patientConsentDirDAO;
    private DelayReasonDAO delayReasonDAO;
    private DelayedConsentReportDAO delayedConsentReportDAO;
    private MailNotificationDAO mailNotificationDAO;
    private DqTransmissionLogManager dqTransmissionLogManager;
    private boolean isEnabled;
    
    
    @Required
    public void setUserDocumentScheduleDAO(final UserDocumentScheduleDAO userDocumentScheduleDAO) {
        this.userDocumentScheduleDAO = userDocumentScheduleDAO;
    }
    
    @Required
    public void setUserDocumentScheduleAuditDAO(final UserDocumentScheduleAuditDAO userDocumentScheduleAuditDAO) {
        this.userDocumentScheduleAuditDAO = userDocumentScheduleAuditDAO;
    }
    
    @Required
    public void setUserDocumentDAO(final UserDocumentDAO userDocumentDAO) {
        this.userDocumentDAO = userDocumentDAO;
    }
    
    @Required
    public void setReportHelper(final ReportHelper reportHelper) {
        this.reportHelper = reportHelper;
    }
    
    @Required
    public void setFacilityHelper(final FacilityHelper facilityHelper) {
        this.facilityHelper = facilityHelper;
    }
    
    @Required
    public void setExcelExporter(final ExcelExporter excelExporter) {
        this.excelExporter = excelExporter;
    }
    
    @Required
    public void setFacilityDAO(final FacilityDAO facilityDAO) {
        this.facilityDAO = facilityDAO;
    }
    
    @Required
    public void setAdapterAuditManager(final AdapterAuditManager adapterAuditManager) {
        this.adapterAuditManager = adapterAuditManager;
    }
    
    @Required
    public void setConsentManagementHelper( final ConsentManagementHelper consentManagementHelper) {
        this.consentManagementHelper = consentManagementHelper;
    }
    
    @Required
    public void setDocumentHelper( final DocumentHelper documentHelper) {
        this.documentHelper = documentHelper;
    }
    
    @Required
    public void setAuditedConsentDAO( final AuditedConsentDAO auditedConsentDAO) {
        this.auditedConsentDAO = auditedConsentDAO;
    }
    
    @Required
    public void setPatientConsentDirDAO( final PatientConsentDirDAO patientConsentDirDAO) {
        this.patientConsentDirDAO = patientConsentDirDAO;
    }
    
    @Required
    public void setDelayReasonDAO( final DelayReasonDAO delayReasonDAO) {
        this.delayReasonDAO = delayReasonDAO;
    }
    
    @Required
    public void setDelayedConsentReportDAO( final DelayedConsentReportDAO delayedConsentReportDAO) {
        this.delayedConsentReportDAO = delayedConsentReportDAO;
    }
    
    @Required
    public void setMailNotificationDAO( final MailNotificationDAO mailNotificationDAO) {
        this.mailNotificationDAO = mailNotificationDAO;
    }
    
    @Required
    public void setDqTransmissionLogManager( final DqTransmissionLogManager dqTransmissionLogManager) {
        this.dqTransmissionLogManager = dqTransmissionLogManager;
    }
    
    @Required
    public void setIsEnabled( final boolean isEnabled) {
        this.isEnabled = isEnabled;
    }
    
    @EJB(beanInterface = DirectAuditManager.class, mappedName = "DirectAuditManager")    
    public void setDirectAuditManager(final DirectAuditManager directAuditManager) {
        this.directAuditManager = directAuditManager;
    }    
    
    public void run() throws Exception {
        //first check if anything is currently running if not, proceed
        if(this.isEnabled) {
            //get the next task to run for today that hasn't been run already 
            UserDocumentSchedule nextToRun = this.userDocumentScheduleDAO.getNextScheduledTask();
            
            UserDocumentScheduleAudit audit = new UserDocumentScheduleAudit();
            
            if(nextToRun != null) {
                try{
                    //create an initial audit log record. set it to PROCESSING
                    audit.setScheduleId(nextToRun.getScheduleId());
                    audit.setFormat(nextToRun.getFormat());
                    audit.setRunStartDate(new Date());
                    audit.setStatus("PROCESSING");

                    this.userDocumentScheduleAuditDAO.create(audit);
                } catch(Exception e) {
                    //we couldn't add an audit record, so most likely another server is running it, log it and break outta here
                    logger.log(Level.INFO, "ADVANCED_SCHEDULED_TASK_CANCELED_PROCESSING " + "scheduleId: " + nextToRun.getScheduleId() + " " + Thread.currentThread().getId() + " " + System.currentTimeMillis());
                    
                    return;
                } 
                
                //log it
                logger.log(Level.INFO, "ADVANCED_SCHEDULED_TASK_BEGIN_PROCESSING " + "scheduleId: " + nextToRun.getScheduleId() + " " + Thread.currentThread().getId() + " " + System.currentTimeMillis());
                
                //set up "common" properties for the run
                ExcelGeneratorThread exGenThread = new ExcelGeneratorThread("exGenThread");
                exGenThread.setTitle(nextToRun.getReportType());
                exGenThread.setUserId(nextToRun.getUserId());
                
                //set up "common" helpers
                exGenThread.setReportHelper(this.reportHelper);
                exGenThread.setFacilityHelper(this.facilityHelper);
                exGenThread.setExcelExporter(this.excelExporter);
                exGenThread.setDocumentHelper(documentHelper);
                exGenThread.setDqtlReportHelper(new DqTransmissionLogReportHelper());
                
                //build up what object needs to be created per the reportType
                switch(nextToRun.getReportType()) {
                    case "Accounting of Disclosures Report" :                
                        this.setupAccountingOfDisclosuresReport(exGenThread, nextToRun);
                        break;
                    case "Received eHealth Exchange Documents Report" : 
                        this.setupReceivedEHealthExchangeDocumentsReport(exGenThread, nextToRun);
                        break;
                    case "Consent Directive Report" :
                        this.setupConsentDirectiveReport(exGenThread, nextToRun);
                        break;
                    case "Opt-In Patients Report" :
                        this.setupOptInPatientsReport(exGenThread, nextToRun);
                        break;
                    case "Expiring Consent Report" :
                        this.setupExpiringConsentReport(exGenThread, nextToRun);
                        break;
                    case "Delayed Consent Detail Report" :
                        this.setupDelayedConsentDetailReport(exGenThread, nextToRun);
                        break;
                    case "Patient Discovery Audit Report" :
                        this.setupPatientDiscoveryAuditReport(exGenThread, nextToRun);
                        break;
                    case "Data Quality Upload Transmission Log Report" :
                        this.setupDataQualityUploadTransmissionLogReport(exGenThread, nextToRun);
                        break;
                }
                
                //send it to the ExcelGeneratorThread
                try {
                    exGenThread.run();
                    
                    //once it returns, update the audit log and set it to COMPLETED
                    audit.setRunEndDate(new Date());
                    audit.setStatus("COMPLETED");

                    this.userDocumentScheduleAuditDAO.update(audit);
                } catch(Exception e) {
                    //once it returns, update the audit log and set it to FAILED
                    audit.setRunEndDate(new Date());
                    audit.setErrorMessage(StackTraceUtil.getStackTraceUtilAsString(e));
                    audit.setStatus("FAILED");

                    this.userDocumentScheduleAuditDAO.update(audit);
                }
                    
                //log it
                logger.log(Level.INFO, "ADVANCED_SCHEDULED_TASK_PROCESSING_COMPLETE " + "scheduleId: " + nextToRun.getScheduleId() + " " + Thread.currentThread().getId() + " " + System.currentTimeMillis());
            }
        }
    }
    
    //private helper methods (currently not used but here for future functionality if needed)
    /* private boolean isTaskCurrentlyRunning() {
        //does a check to see if there is a task currently running. we don't want two running at the same time...
        return this.userDocumentScheduleAuditDAO.isScheduledTaskCurrentlyRunning();
    }*/
    
    private Date generateStartDateForPeriodOfReporting(UserDocumentSchedule uds) {
        Date date = null;
        Calendar cal = Calendar.getInstance();
        
        switch(uds.getPeriodOfReporting()) {
            case "yesterday" :
                cal.add(Calendar.HOUR, -24);
                break;
            case "lastWeek" :
                cal.set(Calendar.DAY_OF_WEEK, cal.getFirstDayOfWeek());
                cal.add(Calendar.HOUR, -24 * 7);
                break;
            case "currentWeek" :
                cal.set(Calendar.DAY_OF_WEEK, cal.getFirstDayOfWeek());
                break;
            case "lastMonth" :
                cal.add(Calendar.MONTH, -1);
                cal.add(Calendar.DATE, 1);
                break;
            case "currentMonth" :
                cal.set(Calendar.DAY_OF_MONTH, 1);
                break;
            case "currentQuarter" :
                cal.set(Calendar.MONTH, cal.get(Calendar.MONTH)/3 * 3);
                cal.set(Calendar.DAY_OF_MONTH, 1);
                break;
        }
        
        //final tweaks to remove the time so start date starts at 12:00:00 AM
        date = cal.getTime();
        date = DateUtil.voidTime(date);
        
        return date;
    }
    
    private Date generateEndDateForPeriodOfReporting(UserDocumentSchedule uds) {
        Date date = null;
        Calendar cal = Calendar.getInstance();
        
        switch(uds.getPeriodOfReporting()) {
            case "yesterday" :
                cal.add(Calendar.HOUR, -24);
                break;
            case "lastWeek" :
                cal.set(Calendar.DAY_OF_WEEK, Calendar.SATURDAY);
                cal.add(Calendar.HOUR, -24 * 7);
                break;
            case "currentWeek" :
                cal.set(Calendar.DAY_OF_WEEK, Calendar.SATURDAY);
                break;
            case "lastMonth" :
                cal.add(Calendar.MONTH, -1);
                cal.add(Calendar.DAY_OF_MONTH, cal.getActualMaximum(Calendar.DAY_OF_MONTH));
                break;
            case "currentMonth" :
                cal.set(Calendar.DAY_OF_MONTH, cal.getActualMaximum(Calendar.DAY_OF_MONTH));
                break;
            case "currentQuarter" :
                cal.set(Calendar.MONTH, cal.get(Calendar.MONTH)/3 * 3 + 2);
                cal.set(Calendar.DAY_OF_MONTH, cal.getActualMaximum(Calendar.DAY_OF_MONTH));
                break;
        }
        
        //final tweaks to set the time for the end date so it's time is 11:59:59 PM
        date = cal.getTime();
        DateFormat dateFormat = new SimpleDateFormat("MM/dd/yyyy");
        date = ReportHelper.getEndDate(dateFormat.format(date));
        
        return date;
    }
    
    
    //setup methods for each report that can be scheduled
    private void setupAccountingOfDisclosuresReport(ExcelGeneratorThread egt, UserDocumentSchedule uds) throws Exception {
        //set up the required services/DAOs for this run
        egt.setAdapterAuditManager(this.adapterAuditManager);
        egt.setDirectAuditManager(this.directAuditManager);
        egt.setCmsHelper(this.consentManagementHelper);
        egt.setUserDocumentDao(this.userDocumentDAO);
        
        if(uds.getFormat().equals("Excel")) { 
            final Map<String, String> disclosureMap = new LinkedHashMap<String, String>();
            final Map<String, List<Object>> filterMap = new LinkedHashMap<String, List<Object>>();
            final LinkedHashMap<String, Object> filters = new LinkedHashMap<String, Object>();

            filters.put("Task Name", uds.getUserDefinedReportName());
            filters.put("Disclosure Source", uds.getReportParameterValue("source").toString());
            filters.put("SSN", uds.getReportParameterValue("ssn").toString());
            filters.put("Last Name", uds.getReportParameterValue("lastName").toString());
            filters.put("First Name", uds.getReportParameterValue("firstName").toString());
            filters.put("Start Date", this.reportHelper.getFormattedDate(this.generateStartDateForPeriodOfReporting(uds))); 
            filters.put("End Date", this.reportHelper.getFormattedDate(this.generateEndDateForPeriodOfReporting(uds)));
            filters.put("Purpose of Use", ExcelExporter.getFilterValue(uds.getReportParameterValue("purposeOfUse").toString()));
            if ("exchange".equals(uds.getReportParameterValue("source").toString())) {
                filters.put("Patient Preferred Facility", ExcelExporter.getPatientPreferredFacilityFilter(this.facilityDAO, uds.getReportParameterValue("facility").toString()));
                filters.put("eHealth Exchange Organization", ExcelExporter.getFilterValue(uds.getReportParameterValue("organizationName").toString()));
                
                ExcelExporter.populateFilterMapForExport(null, filters, filterMap, uds.getReportParameterValue("patientTypes").toString());
            } else {
                ExcelExporter.populateFilterMapForExport(null, filters, filterMap, "");
            }

            disclosureMap.put("ssn", "SSN");
            disclosureMap.put("lastName", "Patient Last Name");
            disclosureMap.put("firstName", "Patient First Name");
            disclosureMap.put("auditTimeFormatted", "Date of Disclosures (CT)");
            disclosureMap.put("documentTitle", "Disclosures");
            disclosureMap.put("patientFacilityName", "Patient Preferred Facility");
            disclosureMap.put("patientFacilityNumber", "Patient Preferred Facility Station ID");
            disclosureMap.put("organizationName", "eHealth Exchange Organization");
            disclosureMap.put("facilityOid", "OID");
            disclosureMap.put("userId", "User ID");
            disclosureMap.put("purposeForUse", "Purpose of Use");
            disclosureMap.put("roleName", "User Role");
            
            egt.setReportSource(uds.getReportParameterValue("source").toString());
            egt.setFilterMap(filterMap);
            egt.setReportMap(disclosureMap);
            
            // Set search attributes for getting audit results
            egt.setAttributes(mapAttributesForAccountingOfDisclosuresReport(uds));
        } else {
            //CSV
            Map<String, String> csvReportMap = new LinkedHashMap<>();
            
            csvReportMap.put("SSN", "ssn");
            csvReportMap.put("Patient Last Name", "lastName");
            csvReportMap.put("Patient First Name", "firstName");
            csvReportMap.put("Date of Disclosures (CT)", "auditTimeFormatted");
            csvReportMap.put("Disclosures", "documentTitle");
            csvReportMap.put("Patient Preferred Facility", "patientFacilityName");
            csvReportMap.put("Patient Preferred Facility Station ID", "patientFacilityNumber");
            csvReportMap.put("eHealth Exchange Organization", "organizationName");
            csvReportMap.put("OID", "facilityOid");
            csvReportMap.put("User ID", "userId");
            csvReportMap.put("Purpose of Use", "purposeForUse");
            csvReportMap.put("User Role", "roleName");
            
            egt.setDocumentType("csv");
            egt.setReportMap(csvReportMap);
            egt.setCsvExport(new CsvExporter());

            // Set search attributes for getting audit results
            egt.setAttributes(this.mapAttributesForAccountingOfDisclosuresReport(uds));
        }
        
    }
    
    private void setupReceivedEHealthExchangeDocumentsReport(ExcelGeneratorThread egt, UserDocumentSchedule uds) throws Exception {
        //set up the required services/DAOs for this run
        egt.setAdapterAuditManager(this.adapterAuditManager);
        egt.setUserDocumentDao(this.userDocumentDAO);
        
        if(uds.getFormat().equals("Excel")) {
        
            final Map<String, String> inboundDocumentMap = new LinkedHashMap<String, String>();
            final Map<String, List<Object>> filterMap = new LinkedHashMap<String, List<Object>>();
            final LinkedHashMap<String, Object> filters = new LinkedHashMap<String, Object>();

            //filters are NOT used in the query, they are just shown at the top of the Excel document
            filters.put("Task Name", uds.getUserDefinedReportName());
            filters.put("SSN", uds.getReportParameterValue("ssn").toString());
            filters.put("Last Name", uds.getReportParameterValue("lastName").toString());
            filters.put("First Name", uds.getReportParameterValue("firstName").toString());
            filters.put("User ID", uds.getReportParameterValue("userId").toString());
            filters.put("Start Date", this.reportHelper.getFormattedDate(this.generateStartDateForPeriodOfReporting(uds))); 
            filters.put("End Date", this.reportHelper.getFormattedDate(this.generateEndDateForPeriodOfReporting(uds)));
            filters.put("Purpose of Use", ExcelExporter.getFilterValue(uds.getReportParameterValue("purposeOfUse")));
            filters.put("Patient Preferred Facility", ExcelExporter.getPatientPreferredFacilityFilter(this.facilityDAO, uds.getReportParameterValue("facility").toString()));
            filters.put("eHealth Exchange Org", ExcelExporter.getFilterValue(uds.getReportParameterValue("remoteOrganization")));
            ExcelExporter.populateFilterMapForExport(null, filters, filterMap, uds.getReportParameterValue("patientTypes").toString()); 

            //maps the column heading to the actual return from the query so we know where to put the data
            inboundDocumentMap.put("ssn", "SSN");
            inboundDocumentMap.put("lastName", "Patient Last Name");
            inboundDocumentMap.put("firstName", "Patient First Name");
            inboundDocumentMap.put("auditTimeFormatted", "Date Received (CT)");
            inboundDocumentMap.put("documentTitle", "Document Title");
            inboundDocumentMap.put("patientFacilityName", "Patient Preferred Facility");
            inboundDocumentMap.put("patientFacilityNumber", "Patient Preferred Facility Station ID");
            inboundDocumentMap.put("remoteOrganizationName", "eHealth Exchange Organization");
            inboundDocumentMap.put("remoteFacilityOid", "OID");
            inboundDocumentMap.put("userId", "User ID");
            inboundDocumentMap.put("userFacilityCode", "User Facility Code");
            inboundDocumentMap.put("userFacilityName", "User Facility");
            inboundDocumentMap.put("userName", "User Name");
            inboundDocumentMap.put("purposeForUse", "Purpose of Use");
            inboundDocumentMap.put("roleName", "User Role");
            inboundDocumentMap.put("systemId", "System ID");
            
            egt.setFilterMap(filterMap);
            egt.setReportMap(inboundDocumentMap);

            // Set search attributes for getting audit results (actually used in the query)
            egt.setAttributes(mapAttributesForReceivedEHealthExchangeDocumentsReport(uds));
        } else {
            //CSV
            Map<String, String> csvReportMap = new LinkedHashMap<String, String>();
            
            csvReportMap.put("SSN", "ssn");
            csvReportMap.put("Patient Last Name", "lastName");
            csvReportMap.put("Patient First Name", "firstName");
            csvReportMap.put("Date Received (CT)", "auditTimeFormatted");
            csvReportMap.put("Document Title", "documentTitle");
            csvReportMap.put("Patient Preferred Facility", "patientFacilityName");
            csvReportMap.put("Patient Preferred Facility Station ID", "patientFacilityNumber");
            csvReportMap.put("eHealth Exchange Organization", "remoteOrganizationName");
            csvReportMap.put("OID", "remoteFacilityOid");
            csvReportMap.put("User ID", "userId");
            csvReportMap.put("User Facility Code", "userFacilityCode");
            csvReportMap.put("User Facility", "userFacilityName");
            csvReportMap.put("User Name", "userName");
            csvReportMap.put("Purpose of Use", "purposeForUse");
            csvReportMap.put("User Role", "roleName");
            csvReportMap.put("System ID", "systemId");
            
            egt.setDocumentType("csv");
            egt.setReportMap(csvReportMap);

            egt.setCsvExport(new CsvExporter());

            egt.setAttributes(mapAttributesForReceivedEHealthExchangeDocumentsReport(uds));
        }
    }
    
    private void setupConsentDirectiveReport(ExcelGeneratorThread egt, UserDocumentSchedule uds) throws Exception {
        //set up the required services/DAOs for this run
        egt.setAuditedConsentDAO(this.auditedConsentDAO);
        egt.setUserDocumentDao(this.userDocumentDAO);
        
        if(uds.getFormat().equals("Excel")) {
            final Map<String, String> reportMap = new LinkedHashMap<String, String>();

            // Generate filters.
            final Map<String, List<Object>> filterMap = new LinkedHashMap<String, List<Object>>();
            final LinkedHashMap<String, Object> filters = new LinkedHashMap<String, Object>();
            final String consentType = this.getConsentTypeFilterText(uds.getReportParameterValue("consentType").toString());
            
            filters.put("Task Name", uds.getUserDefinedReportName());
            filters.put("SSN", uds.getReportParameterValue("ssn").toString());
            filters.put("Last Name", uds.getReportParameterValue("lastName").toString());
            filters.put("First Name", uds.getReportParameterValue("firstName").toString());
            filters.put("Start Date", this.reportHelper.getFormattedDate(this.generateStartDateForPeriodOfReporting(uds))); 
            filters.put("End Date", this.reportHelper.getFormattedDate(this.generateEndDateForPeriodOfReporting(uds)));
            filters.put("Authenticating Facility", ExcelExporter.getFacilitiesFilter(this.facilityDAO, uds.getReportParameterValue("stationNumbers").toString()));
            filters.put("Consent Type", ExcelExporter.getFilterValue(consentType));
            if (consentType.contains("Revocation")) {
                filters.put("Inactivation Reason", ExcelExporter.getFilterValue(uds.getReportParameterValue("inactivationReason").toString()));
            }
            
            filters.put("Entered By",uds.getReportParameterValue("enteredBy").toString());
            
            ExcelExporter.populateFilterMapForExport(null, filters, filterMap, uds.getReportParameterValue("patientTypes").toString());
            
            reportMap.put("ssnMasked", "SSN");
            reportMap.put("lastName", "Patient Last Name");
            reportMap.put("firstName", "Patient First Name");
            reportMap.put("auditTimeFormatted", "Time of Event (CT)");
            reportMap.put("patientDateFormatted", "Patient Signature/Patient Deceased Date");
            reportMap.put("purposeOfUse", "Purpose of Use");
            reportMap.put("consentType", "Consent Type");
            reportMap.put("inactivationReason", "Inactivation Reason");
            reportMap.put("userId", "Entered By");
            reportMap.put("facilityName", "Authenticating Facility");
            reportMap.put("facilityId", "Authenticating Facility Station ID");
            reportMap.put("visnName", "VISN");
            
            egt.setFilterMap(filterMap);
            egt.setReportMap(reportMap);
            
            egt.setAttributes(mapAttributesForConsentDirectiveReport(uds));
        } else {
            //CSV
            Map<String, String> csvReportMap = new LinkedHashMap<String, String>();

            csvReportMap.put("SSN", "ssnMasked");
            csvReportMap.put("Patient Last Name", "lastName");
            csvReportMap.put("Patient First Name", "firstName");
            csvReportMap.put("Time of Event (CT)", "auditTimeFormatted");
            csvReportMap.put("Patient Signature/Patient Deceased Date", "patientDateFormatted");
            csvReportMap.put("Purpose of Use", "purposeOfUse");
            csvReportMap.put("Consent Type", "consentType");
            csvReportMap.put("Inactivation Reason", "inactivationReason");
            csvReportMap.put("Entered By", "userId");
            csvReportMap.put("Authenticating Facility", "facilityName");
            csvReportMap.put("Authenticating Facility Station ID", "facilityId");
            csvReportMap.put("VISN", "visnName");
            
            egt.setDocumentType("csv");
            egt.setCsvExport(new CsvExporter());
            egt.setReportMap(csvReportMap);
            
            egt.setAttributes(mapAttributesForConsentDirectiveReport(uds));
        }
    }
    
    private void setupOptInPatientsReport(ExcelGeneratorThread egt, UserDocumentSchedule uds) throws Exception {
        //set up the required services/DAOs for this run
        egt.setPatientConsentDirDAO(this.patientConsentDirDAO);
        egt.setUserDocumentDao(this.userDocumentDAO);
        
        if(uds.getFormat().equals("Excel")) {
            final Map<String, String> reportMap = new LinkedHashMap<String, String>();

            // Generate filters.
            final Map<String, List<Object>> filterMap = new LinkedHashMap<String, List<Object>>();
            final LinkedHashMap<String, Object> filters = new LinkedHashMap<String, Object>();
            
            filters.put("Task Name", uds.getUserDefinedReportName());
            filters.put("Start Date", this.reportHelper.getFormattedDate(this.generateStartDateForPeriodOfReporting(uds))); 
            filters.put("End Date", this.reportHelper.getFormattedDate(this.generateEndDateForPeriodOfReporting(uds)));
            filters.put("Entered By", uds.getReportParameterValue("enteredBy").toString());

            filters.put("Selection Type", uds.getReportParameterValue("dateType").toString());
            String facilityNames = ExcelExporter.getFacilitiesFilter(this.facilityDAO, uds.getReportParameterValue("stationNumbers").toString());
            filters.put("Authenticating Facility", facilityNames);
            filters.put("Consent Type", ExcelExporter.getConsentTypeFilter(uds.getReportParameterValue("consentTypeName").toString()));
            ExcelExporter.populateFilterMapForExport(null, filters, filterMap, uds.getReportParameterValue("patientTypes").toString());

            reportMap.put("ssnMasked", "SSN");
            reportMap.put("lastName", "Patient Last Name");
            reportMap.put("firstName", "Patient First Name");
            reportMap.put("middleName", "Patient Middle Name");
            reportMap.put("entryDate", "Entry Date");
            reportMap.put("optInDate", "Opt In Date");
            reportMap.put("expirationDate", "Expiration Date");
            reportMap.put("consentTypeName", "Consent Type");
            reportMap.put("userId", "Entered By");
            reportMap.put("facilityName", "Authenticating Facility");
            
            egt.setFilterMap(filterMap);
            egt.setReportMap(reportMap);

            // Set search attributes for getting audit results
            egt.setAttributes(this.mapAttributesForOptInPatientsReport(uds));
        } else {
            //CSV
            Map<String, String> csvReportMap = new LinkedHashMap<String, String>();

            csvReportMap.put("SSN", "ssnMasked");
            csvReportMap.put("Patient Last Name", "lastName");
            csvReportMap.put("Patient First Name", "firstName");
            csvReportMap.put("Patient Middle Name", "middleName");
            csvReportMap.put("Entry Date", "entryDate");
            csvReportMap.put("Opt In Date", "optInDate");
            csvReportMap.put("Expiration Date", "expirationDate");
            csvReportMap.put("Consent Type", "consentTypeName");
            csvReportMap.put("Entered By", "userId");
            csvReportMap.put("Authenticating Facility", "facilityName");
            
            egt.setDocumentType("csv");
            egt.setCsvExport(new CsvExporter());
            egt.setReportMap(csvReportMap);

            // Set search attributes for getting audit results
            egt.setAttributes(this.mapAttributesForOptInPatientsReport(uds));
        }
    }
    
    private void setupExpiringConsentReport(ExcelGeneratorThread egt, UserDocumentSchedule uds) throws Exception {
        //set up the required services/DAOs for this run
        egt.setPatientConsentDirDAO(this.patientConsentDirDAO);
        egt.setUserDocumentDao(this.userDocumentDAO);
        
        if(uds.getFormat().equals("Excel")) {
            final Map<String, String> reportMap = new LinkedHashMap<String, String>();

            // Generate filters.
            final Map<String, List<Object>> filterMap = new LinkedHashMap<String, List<Object>>();
            final LinkedHashMap<String, Object> filters = new LinkedHashMap<String, Object>();
            
            filters.put("Task Name", uds.getUserDefinedReportName());
            filters.put("Start Date", this.reportHelper.getFormattedDate(this.generateStartDateForPeriodOfReporting(uds))); 
            filters.put("End Date", this.reportHelper.getFormattedDate(this.generateEndDateForPeriodOfReporting(uds)));
            filters.put("Entered By", uds.getReportParameterValue("enteredBy").toString());

            String facilityNames = ExcelExporter.getFacilitiesFilter(this.facilityDAO, uds.getReportParameterValue("stationNumbers").toString());
            filters.put("Authenticating Facility", facilityNames);
            filters.put("Consent Type", ExcelExporter.getConsentTypeFilter(uds.getReportParameterValue("consentTypeName").toString()));
            ExcelExporter.populateFilterMapForExport(null, filters, filterMap, uds.getReportParameterValue("patientTypes").toString());

            reportMap.put("ssnMasked", "SSN");
            reportMap.put("lastName", "Patient Last Name");
            reportMap.put("firstName", "Patient First Name");
            reportMap.put("middleName", "Patient Middle Name");
            reportMap.put("optInDate", "Opt In Date");
            reportMap.put("expirationDate", "Expiration Date");
            reportMap.put("consentTypeName", "Consent Type");
            reportMap.put("userId", "Entered By");
            reportMap.put("facilityName", "Authenticating Facility");
            
            egt.setFilterMap(filterMap);
            egt.setReportMap(reportMap);

            // Set search attributes for getting audit results
            egt.setAttributes(this.mapAttributesForExpiringConsentReport(uds));
        } else {
            //CSV
            Map<String, String> csvReportMap = new LinkedHashMap<String, String>();

            csvReportMap.put("SSN", "ssnMasked");
            csvReportMap.put("Patient Last Name", "lastName");
            csvReportMap.put("Patient First Name", "firstName");
            csvReportMap.put("Patient Middle Name", "middleName");
            csvReportMap.put("Opt In Date", "optInDate");
            csvReportMap.put("Expiration Date", "expirationDate");
            csvReportMap.put("Consent Type", "consentTypeName");
            csvReportMap.put("Entered By", "userId");
            csvReportMap.put("Authenticating Facility", "facilityName");
            
            egt.setDocumentType("csv");
            egt.setCsvExport(new CsvExporter());
            egt.setReportMap(csvReportMap);

            // Set search attributes for getting audit results
            egt.setAttributes(this.mapAttributesForExpiringConsentReport(uds));
        }
    }
    
    private void setupDelayedConsentDetailReport(ExcelGeneratorThread egt, UserDocumentSchedule uds) throws Exception {
        //set up the required services/DAOs for this run
        egt.setDelayedConsentReportDAO(this.delayedConsentReportDAO);
        egt.setMailNotificationDAO(this.mailNotificationDAO);
        egt.setUserDocumentDao(this.userDocumentDAO);
        
        if(uds.getFormat().equals("Excel")) {
            // Generate column headers.
            final Map<String, String> columnHeaders = new LinkedHashMap<String, String>();
            columnHeaders.put("dateAdded", "Date Entered (CT)");
            columnHeaders.put("icn", "ICN");
            columnHeaders.put("ssn", "SSN");
            columnHeaders.put("lastName", "Last Name");
            columnHeaders.put("firstName", "First Name");
            columnHeaders.put("middleName", "Middle Name");
            columnHeaders.put("consentType", "Consent Type");
            columnHeaders.put("reasonsForDelay", "Reason(s) for Delay");
            columnHeaders.put("enteredBy", "Entered By");
            columnHeaders.put("authenticatingFacility", "Authenticating Facility");
            columnHeaders.put("mailNotifications", "Mailed Dates");

            // Generate filters.
            final Map<String, List<Object>> filterMap = new LinkedHashMap<String, List<Object>>();
            final LinkedHashMap<String, Object> filters = new LinkedHashMap<String, Object>();
            
            String daysSinceDelayed = uds.getReportParameterValue("daysSinceDelayed").toString();
            if (!NullChecker.isNullOrEmpty(daysSinceDelayed) && !"ALL".equals(daysSinceDelayed)) {
                if (daysSinceDelayed.equals("0")) {
                    daysSinceDelayed = "0-5";
                }
                else {
                    daysSinceDelayed = ">" + daysSinceDelayed;
                }
            }

            filters.put("Task Name", uds.getUserDefinedReportName());
            filters.put("SSN", uds.getReportParameterValue("ssn").toString());
            filters.put("Last Name", uds.getReportParameterValue("lastName").toString());
            filters.put("First Name", uds.getReportParameterValue("firstName").toString());
            filters.put("Authenticating Facility", ExcelExporter.getFacilitiesFilter(this.facilityDAO, uds.getReportParameterValue("stationNumbers").toString()));
            filters.put("Reason(s) for Delay", ExcelExporter.getReasonsForDelayFilter(this.delayReasonDAO, uds.getReportParameterValue("reasonsForDelay").toString()));
            filters.put("Days Since Delayed", daysSinceDelayed);
            filters.put("Consent Type", ExcelExporter.getConsentTypeFilter(uds.getReportParameterValue("consentType").toString()));
            filters.put("Entered By", ExcelExporter.getConsentTypeFilter(uds.getReportParameterValue("enteredBy").toString()));
            ExcelExporter.populateFilterMapForExport(null, filters, filterMap, uds.getReportParameterValue("patientTypes").toString());
            
            egt.setReportMap(columnHeaders);
            egt.setFilterMap(filterMap);

            // Set search attributes for getting audit results
            egt.setAttributes(this.mapAttributesForDelayedConsentDetailReport(uds));
        } else {
            //CSV
            Map<String, String> columnHeaders = new LinkedHashMap<String, String>();
            
            columnHeaders.put("Date Entered (CT)", "dateAdded");
            columnHeaders.put("ICN", "icn");
            columnHeaders.put("SSN", "ssn");
            columnHeaders.put("Last Name", "lastName");
            columnHeaders.put("First Name", "firstName");
            columnHeaders.put("Middle Name", "middleName");
            columnHeaders.put("Consent Type", "consentType");
            columnHeaders.put("Reason(s) for Delay", "reasonsForDelay");
            columnHeaders.put("Entered By", "enteredBy");
            columnHeaders.put("Authenticating Facility", "authenticatingFacility");
            columnHeaders.put("Mailed Dates", "mailNotifications");
            
            egt.setDocumentType("csv");
            egt.setCsvExport(new CsvExporter());
            egt.setReportMap(columnHeaders);
            
            // Set search attributes for getting audit results
            egt.setAttributes(this.mapAttributesForDelayedConsentDetailReport(uds));
        }
    }
    
    private void setupPatientDiscoveryAuditReport(ExcelGeneratorThread egt, UserDocumentSchedule uds) throws Exception {
        //set up the required services/DAOs for this run
        egt.setAdapterAuditManager(this.adapterAuditManager);
        egt.setUserDocumentDao(this.userDocumentDAO);
        
        if(uds.getFormat().equals("Excel")) {
            final Map<String, String> patientDiscoveryMap = new LinkedHashMap<String, String>();
            
            String patientMpiResPmpi = (uds.getReportParameterValue("mpiMatch").toString() != null) ? uds.getReportParameterValue("mpiMatch").toString() : "ALL";

            if (patientMpiResPmpi.equalsIgnoreCase("all")) {
                patientMpiResPmpi = null;
            }

            // Generate filters.
            final Map<String, List<Object>> filterMap = new LinkedHashMap<String, List<Object>>();
            final LinkedHashMap<String, Object> filters = new LinkedHashMap<String, Object>();

            //map of what to put in the excel sheet at the top showing what filters were used to generate the report
            filters.put("Task Name", uds.getUserDefinedReportName());
            filters.put("Start Date", this.reportHelper.getFormattedDate(this.generateStartDateForPeriodOfReporting(uds))); 
            filters.put("End Date", this.reportHelper.getFormattedDate(this.generateEndDateForPeriodOfReporting(uds)));
            filters.put("SSN", uds.getReportParameterValue("ssn").toString());
            filters.put("Last Name", uds.getReportParameterValue("lastName").toString());
            filters.put("First Name", uds.getReportParameterValue("firstName").toString());
            filters.put("User ID", uds.getReportParameterValue("userId").toString());
            filters.put("MPI Results", ExcelExporter.getFilterValue(patientMpiResPmpi));
            filters.put("Sending Organization", uds.getReportParameterValue("sendingOrganizationName").toString());
            filters.put("Receiving Organization", uds.getReportParameterValue("receivingOrganizationName").toString());
            ExcelExporter.populateFilterMapForExport(null, filters, filterMap, uds.getReportParameterValue("patientTypes").toString());

            //map used to put column headings for all query data returned
            patientDiscoveryMap.put("auditTimeFormatted", "Date Received (CT)");
            patientDiscoveryMap.put("ssn", "SSN");
            patientDiscoveryMap.put("lastName", "Patient Last Name");
            patientDiscoveryMap.put("firstName", "Patient First Name");
            patientDiscoveryMap.put("middleName", "Patient Middle Name");
            patientDiscoveryMap.put("organizationName", "Sending Organization");
            patientDiscoveryMap.put("facilityOid", "Sending Organization OID");
            patientDiscoveryMap.put("purposeForUse", "Purpose of Use");
            patientDiscoveryMap.put("remoteOrganizationName", "Receiving Organization");
            patientDiscoveryMap.put("remoteFacilityOid", "Receiving Organization OID");
            patientDiscoveryMap.put("message", "Message");
            patientDiscoveryMap.put("failureExplanation", "Explanation of Failure");
            patientDiscoveryMap.put("details", "Details");
            
            egt.setFilterMap(filterMap);
            egt.setReportMap(patientDiscoveryMap);
            egt.setHasOptionalRows(true);
            egt.setOptionalRowsMap(createOptionalRowsMapForPatientDiscoveryAuditReport(Integer.valueOf(uds.getReportParameterValue("patientTypes").toString())));
            
            // Set search attributes for getting audit results
            egt.setAttributes(this.mapAttributesForPatientDiscoveryAuditReport(uds));
        } else {
            //CSV
            Map<String, String> csvReportMap = new LinkedHashMap<>();
            
            csvReportMap.put("Date Received (CT)", "auditTimeFormatted");
            csvReportMap.put("SSN", "ssn");
            csvReportMap.put("Patient Last Name", "lastName");
            csvReportMap.put("Patient First Name", "firstName");
            csvReportMap.put("Patient Middle Name", "middleName");
            csvReportMap.put("Sending Organization", "organizationName");
            csvReportMap.put("Sending Organization OID", "facilityOid");
            csvReportMap.put("Purpose of Use", "purposeForUse");
            csvReportMap.put("Receiving Organization", "remoteOrganizationName");
            csvReportMap.put("Receiving Organization OID", "remoteFacilityOid");
            csvReportMap.put("Message", "message");
            csvReportMap.put("Explanation of Failure", "failureExplanation");
            csvReportMap.put("Details", "details");
            
            egt.setDocumentType("csv");
            egt.setCsvExport(new CsvExporter());
            egt.setReportMap(csvReportMap);

            // Set search attributes for getting audit results
            egt.setAttributes(this.mapAttributesForPatientDiscoveryAuditReport(uds));
        }
    }
    
    private void setupDataQualityUploadTransmissionLogReport(ExcelGeneratorThread egt, UserDocumentSchedule uds) throws Exception {
        //set up the required services/DAOs for this run
        egt.setDqTransmissionLogManager(this.dqTransmissionLogManager);
        egt.setUserDocumentDao(this.userDocumentDAO);
        
        if(uds.getFormat().equals("Excel")) {
            final Map<String, List<Object>> filterMap = new LinkedHashMap<String, List<Object>>();
            final LinkedHashMap<String, Object> filters = new LinkedHashMap<String, Object>();
            
            filters.put("Task Name", uds.getUserDefinedReportName());
            filters.put("Start Date", this.reportHelper.getFormattedDate(this.generateStartDateForPeriodOfReporting(uds))); 
            filters.put("End Date", this.reportHelper.getFormattedDate(this.generateEndDateForPeriodOfReporting(uds)));
            filters.put("Event Type", uds.getReportParameterValue("eventType").toString());
            filters.put("Status", uds.getReportParameterValue("status").toString());
            filters.put("File Name", uds.getReportParameterValue("fileName").toString());
            
            ExcelExporter.populateFilterMapForExport(null, filters, filterMap, "Data Quality Upload Transmission Log Report");
            
            final Map<String, String> inboundDocumentMap = new LinkedHashMap<String, String>();
            inboundDocumentMap.put("dqTransmissionId", "DQ Transmission Id");
            inboundDocumentMap.put("batchId", "Batch Id");
            inboundDocumentMap.put("eventType", "Event Type");
            inboundDocumentMap.put("startTimeFormatted", "Start Time (CT)");
            inboundDocumentMap.put("endTimeFormatted", "End Time (CT)");
            inboundDocumentMap.put("hostName", "Host Name");
            inboundDocumentMap.put("fileName", "File Name");
            inboundDocumentMap.put("fileType", "File Type");
            inboundDocumentMap.put("fileSize", "File Size");
            inboundDocumentMap.put("destination", "Destination");
            inboundDocumentMap.put("status", "Status");
            inboundDocumentMap.put("localDocId", "Local Doc Id");
            inboundDocumentMap.put("remoteDocId", "Remote Doc Id");
            inboundDocumentMap.put("error", "Error");
            
            egt.setFilterMap(filterMap);
            egt.setReportMap(inboundDocumentMap);
            
            // Set search attributes for getting log results
            egt.setAttributes(this.mapAttributesForDataQualityUploadTransmissionLogReport(uds));
        } else {
            //CSV
            // set the document column headers for the output
            Map<String, String> csvReportMap = new LinkedHashMap<String, String>();
            csvReportMap.put("DQ Transmission Id", "dqTransmissionId");
            csvReportMap.put("Batch Id", "batchId");
            csvReportMap.put("Event Type", "eventType");
            csvReportMap.put("Start Time (CT)", "startTimeFormatted");
            csvReportMap.put("End Time (CT)", "endTimeFormatted");
            csvReportMap.put("Host Name", "hostName");
            csvReportMap.put("File Name", "fileName");
            csvReportMap.put("File Type", "fileType");
            csvReportMap.put("File Size", "fileSize");
            csvReportMap.put("Destination", "destination");
            csvReportMap.put("Status", "status");
            csvReportMap.put("Local Doc Id", "localDocId");
            csvReportMap.put("Remote Doc Id", "remoteDocId");
            csvReportMap.put("Error", "error");      
            
            egt.setDocumentType("csv");
            egt.setReportMap(csvReportMap);

            // Set search attributes for getting audit results
            egt.setAttributes(this.mapAttributesForDataQualityUploadTransmissionLogReport(uds));
        }
    }
    
    //private methods for mapping of attributes for different exports
    
    private Map<String, Object> mapAttributesForReceivedEHealthExchangeDocumentsReport(UserDocumentSchedule uds) throws Exception {
        // Set search attributes for getting audit results (actually used in the query)
        Map<String, Object> attributes = new HashMap<String, Object>();

        attributes.put("ssn", uds.getReportParameterValue("ssn").toString());
        attributes.put("lastName", uds.getReportParameterValue("lastName").toString());
        attributes.put("firstName", uds.getReportParameterValue("firstName").toString());
        attributes.put("userId", uds.getReportParameterValue("userId").toString());
        attributes.put("startDate", this.generateStartDateForPeriodOfReporting(uds));
        attributes.put("endDate", this.generateEndDateForPeriodOfReporting(uds));
        attributes.put("facility", uds.getReportParameterValue("facility").toString());
        attributes.put("purposeOfUse", uds.getReportParameterValue("purposeOfUse").toString());
        attributes.put("remoteOrganization", uds.getReportParameterValue("remoteOrganization").toString());
        attributes.put("patientTypes", Integer.parseInt(uds.getReportParameterValue("patientTypes").toString()));

        // Set Action Types
        final ActionValuesType actionsType = new ActionValuesType();
        actionsType.getValue().add(ActionType.RETRIEVE_DOCUMENT_OUT);
        attributes.put("actionsType", actionsType);
        
        return attributes;
    }
    
    private Map<String, Object> mapAttributesForAccountingOfDisclosuresReport(UserDocumentSchedule uds) throws Exception {
        // Set search attributes for getting results (actually used in the query)
        Map<String, Object> attributes = new HashMap<String, Object>();
        
        attributes.put("ssn", uds.getReportParameterValue("ssn").toString());
        attributes.put("lastName", uds.getReportParameterValue("lastName").toString());
        attributes.put("firstName", uds.getReportParameterValue("firstName").toString());

        attributes.put("startDate", this.generateStartDateForPeriodOfReporting(uds));
        attributes.put("endDate", this.generateEndDateForPeriodOfReporting(uds));

        //what it was originally and no idea where this is set attributes.put("organization", (String) session.getAttribute("documentDisclosureQueryOrganization"));
        attributes.put("organization", "eHealth Exchange Organization");
        attributes.put("facility", uds.getReportParameterValue("facility").toString());
        attributes.put("purposeOfUse", uds.getReportParameterValue("purposeOfUse").toString());
        if(uds.getReportParameterValue("source").toString().equals("exchange")) {
            attributes.put("patientTypes", Integer.parseInt(uds.getReportParameterValue("patientTypes").toString()));
        } else {
            attributes.put("patientTypes", 1); //not used but we have to default it so it doesn't error on export
        }
            
        // Set Action Types
        final ActionValuesType actionsType = new ActionValuesType();
        actionsType.getValue().add(ActionType.RETRIEVE_DOCUMENT);
        actionsType.setNotIn(false);
        attributes.put("actionsType", actionsType);
        
        return attributes;
    }
    
    private Map<String, Object> mapAttributesForConsentDirectiveReport(UserDocumentSchedule uds) throws Exception {
        // Set search attributes for getting results (actually used in the query)
        Map<String, Object> attributes = new HashMap<String, Object>();
        
        attributes.put("ssn", uds.getReportParameterValue("ssn").toString());
        attributes.put("lastName", uds.getReportParameterValue("lastName").toString());
        attributes.put("firstName", uds.getReportParameterValue("firstName").toString());
        attributes.put("userId", uds.getReportParameterValue("enteredBy").toString());
        attributes.put("startDate", this.generateStartDateForPeriodOfReporting(uds));
        attributes.put("endDate", this.generateEndDateForPeriodOfReporting(uds));

        attributes.put("stationNumbers", uds.getReportParameterValue("stationNumbers").toString());
        attributes.put("patientTypes", Integer.parseInt(uds.getReportParameterValue("patientTypes").toString()));
        attributes.put("consentType", uds.getReportParameterValue("consentType").toString());
        attributes.put("inactivationReason", uds.getReportParameterValue("inactivationReason").toString());
        attributes.put("includeUnknownVisn", "true".equals(uds.getReportParameterValue("includeUnknownVisn").toString()));
        
        // Set Action Types
        List<String> actionTypes = new ArrayList<String>();
        actionTypes.add(ConsentAuditType.OPTIN_AUDIT_TYPE.value());
        actionTypes.add(ConsentAuditType.OPTOUT_AUDIT_TYPE.value());
        attributes.put("actionTypes", actionTypes);
        
        return attributes;
    }
    
    private Map<String, Object> mapAttributesForOptInPatientsReport(UserDocumentSchedule uds) throws Exception {
        // Set search attributes for getting results (actually used in the query)
        Map<String, Object> attributes = new HashMap<String, Object>();
        
        attributes.put("userId", uds.getReportParameterValue("enteredBy").toString());
        attributes.put("isOptInReport", true);
        
        attributes.put("startDate", this.generateStartDateForPeriodOfReporting(uds));
        attributes.put("endDate", this.generateEndDateForPeriodOfReporting(uds));
        attributes.put("dateType", uds.getReportParameterValue("dateType").toString());
        attributes.put("stationNumbers", uds.getReportParameterValue("stationNumbers").toString());
        attributes.put("patientTypes", Integer.parseInt(uds.getReportParameterValue("patientTypes").toString()));
        attributes.put("consentTypeName", uds.getReportParameterValue("consentTypeName").toString());
        attributes.put("includeUnknownVisn", "true".equals(uds.getReportParameterValue("includeUnknownVisn").toString()));
        
        return attributes;
    }
    
    private Map<String, Object> mapAttributesForExpiringConsentReport(UserDocumentSchedule uds) throws Exception {
        // Set search attributes for getting results (actually used in the query)
        Map<String, Object> attributes = new HashMap<String, Object>();
        
        attributes.put("userId", uds.getReportParameterValue("enteredBy").toString());
        attributes.put("isOptInReport", false);
        
        attributes.put("startDate", this.generateStartDateForPeriodOfReporting(uds));
        attributes.put("endDate", this.generateEndDateForPeriodOfReporting(uds));
        attributes.put("stationNumbers", uds.getReportParameterValue("stationNumbers").toString());
        attributes.put("patientTypes", Integer.parseInt(uds.getReportParameterValue("patientTypes").toString()));
        attributes.put("consentTypeName", uds.getReportParameterValue("consentTypeName").toString());
        attributes.put("includeUnknownVisn", "true".equals(uds.getReportParameterValue("includeUnknownVisn").toString()));
        
        return attributes;
    }
    
    private Map<String, Object> mapAttributesForDelayedConsentDetailReport(UserDocumentSchedule uds) throws Exception {
        // Set search attributes for getting results (actually used in the query)
        Map<String, Object> attributes = new HashMap<String, Object>();
        
        int patientType = NullChecker.isNullOrEmpty(uds.getReportParameterValue("patientTypes").toString()) ? 1 : Integer.parseInt(uds.getReportParameterValue("patientTypes").toString());

        // Get parameters for the search from the request.
        attributes.put("ssn", uds.getReportParameterValue("ssn").toString());
        attributes.put("lastName", uds.getReportParameterValue("lastName").toString());
        attributes.put("firstName", uds.getReportParameterValue("firstName").toString());
        attributes.put("startDate", this.generateStartDateForPeriodOfReporting(uds));
        attributes.put("endDate", this.generateEndDateForPeriodOfReporting(uds));
        attributes.put("userId", uds.getReportParameterValue("enteredBy").toString());

        attributes.put("stationNumbers", uds.getReportParameterValue("stationNumbers").toString());
        attributes.put("patientTypes", patientType);
        attributes.put("consentType", uds.getReportParameterValue("consentType").toString());
        attributes.put("reasonsForDelay", uds.getReportParameterValue("reasonsForDelay").toString());
        attributes.put("daysSinceDelayed", uds.getReportParameterValue("daysSinceDelayed").toString());
        attributes.put("includeUnknownVisn", "true".equals(uds.getReportParameterValue("includeUnknownVisn").toString()));
        
        return attributes;
    }
    
    private Map<String, Object> mapAttributesForPatientDiscoveryAuditReport(UserDocumentSchedule uds) throws Exception {
        // Set search attributes for getting results (actually used in the query)
        Map<String, Object> attributes = new HashMap<String, Object>();
        
        String mpiMatch = (uds.getReportParameterValue("mpiMatch").toString()!=null) ? uds.getReportParameterValue("mpiMatch").toString() : "ALL";
        final String sendingOrganizationName = uds.getReportParameterValue("sendingOrganizationName").toString();
        final String receivingOrganizationName = uds.getReportParameterValue("receivingOrganizationName").toString();
        final String sendingOrganization = uds.getReportParameterValue("sendingOrganization").toString().equals("") ? null : uds.getReportParameterValue("sendingOrganization").toString();
        final String receivingOrganization = uds.getReportParameterValue("receivingOrganization").toString().equals("") ? null : uds.getReportParameterValue("receivingOrganization").toString();
        
        if (mpiMatch.equalsIgnoreCase("all")) {
            mpiMatch = null;
        }
        
        //not using this as it seems not applicable anymore for this report attributes.put("icn", (String) session.getAttribute("patientDiscoveryQueryICN"));
        attributes.put("ssn", uds.getReportParameterValue("ssn").toString());
        attributes.put("lastName", uds.getReportParameterValue("lastName").toString());
        attributes.put("firstName", uds.getReportParameterValue("firstName").toString());
        attributes.put("userId", uds.getReportParameterValue("enteredBy").toString());

        attributes.put("startDate", this.generateStartDateForPeriodOfReporting(uds));
        attributes.put("endDate", this.generateEndDateForPeriodOfReporting(uds));

        //now that we have the added ability to search on sending/receiving organizations, we need to split them out here
        attributes.put("organization", sendingOrganization);
        attributes.put("remoteOrganization", receivingOrganization);
        attributes.put("organizationName", sendingOrganizationName);
        attributes.put("remoteOrganizationName", receivingOrganizationName);
        
        attributes.put("mpiMatch", mpiMatch);
        attributes.put("patientTypes", Integer.parseInt(uds.getReportParameterValue("patientTypes").toString()));

        // Set Action Types
        final ActionValuesType actionsType = new ActionValuesType();
        actionsType.getValue().add(ActionType.ANNOUNCE);
        actionsType.getValue().add(ActionType.CHECK_POLICY);
        actionsType.getValue().add(ActionType.MPI_FIND_MATCH);
        actionsType.getValue().add(ActionType.ADD_PATIENT_CORRELATION);
        actionsType.setNotIn(false);
        attributes.put("actionsType", actionsType);
        
        return attributes;
    }
    
    private Map<String, Object> mapAttributesForDataQualityUploadTransmissionLogReport(UserDocumentSchedule uds) throws Exception {
        // Set search attributes for getting results (actually used in the query)
        Map<String, Object> attributes = new HashMap<String, Object>();
        
        attributes.put("startDate", this.generateStartDateForPeriodOfReporting(uds));
        attributes.put("endDate", this.generateEndDateForPeriodOfReporting(uds));
        attributes.put("eventType", uds.getReportParameterValue("eventType").toString());
        attributes.put("status", uds.getReportParameterValue("status").toString());
        attributes.put("filename", uds.getReportParameterValue("fileName").toString());  
        
        return attributes;
    }
    
    private Map<String, String> createOptionalRowsMapForPatientDiscoveryAuditReport(int patientTypes) {
        Map<String, String> optionalRowsMap = new LinkedHashMap<String, String>();

        if (patientTypes == 1) {
            optionalRowsMap.put("uniqueRealPatients", "Unique patients");
            optionalRowsMap.put("realPatientMessages", "Total messages");
            optionalRowsMap.put("realPatientFails", "Match failures");
            optionalRowsMap.put("realPatientMatches", "Matches found");
        } else if (patientTypes == 2) {
            optionalRowsMap.put("uniqueTestPatients", "Unique patients");
            optionalRowsMap.put("testPatientMessages", "Total messages");
            optionalRowsMap.put("testPatientFails", "Match failures");
            optionalRowsMap.put("testPatientMatches", "Matches found");
        } else {
            optionalRowsMap.put("uniqueRealPatients", "Unique real patients");
            optionalRowsMap.put("realPatientMessages", "Total messages for real patients");
            optionalRowsMap.put("realPatientFails", "Match failures for real patients");
            optionalRowsMap.put("realPatientMatches", "Matches found for real patients");

            optionalRowsMap.put("", "");

            optionalRowsMap.put("uniqueTestPatients", "Unique test patients");
            optionalRowsMap.put("testPatientMessages", "Total messages for test patients");
            optionalRowsMap.put("testPatientFails", "Match failures for test patients");
            optionalRowsMap.put("testPatientMatches", "Matches found for test patients");
        }

        return optionalRowsMap;
    }
    
    private String getConsentTypeFilterText(String consentType) {
        if (NullChecker.isNullOrEmpty(consentType)) {
            return "ALL";
        } else if (consentType.equalsIgnoreCase("allAuthorizations")) {
            return "ALL Authorizations";
        } else if (consentType.equalsIgnoreCase("allRevocations")) {
             return "ALL Revocations";
        } else if (consentType.equalsIgnoreCase("NwHINAuthorizations")) {
             return "eHealth Exchange Authorizations";
        } else if (consentType.equalsIgnoreCase("SSAAuthorizations")) {
            return "SSA Authorizations";
        } else if (consentType.equalsIgnoreCase("NwHINRevocations")) {
            return "eHealth Exchange Revocations";
        } else if (consentType.equalsIgnoreCase("SSARevocations")) {
            return "SSA Revocations";
        } else if (consentType.equalsIgnoreCase("NwHINRestrictions")) {
            return "eHealth Exchange Restrictions";
        } else if (consentType.equalsIgnoreCase("NwHINRestrictionRevocations")) {
            return "eHealth Exchange Restriction Revocations";
        } else {
            return "";
        }
    }
}