package gov.va.nvap.web.util.xls;

import gov.va.nvap.common.validation.NullChecker;
import gov.va.nvap.web.helper.report.ReportHelper;
import gov.va.nvap.web.report.ReportDataProcessor;
import org.apache.commons.lang.StringEscapeUtils;
import org.identityconnectors.common.security.GuardedString;
import java.io.IOException;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.LinkedHashMap;
import java.util.Date;
import java.util.List;
import java.util.Map;
import javax.servlet.http.HttpServletResponse;

/**
 *
 * @author Zachary Tubb
 */
public class CsvExporter {

	Date date;
	SimpleDateFormat df = new SimpleDateFormat("yyyyMMdd_HHmmss");
	String formattedDate;

	public void exportToCSV(final HttpServletResponse response, String reportType, List<Map<String, Object>> results,
			LinkedHashMap<String, String> reportMap) throws IOException {

		OutputStream outputStream = null;

		date = new Date();
		formattedDate = df.format(date);

		response.setContentType("text/csv");
		response.setHeader("Content-Disposition",
				"attachment; filename=\"" + reportType + "_" + formattedDate + ".csv" + "\"");

		try {
			StringBuilder sbOutput;

			outputStream = response.getOutputStream();

			if (NullChecker.isNullOrEmpty(results)) {
				sbOutput = new StringBuilder();
				sbOutput.append("No records were found.");
			} else {
				//SimpleDateFormat dt = new SimpleDateFormat("MM/dd/yyyy");
				// Allocate string builder based on the number of rows, but no
				// larger than 20MB.
				if (results.size() > 100000) {
					sbOutput = new StringBuilder(20000000);
				} else {
					sbOutput = new StringBuilder(results.size() * 200);
				}

				// Append column headers
				for (String heading : reportMap.keySet()) {
					sbOutput.append(heading);
					sbOutput.append(",");
				}
				sbOutput.append("\n");

				// Append report data
				for (Map<String, Object> row : results) {
					// Create the row of data
					String separator = "";
					for (String dataField : reportMap.values()) {
						sbOutput.append(separator);
						separator = ",";
						// Include escape character.
						sbOutput.append("\"");
						GuardedString gs = null;

						if (dataField.toLowerCase().contains("oid")
								&& !NullChecker.isNullOrEmpty(row.get(dataField))) {
							gs = ReportDataProcessor.processDataField(ReportHelper.trimOrgOid( (String) row.get(dataField)));
						}else{
							gs = ReportDataProcessor.processDataField(row.get(dataField));
						}

						sbOutput.append(ReportDataProcessor.decryptToString(gs));
						// Include closing escape character.
						sbOutput.append("\"");
					}
					sbOutput.append("\n");
				}
			}


			byte[] b = (sbOutput.toString()).getBytes();
			outputStream.write(b);

			// Blank out this string builder so nothing is in memory (i.e. ssn,
			// other PII). Even though sbOutput is scoped to this method
			// and will be destroyed and set to be garbage collected.
			sbOutput.setLength(0);
		} catch (Exception e) {
			throw new IOException("Error exporting to CSV");
		} finally {
			if (outputStream != null) {
				outputStream.flush();
				outputStream.close();
			}
		}
	}



//	public void exportToCSV(final HttpServletResponse response, String reportType, List<Map<String, Object>> results,
//			LinkedHashMap<String, String> reportMap) throws IOException {
//
//		OutputStream outputStream = null;
//
//		date = new Date();
//		formattedDate = df.format(date);
//
//		response.setContentType("text/csv");
//		response.setHeader("Content-Disposition",
//				"attachment; filename=\"" + reportType + "_" + formattedDate + ".csv" + "\"");
//	try {
//
//		char[] sbOutput = new char[0];
//
//		// outputStream = new OutputStream();
//		// o.write(s.getBytes("UTF-8"));
//		if (NullChecker.isNullOrEmpty(results)) {
//			sbOutput = new char[0]; // new StringBuilder();
//			sbOutput = append(sbOutput, "No records were found.".toCharArray());// sbOutput.append("No
//																				// records
//																				// were
//																				// found.");
//			// System.out.print("No records were found.");
//		} else {
//			SimpleDateFormat dt = new SimpleDateFormat("MM/dd/yyyy");
//			// Allocate string builder based on the number of rows, but no
//			// larger than 20MB.
//			// if (results.size() > 100000) {
//			// sbOutput = append(sbOutput, heading.toCharArray());//sbOutput
//			// = new StringBuilder(20000000);
//			// } else {
//			// sbOutput = append(sbOutput, heading.toCharArray());//sbOutput
//			// = new StringBuilder(results.size() * 200);
//			// }
//
//			// Append column headers
//			for (String heading : reportMap.keySet()) {
//				sbOutput = append(sbOutput, heading.toCharArray());// sbOutput.append(heading);
//				// System.out.print(heading);
//				sbOutput = append(sbOutput, ",".toCharArray());// sbOutput.append(",");
//				// System.out.print(",");
//			}
//			sbOutput = append(sbOutput, "\\n".toCharArray());// sbOutput.append("\n");
//
//			// Append report data
//			for (Map<String, Object> row : results) {
//				// Create the row of data
//				String separator = "";
//				for (String dataField : reportMap.values()) {
//					sbOutput = append(sbOutput, separator.toCharArray()); // sbOutput.append(separator);
//					separator = ",";
//					// Include escape character.
//					sbOutput = append(sbOutput, "\"".toCharArray());// sbOutput.append("\"");
//					Object rowData = row.get(dataField);
//					if (NullChecker.isNullOrEmpty(row.get(dataField))) {
//						sbOutput = append(sbOutput, "".toCharArray());// sbOutput.append("");
//					} else {
//						if (dataField.contains("ssn")) {
//							Ssn ssn = (Ssn) rowData;
//							append(sbOutput,ssn.getSsn());
//						}else if (dataField.contains("Date")) {
//							// Format dates. patientDateFormatted is an
//							// exception - comes pre-formatted
//							if (dataField.equals("patientDateFormatted")) {
//								sbOutput = append(sbOutput, String.valueOf(rowData).toCharArray());// sbOutput.append(rowData);
//							} else {
//								Date oldDate = (Date) rowData;
//								sbOutput = append(sbOutput, String.valueOf(dt.format(oldDate)).toCharArray());//// sbOutput.append(dt.format(oldDate));
//							}
//						} else if (dataField.toLowerCase().contains("oid") && !NullChecker.isNullOrEmpty(rowData)) {
//							sbOutput = append(sbOutput, ReportHelper.trimOrgOid(String.valueOf(rowData)).toCharArray());
//						} else {
//							// Else print the data as-is.
//							sbOutput = append(sbOutput, String.valueOf(rowData).toCharArray());// sbOutput.append(rowData);
//						}
//					}
//					// Include closing escape character.
//					sbOutput = append(sbOutput, "\"".toCharArray());// sbOutput.append("\"");
//
//				}
//				sbOutput = append(sbOutput, "\\n".toCharArray());// sbOutput.append("");
//				
//			}
//
//		}
//
//		    String safeOutput = StringEscapeUtils.escapeXml(String.valueOf(sbOutput));
//			outputStream.write(safeOutput.getBytes());
//
//			// Blank out this string builder so nothing is in memory (i.e. ssn,
//			// other PII). Even though sbOutput is scoped to this method
//			// and will be destroyed and set to be garbage collected.
////			sbOutput.setLength(0);
//			Arrays.fill(sbOutput,'0');
//
//		} catch (Exception e) {
//			throw new IOException("Error exporting to CSV");
//		} finally {
//			if (outputStream != null) {
//				outputStream.flush();
//				outputStream.close();
//			}
//		}
//	}
//
//public char[] append(char[] current, char[] toAppend) {
//
//	int currentChars = current.length;
//	int charsToAppend = toAppend.length;
//	int size = current.length + toAppend.length;
//
//	char[] results = new char[size];
//
//	for (int i = 0; i <= current.length - 1; i++) {
//		results[i] = current[i];
//	}
//
//	for (int i = 0; i <= toAppend.length - 1; i++) {
//		results[i + current.length] = toAppend[i];
//	}
//
//	return results;
//}

}
