<%-- 
    Document   : Inclusion of this tag will add the Advanced Scheduling to a page. 

                 To add a button for scheduling use this code (the id of the button and data-target attribute MATTERS): 
                 <button name="btnScheduleExport" id="btnScheduleExport" class="btn btn-default" data-toggle="modal" data-target="#scheduleExportModal">Schedule</button>

                 A div for success/error messaging needs to be added to the page as well like so (the id of the div MATTERS):
                 <div id="userScheduledExportModalMessage" class="alert alert-dismissible" role="alert" style="display: none;">
                     <button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>
                 </div>

                 You will also need to add JavaScript to add the parameters and view parameters to the be passed to the modal when adding a task (again the id of the button MATTERS). Here is an example:
                $('#btnScheduleExport').on('click', function(){
                    //set up array of items to view at the top of the schedule export modal
                    var viewParameters = [ 
                        {name : 'Disclosure Source', value : $('#source').val()}, 
                        {name : 'Patient Preferred Facility', value : $("#facility option:selected").text()}, 
                        {name : 'Organization', value : $("#organization option:selected").text()}, 
                        {name : 'Patient Types', value : $("#patientTypes option:selected").text()} ];

                    //set up JSON object to pass as JSON in the schedule export modal form
                    var parameters = { 
                        'source' : $('#source').val(), 
                        'facility' : $('#facility').val(), 
                        'organization' : $('#organization').val(), 
                        'patientTypes' : $('#patientTypes').val() 
                    };

                    VAP.common.advancedSchedulingPopulateScheduleParameters(viewParameters, parameters, 'Disclosures Summary Report');
                });

    Created on : Mar 15, 2018, 9:48:39 AM
    Author     : Johann P. Sonnenberg
--%>

<%@taglib tagdir="/WEB-INF/tags/date" prefix="date"%>
<%@taglib tagdir="/WEB-INF/tags/formatting" prefix="f" %>
<%@taglib uri="http://java.sun.com/jsp/jstl/core" prefix="c"%>

<%@tag description="Inclusion of this tag will add the Advanced Scheduling to a page. See more information in the tag itself." pageEncoding="UTF-8"%>

<script language="JavaScript">
    $(document).ready(function () { 
        $('#scheduledEndDate').datepicker({
            theme: 'vap',
            inputFormat: 'MM/dd/yyyy',
            outputFormat: 'MM/dd/yyyy',
            gainFocusOnConstruction: false,
            min: '<c:out value="${tomorrow}" />',
            max: '<c:out value="${sixMonthsFromToday}" />'
        });

        $('.frequency').on('change', function(){
            var detail = $(this).data('detail'); 

            $('#frequencyWeekly').hide();
            $('#frequencyMonthly').hide();

            $('#frequency' + detail).show();
        });

        $('#scheduleExportModal').on('hidden.bs.modal', function (e) {
            //reset the form
            $('#frequencyWeekly').hide();
            $('#frequencyMonthly').hide();

            $('#frmScheduleExport')[0].reset();
        });

        $('#scheduleExportModal').on('show.bs.modal', function (e) {
            //kind of hacky but because of how the radio buttons are rendered, needed to make the scheduled end date input smaller
            $('#scheduledEndDate').parent().addClass('col-lg-3');

            $('#scheduleExportModalFormErrorMessage').hide();
            
            //make a call out to get advanced scheduling task info (cap, user's current count, if they can add more)
            var getAdvancedScheduleTaskCount = VAP.common.advancedSchedulingGetUserTaskInfo();
            
            //promise to the ajax call to get the count of the tasks a user has set up. this is done asynchronously so we needed a promise to wait until its done getting the info then we can display "stuff"
            $.when(getAdvancedScheduleTaskCount).done(function() {        
                //remove all alert-x classes from scheduledExportModalCapMessage
                $('#scheduledExportModalCapMessage').hide();            
                $('#scheduledExportModalCapMessage').removeClass('alert-danger');
                $('#scheduledExportModalCapMessage').removeClass('alert-warning');

                if(VAP.common.advancedScheduledTaskUserCanAddMore) {    
                    $('#scheduledExportModalCapMessage').html('Currently you have created ' + VAP.common.advancedScheduledTaskUserCurrentTaskCount + ' advanced scheduled task(s) of the ' + VAP.common.advancedScheduledTaskCap + ' permitted.');
                } else {
                    $('#scheduledExportModalCapMessage').html('Currently you have created ' + VAP.common.advancedScheduledTaskUserCurrentTaskCount + ' advanved scheduled task(s) of the ' + VAP.common.advancedScheduledTaskCap + ' permitted. You cannot add more at this time. To add another task, you need to remove one first by navigating to the Welcome menu and then Advanced Scheduling.');
                }

                $('#scheduledExportModalCapMessage').addClass('alert-warning');
                $('#scheduledExportModalCapMessage').show();
            });
        });

        $('#scheduleExportModal').on('shown.bs.modal', function (e) {
            $('#frmScheduleExport').validator('update');
        });

        $('.validateFrequencyEndDate').on('blur change', function(e) {
            VAP.common.advancedSchedulingValidateFrequency();
            VAP.common.advancedSchedulingValidateEndDate();
        });

        $('#confirmScheduleExportModalExport').on('click', function(e) {
            //remove any class on the userScheduledExportModalMessage div
            $('#userScheduledExportModalMessage').removeClass('alert-danger');
            $('#userScheduledExportModalMessage').removeClass('alert-success');

            //logic here for the frequency and endDate radios. have to do it custom here because bootstrap validator is having issues with the radios because of how they are rendered
            VAP.common.advancedSchedulingValidateFrequency();
            VAP.common.advancedSchedulingValidateEndDate();
            
            //make a call out to get advanced scheduling task info (cap, user's current count, if they can add more)
            var getAdvancedScheduleTaskCount = VAP.common.advancedSchedulingGetUserTaskInfo();
            
            //validate the form
            $.when(getAdvancedScheduleTaskCount).done(function() {
                if(!VAP.common.advancedScheduledTaskUserCanAddMore) {
                    //uh...no, this user can't add any more advanced scheduled tasks because they have hit the limit already
                    $('#scheduleExportModalFormErrorMessage').html('You cannot add any more advanced scheduled tasks at this time as you have already created the maximum of ' + VAP.common.advancedScheduledTaskCap + '.');
                    $('#scheduleExportModalFormErrorMessage').show();
                } else if ($('#frmScheduleExport').validator('validate').has('.has-error').length) {
                    // handle the invalid form...
                    $('#scheduleExportModalFormErrorMessage').html('All fields are required. Please fix the errors below then click Submit again.');
                    $('#scheduleExportModalFormErrorMessage').show();
                } else {
                    //submit the form via ajax
                    $.ajax({
                        url: "ManageScheduledExports.do_sec", 
                        headers: { 'skipLoadingAlert' : 'true' },
                        data: {
                            addUserScheduledExportAttempt : true, 
                            scheduledExportParametersJSON : $('#scheduledExportParametersJSON').val(),
                            scheduledExportViewParametersJSON : $('#scheduledExportViewParametersJSON').val(), 
                            reportType : $('#reportType').val(),
                            userDefinedReportName : $('#userDefinedReportName').val(),
                            periodOfReporting : $('#periodOfReporting').val(),
                            format : $('input[name=format]:checked').val(),
                            frequency : $('input[name=frequency]:checked').val(),
                            weeklyRecurrences : $('#weeklyRecurrences').val(),
                            frequencyWeeklyDayOfWeek : $('#frequencyWeeklyDayOfWeek').val(),
                            monthlyOccurrences : $('#monthlyOccurrences').val(),
                            monthlyOccurrenceInMonth : $('#monthlyOccurrenceInMonth').val(),
                            endDateSelection : $('input[name=endDateSelection]:checked').val(),
                            endAfterOccurrences : $('#endAfterOccurrences').val(),
                            scheduledEndDate : $('#scheduledEndDate').val()
                        },
                        success: function (result) {
                            var info = $.parseJSON(result);

                            if(info.success) {
                                $('#userScheduledExportModalMessage').addClass('alert-success');
                            } else {
                                $('#userScheduledExportModalMessage').addClass('alert-danger');
                            }

                            $('#userScheduledExportModalMessage').html(info.message);
                            $('#userScheduledExportModalMessage').show().delay(5000).slideUp();
                        }, 
                        error: function(result) {
                           $('#userScheduledExportModalMessage').html('There was an error creating an advanced scheduled task. Please try again later.');
                           $('#userScheduledExportModalMessage').addClass('alert-danger');
                           $('#userScheduledExportModalMessage').show().delay(5000).slideUp();
                        }
                    });

                    $('#scheduleExportModal').modal('hide');
                }
            });
        });
    });
</script>
<div class="modal fade" id="scheduleExportModal" role="dialog" data-backdrop="static" aria-labelledby="scheduleExportModalLabel">
    <div class="vertical-alignment-helper">
        <div class="modal-dialog vertical-align-center" role="document">
            <div class="modal-content" tabindex="-1">
                <div class="modal-header">
                    <h4 class="modal-title" id="scheduleExportModalLabel"><!-- filled in via JavaScript so it's dynamic per page/report type --></h4>
                </div>
                <div class="modal-body" aria-live="assertive" role="alert">
                    <div class="row">
                        <div class="col-xs-12 col-sm-12 col-md-12 col-lg-12">
                            <div class="alert alert-info">
                                The Veterans Authorization and Preferences (VAP) application will run any reports scheduled for export, in advance, 
                                starting at 2:00AM CST. It is important to note exact run times may vary based on the number of jobs in the queue. 
                                Once the job is completed, the report will appear in your scheduled export list. 
                            </div>
                            <div id="scheduledExportModalCapMessage" class="alert" role="alert" style="display: none;">
                                <!-- populated on show of the modal by JavaScript above -->
                            </div>
                            <div id="scheduleExportModalFormErrorMessage" class="alert alert-danger" role="alert" style="display: none;">
                                <!-- populated on submit of the form by JavaScript above -->
                            </div>
                            <form id="frmScheduleExport" name="frmScheduleExport" class="form-horizontal" data-toggle="validator" role="form">
                                <input type="hidden" id="scheduledExportParametersJSON" />
                                <input type="hidden" id="scheduledExportViewParametersJSON" />
                                <input type="hidden" id="reportType" />

                                <h4>Parameters</h4>
                                <div id="scheduledExportParametersView" class="col-xs-offset-1 col-sm-offset-1 col-md-offset-1 col-lg-offset-1">
                                    <!-- populated by JavaScript to contain all the viewable parameters --->
                                </div>
                                <h4>Schedule Options</h4>
                                <div class="form-group">
                                    <label for="userDefinedReportName" class="col-xs-3 col-sm-3 col-md-3 col-lg-3 control-label"><f:requiredField altText="Task Name" labelText="Task Name" useNoWrap="false" /></label>
                                    <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6">
                                        <input type="text" name="userDefinedReportName" id="userDefinedReportName" maxlength="255" class="form-control js-simple-tooltip" data-simpletooltip-text="<c:out value="${descriptors['advancedSchedulingReportName']}"/>" value="" pattern="^[-\w ]+$" required />
                                    </div>
                                </div>
                                <div class="form-group">
                                    <label for="periodOfReporting" class="col-xs-3 col-sm-3 col-md-3 col-lg-3 control-label"><f:requiredField altText="Reporting Period" labelText="Reporting Period" useNoWrap="false" /></label>
                                    <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6">
                                        <select id="periodOfReporting" name="periodOfReporting" class="form-control js-simple-tooltip" data-simpletooltip-text="<c:out value="${descriptors['advancedSchedulingReportingPeriod']}"/>" required>
                                            <option value="yesterday">Yesterday</option>
                                            <option value="lastWeek">Last Week</option>
                                            <option value="currentWeek">Current Week</option>
                                            <option value="lastMonth">Last Month</option>
                                            <option value="currentMonth">Current Month</option>
                                            <option value="currentQuarter">Current Quarter</option>
                                        </select>
                                    </div>
                                </div>
                                <div class="form-group">
                                    <fieldset>
                                        <legend class="col-xs-3 col-sm-3 col-md-3 col-lg-3 vap-input-group-legend pad-top-3" id="formatLegend"><f:requiredField altText="Format" labelText="Format" useNoWrap="false" /></legend>
                                        <div class="col-xs-4 col-sm-4 col-md-4 col-lg-4">
                                            <div class="radio">
                                                <label>
                                                    <input type="radio" name="format" value="Excel" checked="checked" aria-label="Format, Excel" required /> Excel 
                                                </label>
                                            </div>
                                            <div class="radio">
                                                <label>
                                                    <input type="radio" name="format" value="CSV" aria-label="Format, CSV" required /> CSV
                                                </label>
                                            </div>
                                        </div>
                                    </fieldset>
                                </div>
                                <div id="divFrequency" class="form-group">
                                    <fieldset>
                                        <legend class="col-xs-3 col-sm-3 col-md-3 col-lg-3 vap-input-group-legend pad-top-3"><f:requiredField altText="Frequency" labelText="Frequency" useNoWrap="false" /></legend>
                                        <div class="col-xs-9 col-sm-9 col-md-9 col-lg-9">
                                            <div class="radio">
                                                <label>
                                                    <input type="radio" name="frequency" value="daily" checked="checked" class="frequency validateFrequencyEndDate" data-detail="Daily" aria-label="Frequency, Daily" required /> Daily 
                                                </label>
                                            </div>
                                            <div class="radio">
                                                <label>
                                                    <input type="radio" id="frequencyWeeklyRadio" name="frequency" value="weekly" class="frequency validateFrequencyEndDate" data-detail="Weekly" aria-label="Frequency, Weekly" required /> Weekly
                                                </label>
                                                <div id="frequencyWeekly" class="form-inline pad-left-20" style="display: none;">
                                                    Recur every <input type="text" id="weeklyRecurrences" name="weeklyRecurrences" class="form-control validateFrequencyEndDate js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="<c:out value="${descriptors['advancedSchedulingFrequencyWeeklyWeeks']}"/>" style="width: 30px;" maxlength="2" aria-label="Recur every X weeks" /> week(s) on
                                                    <date:dayOfWeekPicker prefixId="frequencyWeekly" prefixName="frequencyWeekly" labelPrefix="Weekly Frequency" useNumericDayOfWeekValue="true" descriptor="${descriptors['advancedSchedulingFrequencyWeeklyDayOfWeek']}" tooltipPrefix="secondary" />
                                                </div>
                                            </div>
                                            <div class="radio">
                                                <label>
                                                    <input type="radio" name="frequency" value="monthly" class="frequency validateFrequencyEndDate" data-detail="Monthly" aria-label="Frequency, Monthly" required /> Monthly
                                                </label>
                                                <div id="frequencyMonthly" class="form-inline pad-left-20" style="display: none;">
                                                    Recur every <input type="text" id="monthlyOccurrences" name="monthlyOccurrences" class="form-control validateFrequencyEndDate js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="<c:out value="${descriptors['advancedSchedulingFrequencyMonthlyMonths']}"/>" style="width: 30px;" maxlength="2" aria-label="Recur every X months" /> month(s) on the 
                                                    <select id="monthlyOccurrenceInMonth" name="monthlyOccurrenceInMonth" class="form-control js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="<c:out value="${descriptors['advancedSchedulingFrequencyMonthlyDayOfMonth']}"/>" aria-label="Day of month to recur on">
                                                        <option value="first">1st day of the month</option>
                                                        <option value="fifteenth">15th of the month</option>
                                                        <option value="last">Last day of the month</option>
                                                    </select>
                                                </div>
                                            </div>
                                        </div>
                                    </fieldset>
                                </div>

                                <div id="divEndDate" class="form-group">
                                    <fieldset>
                                        <legend id="endDateLegend" class="col-xs-3 col-sm-3 col-md-3 col-lg-3 vap-input-group-legend pad-top-5"><f:requiredField altText="End Date" labelText="End Date" useNoWrap="false" /></legend>
                                        <div class="col-xs-5 col-sm-5 col-md-5 col-lg-5">
                                            <div id="divEndDateEndAfter" class="radio form-inline">
                                                <label>
                                                    <input type="radio" name="endDateSelection" value="endAfter" checked="checked" class="validateFrequencyEndDate" aria-label="End Date, End After" required /> End after 
                                                </label>
                                                <input type="text" id="endAfterOccurrences" class="form-control validateFrequencyEndDate js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="<c:out value="${descriptors['advancedSchedulingEndDateAfterOccurrences']}"/>" style="width: 30px;" maxlength="2" aria-label="End after X occurrences" /> <span id="spanEndDateOccurrences">occurrence(s)</span>
                                            </div>
                                            <div id="divEndDateEndBy" class="radio form-inline">
                                                <label>
                                                    <input type="radio" name="endDateSelection" value="endBy" class="validateFrequencyEndDate" aria-label="End Date, End By" required /> End by 
                                                </label>
                                                <label class="control-label vap-visuallyhidden" for="scheduledEndDate" aria-hidden="true" style="margin-right: -12px;">End by</label>
                                                <input type="text" name="scheduledEndDate" id="scheduledEndDate" class="form-control validateFrequencyEndDate js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="<c:out value="${descriptors['advancedSchedulingEndBy']}"/>" aria-label="Scheduled End Date" />
                                            </div>
                                            <%--<div class="radio">
                                                <label>
                                                    <input type="radio" name="endDateSelection" value="noEnd" class="validateFrequencyEndDate" /> No end date 
                                                </label>
                                            </div>--%>
                                        </div>
                                    </fieldset>
                                </div>
                            </form>
                        </div>
                    </div>
                </div>
                <div class="modal-footer">
                    <button type="button" class="btn btn-default" data-dismiss="modal" title="Cancel" aria-label="Cancel">Cancel</button>
                    <button type="button" id="confirmScheduleExportModalExport" class="btn btn-primary" title="Save" aria-label="Save">Save</button>
                </div>
            </div>
        </div>
    </div>
</div>