<%-- 
    Document   : edit page for advanced scheduling tasks
    Created on : Feb 19, 2018, 11:09:07 AM
    Author     : Johann P. Sonnenberg
--%>

<%@taglib uri="http://java.sun.com/jsp/jstl/core" prefix="c" %>
<%@taglib tagdir="/WEB-INF/tags/formatting" prefix="f" %>
<%@taglib uri="http://java.sun.com/jsp/jstl/functions" prefix="fn" %>
<%@taglib uri="http://java.sun.com/jsp/jstl/fmt" prefix="fmt"%>
<%@taglib tagdir="/WEB-INF/tags/date" prefix="date"%>

<f:mainTemplate headerText="Advanced Scheduling - Edit">
    <script language="JavaScript">
        $(document).ready(function(){
            $('#scheduledEndDate').datepicker({
                theme: 'vap',
                inputFormat: 'MM/dd/yyyy',
                outputFormat: 'MM/dd/yyyy',
                gainFocusOnConstruction: false,
                min: '<c:out value="${tomorrow}" />'
            });
            
            //determine what to show for frequency on page load
            $("input[name=frequency][value=<c:out value="${fn:toLowerCase(userDocumentSchedule.frequency)}" />]").prop('checked', true);
            
            if('<c:out value="${fn:toLowerCase(userDocumentSchedule.frequency)}" />' == 'weekly' ) {
                $('#frequencyWeekly').show();
            }
            
            if('<c:out value="${fn:toLowerCase(userDocumentSchedule.frequency)}" />' == 'monthly' ) {
                $('#frequencyMonthly').show();
            }
            
            //end date if there is NO end date
            if('<c:out value="${userDocumentSchedule.endDate}"/>'.length == 0) {
                $('[name="endDateSelection"]').removeAttr('checked');
                $("input[name=endDateSelection][value=noEnd]").prop('checked', true);
            } else {
                //we have an end date
                $("input[name=endDateSelection][value=endBy]").prop('checked', true);
            }
            
            //populate the JSON and the view of the parameters
            VAP.common.advancedSchedulingPopulateScheduleParameters(<c:out value="${userDocumentSchedule.reportViewParameters}" escapeXml="false" />, <c:out value="${userDocumentSchedule.reportParameters}" escapeXml="false" />, '<c:out value="${userDocumentSchedule.reportType}" />');
            
            $('.frequency').on('change', function(){
                var detail = $(this).data('detail'); 
                
                $('#frequencyWeekly').hide();
                $('#frequencyMonthly').hide();
                
                $('#frequency' + detail).show();
            });
            
            $('.validateFrequencyEndDate').on('blur change', function(e) {
                VAP.common.advancedSchedulingValidateFrequency();
                VAP.common.advancedSchedulingValidateEndDate();
            });
            
            //handle the save
            $('#btnSave').on('click', function(e) {
                $('#scheduleExportModalFormErrorMessage').hide();
                
                //logic here for the frequency and endDate radios. have to do it custom here because bootstrap validator is having issues with the radios because of how they are rendered
                VAP.common.advancedSchedulingValidateFrequency();
                VAP.common.advancedSchedulingValidateEndDate();
                
                //validate the form
                if ($('#frmEdit').validator('validate').has('.has-error').length) {
                    // handle the invalid form...
                    $('#scheduleExportModalFormErrorMessage').show();
                } else {
                    $('#frmEdit').submit();
                }
            });
        });
    </script>
    
    <div class="row">
        <div class="col-xs-12 col-sm-12 col-md-12 col-lg-12">
            <div id="scheduleExportModalFormErrorMessage" class="alert alert-danger" role="alert" style="display: none;">
                All fields are required. Please fix the errors below then click Submit again.
            </div>
            <form name="frmEdit" id="frmEdit" action="ManageScheduledExports.do_sec?editUserScheduledExportAttempt=true" method="post" class="form-horizontal" data-toggle="validator" role="form">
                <input type="hidden" name="scheduleId" id="scheduleId" value="<c:out value="${userDocumentSchedule.scheduleId}"/>" />
                <input type="hidden" id="scheduledExportParametersJSON" name="scheduledExportParametersJSON" />
                <input type="hidden" id="scheduledExportViewParametersJSON" name="scheduledExportViewParametersJSON" />
                <input type="hidden" id="reportType" name="reportType" />
                
                <h4>Parameters</h4>
                <div id="scheduledExportParametersView" class="col-xs-offset-1 col-sm-offset-1 col-md-offset-1 col-lg-offset-1">
                    <!-- filled in by JavaScript -->
                </div>
                <h4>Schedule Options</h4>
                <div class="form-group">
                    <label for="userDefinedReportName" class="col-xs-1 col-sm-1 col-md-1 col-lg-1 control-label"><f:requiredField altText="Task Name" labelText="Task Name" /></label>
                    <div class="col-xs-4 col-sm-4 col-md-4 col-lg-4">
                        <input type="text" name="userDefinedReportName" id="userDefinedReportName" maxlength="255" value="<c:out value="${userDocumentSchedule.userDefinedReportName}" />" class="form-control js-simple-tooltip" data-simpletooltip-text="<c:out value="${descriptors['advancedSchedulingReportName']}"/>" required pattern="^[-\w ]+$" />
                    </div>
                </div>
                <div class="form-group">
                    <label for="periodOfReporting" class="col-xs-1 col-sm-1 col-md-1 col-lg-1 control-label"><f:requiredField altText="Reporting Period" labelText="Reporting Period" useNoWrap="false" /></label>
                    <div class="col-xs-4 col-sm-4 col-md-4 col-lg-4">
                        <select id="periodOfReporting" name="periodOfReporting" class="form-control js-simple-tooltip" data-simpletooltip-text="<c:out value="${descriptors['advancedSchedulingReportingPeriod']}"/>" required>
                            <option value="yesterday" <c:if test="${userDocumentSchedule.periodOfReporting == 'yesterday'}">selected="selected"</c:if> >Yesterday</option>
                            <option value="lastWeek" <c:if test="${userDocumentSchedule.periodOfReporting == 'lastWeek'}">selected="selected"</c:if>>Last Week</option>
                            <option value="currentWeek" <c:if test="${userDocumentSchedule.periodOfReporting == 'currentWeek'}">selected="selected"</c:if>>Current Week</option>
                            <option value="lastMonth" <c:if test="${userDocumentSchedule.periodOfReporting == 'lastMonth'}">selected="selected"</c:if>>Last Month</option>
                            <option value="currentMonth" <c:if test="${userDocumentSchedule.periodOfReporting == 'currentMonth'}">selected="selected"</c:if>>Current Month</option>
                            <option value="currentQuarter" <c:if test="${userDocumentSchedule.periodOfReporting == 'currentQuarter'}">selected="selected"</c:if>>Current Quarter</option>
                        </select>
                    </div>
                </div>
                <div class="form-group">
                    <fieldset>
                        <legend class="col-xs-1 col-sm-1 col-md-1 col-lg-1 vap-input-group-legend pad-top-3"><f:requiredField altText="Format" labelText="Format" useNoWrap="false" /></legend>
                        <div class="col-xs-4 col-sm-4 col-md-4 col-lg-4">
                            <div class="radio">
                                <label>
                                    <input type="radio" name="format" value="Excel" <c:if test="${userDocumentSchedule.format == 'Excel'}">checked="checked"</c:if> aria-label="Format, Excel" required /> Excel 
                                </label>
                            </div>
                            <div class="radio">
                                <label>
                                    <input type="radio" name="format" value="CSV" <c:if test="${userDocumentSchedule.format == 'CSV'}">checked="checked"</c:if> aria-label="Format, CSV" required /> CSV
                                </label>
                            </div>
                        </div>
                    </fieldset>
                </div>
                <div id="divFrequency" class="form-group">
                    <fieldset>
                        <legend class="col-xs-1 col-sm-1 col-md-1 col-lg-1 vap-input-group-legend pad-top-3"><f:requiredField altText="Frequency" labelText="Frequency" useNoWrap="false" /></legend>
                        <div class="col-xs-4 col-sm-4 col-md-4 col-lg-4">
                            <div class="radio">
                                <label>
                                    <input type="radio" name="frequency" id="frequencyDailyRadio" value="daily" class="frequency validateFrequencyEndDate" data-detail="Daily" aria-label="Frequency, Daily" required /> Daily 
                                </label>
                            </div>
                            <div class="radio">
                                <label>
                                    <input type="radio" id="frequencyWeeklyRadio" name="frequency" value="weekly" class="frequency validateFrequencyEndDate" data-detail="Weekly" aria-label="Frequency, Weekly" required /> Weekly
                                </label>
                                <div id="frequencyWeekly" class="form-inline pad-left-20" style="display: none;">
                                    Recur every <input type="text" id="weeklyRecurrences" name="weeklyRecurrences" class="form-control validateFrequencyEndDate js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="<c:out value="${descriptors['advancedSchedulingFrequencyWeeklyWeeks']}"/>" style="width: 30px;" maxlength="2" value="<c:out value="${userDocumentSchedule.weeklyRecurrenceWeek}" />" /> week(s) on
                                    <date:dayOfWeekPicker prefixId="frequencyWeekly" prefixName="frequencyWeekly" labelPrefix="Weekly Frequency" useNumericDayOfWeekValue="true" selected="${userDocumentSchedule.weeklyRecurrenceDayOfWeek}" descriptor="${descriptors['advancedSchedulingFrequencyWeeklyDayOfWeek']}" tooltipPrefix="secondary" />
                                </div>
                            </div>
                            <div class="radio">
                                <label>
                                    <input type="radio" name="frequency" id="frequencyMonthlyRadio" value="monthly" class="frequency validateFrequencyEndDate" data-detail="Monthly" aria-label="Frequency, Monthly" required /> Monthly
                                </label>
                                <div id="frequencyMonthly" class="form-inline pad-left-20" style="display: none;">
                                    Recur every <input type="text" id="monthlyOccurrences" name="monthlyOccurrences" class="form-control validateFrequencyEndDate js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="<c:out value="${descriptors['advancedSchedulingFrequencyMonthlyMonths']}"/>" style="width: 30px;" maxlength="2" value="<c:out value="${userDocumentSchedule.monthlyRecurrenceMonth}" />" /> month(s) on the 
                                    <select id="monthlyOccurrenceInMonth" name="monthlyOccurrenceInMonth" class="form-control js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="<c:out value="${descriptors['advancedSchedulingFrequencyMonthlyDayOfMonth']}"/>">
                                        <option value="first" <c:if test="${userDocumentSchedule.monthlyRecurrenceDay == 'first'}">selected="selected"</c:if> >1st day of the month</option>
                                        <option value="fifteenth" <c:if test="${userDocumentSchedule.monthlyRecurrenceDay == 'fifteenth'}">selected="selected"</c:if> >15th of the month</option>
                                        <option value="last" <c:if test="${userDocumentSchedule.monthlyRecurrenceDay == 'last'}">selected="selected"</c:if> >Last day of the month</option>
                                    </select>
                                </div>
                            </div>
                        </div>
                    </fieldset>
                </div>
                <div id="divEndDate" class="form-group">
                    <fieldset>
                        <legend id="endDateLegend" class="col-xs-1 col-sm-1 col-md-1 col-lg-1 vap-input-group-legend pad-top-5"><f:requiredField altText="End Date" labelText="End Date" useNoWrap="false" /></legend>
                        <div class="col-xs-8 col-sm-8 col-md-8 col-lg-8">
                            <div id="divEndDateEndAfter" class="radio form-inline">
                                <label>
                                    <input type="radio" name="endDateSelection" value="endAfter" class="validateFrequencyEndDate" aria-label="End Date, End After" required /> End after 
                                </label>
                                <input type="text" id="endAfterOccurrences" name="endAfterOccurrences" class="form-control validateFrequencyEndDate js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="<c:out value="${descriptors['advancedSchedulingEndDateAfterOccurrences']}"/>" style="width: 30px;" maxlength="2" /> <span id="spanEndDateOccurrences">occurrence(s)</span>
                            </div>
                            <div id="divEndDateEndBy" class="radio form-inline">
                                <label>
                                    <input type="radio" name="endDateSelection" value="endBy" class="validateFrequencyEndDate" aria-label="End Date, End By" required /> End by 
                                </label>
                                <label class="control-label" for="scheduledEndDate" aria-hidden="true" style="margin-right: -12px;"></label>
                                <input type="text" name="scheduledEndDate" id="scheduledEndDate" value="<c:if test="${!empty userDocumentSchedule.endDate}"><fmt:formatDate pattern="MM/dd/yyyy" value="${userDocumentSchedule.endDate}" /></c:if>" class="form-control validateFrequencyEndDate js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="<c:out value="${descriptors['advancedSchedulingEndBy']}"/>" />
                            </div>
                            <%--<div class="radio">
                                <label>
                                    <input type="radio" name="endDateSelection" value="noEnd" class="validateFrequencyEndDate" /> No end date 
                                </label>
                            </div>--%>
                        </div>
                    </fieldset>
                </div>
            </form>
            <div class="form-group">
                <div class="col-xs-5 col-sm-5 col-md-5 col-lg-5 text-right pad-bottom-20">
                    <a href="ManageScheduledExports.do_sec?show=true" class="btn btn-default" title="Cancel" aria-label="Cancel">Cancel</a>
                    <button name="btnSave" id="btnSave" class="btn btn-primary" title="Save" aria-label="Save">Save</button>
                </div>
            </div>
        </div>
    </div>
</f:mainTemplate>
