<%--
    Document   : Patient Discovery Report
    Created By : David Vasquez / Robert Snelling
    Modified By: Asha Amritraj
--%>

<%@page import="java.util.*"%>
<%@page import="java.text.SimpleDateFormat"%>
<%@taglib tagdir="/WEB-INF/tags/formatting" prefix="f" %>
<%@taglib uri="http://java.sun.com/jsp/jstl/core" prefix="c"%>
<%@taglib tagdir="/WEB-INF/tags/date" prefix="date"%>
<%@taglib tagdir="/WEB-INF/tags/cpp" prefix="cpp"%>

<f:mainTemplate headerText="Manage Batches">
    <script language="JavaScript">
        $(document).ready(function () {
            //initial data table setup and call for the first recordset
            VAP.common.setProcessingForDataTable('tblReport', 'divReportPanel');
            
            VAP.common.reportTable = $('#tblReport').DataTable({
                deferLoading: 0,
                destroy: true,
                searching: false,
                processing: true,
                serverSide: true,
                order: [[2, 'desc']],
                columnDefs: [
                    {orderable: false, targets: [0,1,2,3,4,5,6,7]}, 
                    {className: "text-center vertical-align-center", targets: [7]},
                    {className: "text-right", targets: [3,4,5,6]}
                ],
                ajax: "BatchManagerQuery.do_sec?doSearch=true", 
                lengthMenu: [ 10, 25, 50, 100, 250, 500 ],
                pageLength: 25,
                "footerCallback": function ( row, data, start, end, display ) {
                    var api = this.api(), data;

                    // Update footer
                    $(api.column(3).footer() ).html(
                        api.column(3, {page:'current'}).data().sum()
                    );
            
                    $(api.column(4).footer() ).html(
                        api.column(4, {page:'current'}).data().sum()
                    );
            
                    $(api.column(5).footer() ).html(
                        api.column(5, {page:'current'}).data().sum()
                    );
            
                    $(api.column(6).footer() ).html(
                        api.column(6, {page:'current'}).data().sum()
                    );
                }
            });

            VAP.common.reportTable.on('draw', function () {
                VAP.common.addAriaToDataTable('tblReport', VAP.common.reportTable);
            });
            
            $("#master").change(function(){
                var status = $(this).is(":checked") ? true : false;
                $(".batchIds").prop("checked",status);
                
                if(status) {
                    VAP.common.fireAriaAlert('Select All checkbox checked.');
                } else {
                    VAP.common.fireAriaAlert('Select All checkbox not checked.');
                }
                
                if($('input[name=batchId]:checked').length > 0) {
                    $('#removeButton').removeClass('disabled');
                    $('#removeButton').attr('aria-label', 'Remove, Active');
                    $('#removeButton').attr('title', 'Remove, Active');
                } else {
                    $('#removeButton').addClass('disabled');
                    $('#removeButton').attr('aria-label', 'Remove, Inactive');
                    $('#removeButton').attr('title', 'Remove, Inactive');
                }
            });
        
            $('#startDate').datepicker({
                theme: 'vap',
                inputFormat: 'MM/dd/yyyy',
                outputFormat: 'MM/dd/yyyy',
                gainFocusOnConstruction: false,
                max: '<c:out value="${today}" />',
                onUpdate: function(curValue) {
                    $('#endDate').datepicker('min', curValue);
                }
            });

            $('#endDate').datepicker({
                theme: 'vap',
                inputFormat: 'MM/dd/yyyy',
                outputFormat: 'MM/dd/yyyy',
                gainFocusOnConstruction: false,
                max: '<c:out value="${today}" />',
                onUpdate: function(curValue) {
                    $('#startDate').datepicker('max', curValue);
                }
            });
            
            //make sure the end date calendar is closed when the start date one opens and vice versa
            $('#startDate').on('ab.datepicker.opening', function(){
                $('#endDate').datepicker('hide');
            });

            $('#endDate').on('ab.datepicker.opening', function(){
                $('#startDate').datepicker('hide');
            });
            
            //when the search button is pressed this function is used
            $('#btnSearch').on('click', function () {
                $('#tblReport .dataTables_empty').html(''); //blank out the empty table message if there is one
                
                var remoteOrganization = $('#remoteOrganization').val();
                var startDate = $('#startDate').val();
                var endDate = $('#endDate').val();

                var errorMessage = VAP.common.validateStartAndEndDate(startDate, endDate);

                if (errorMessage != '')
                {
                    //we have an error so show the error, and then don't send the ajax request
                    $('#errorMessage').html(errorMessage);
                    $('#errorMessage').show();

                    return false;
                } else {
                    //no error, let's make the call to populate the report
                    $('#results').show();
                    
                    VAP.common.reportTable.ajax.url("BatchManagerQuery.do_sec?doSearch=true&remoteOrganization=" + remoteOrganization + "&startDate=" + startDate + "&endDate=" + endDate).load();
                }
            });
            
            $('#startDate,#endDate').on('change', function () {
                //if either date changes, hide the error message if its not already hidden
                $('#errorMessage').hide();
            });
            
            $('#batchAnnounceDetailModal').on('show.bs.modal', function (event) {
                var button = $(event.relatedTarget);
                var batchId = button.data("batch-id");
                var orgId = button.data("org-id");
                var url = 'BatchManagerQuery.do_sec?getDetailView=true&batchId=' + batchId + '&orgId=' + orgId;
                
                var modalBody = $(this).find('.modal-body');
                
                $.ajax({
                    url: url,
                    spinnerTargetElement : 'batchAnnounceDetailModalBody',
                    headers: { 'skipLoadingAlert' : 'true' }
                }).done(function(data) {
                    modalBody.html(data);
                    modalBody.ready(function() { //verify that the modal's body has been generated
                        var detailsTable = $(this).find('#tblBatchAnnounceDetailReport');
                        var bodyContent = $(this).find('#batchAnnounceDetailModalContent');
                        detailsTable.on('draw.dt', function() { //set focus after detail table is generated and drawn
                            bodyContent.focus();
                        });
                    });
                });
            });
            
            $('#removeButton').on('click', function(){
                if(!$(this).hasClass('disabled')) {
                   //pop the confirm removal modal since the button is NOT disabled
                   $('#confirmRemoveBatchModal').modal('show');
                }
            });
            
            $('#confirmRemoveBatchModal').on('show.bs.modal', function (event) {
                var removalText = 'this (1) batch';
                var howManyBatches = $('input[name=batchId]:checked').length;
                
                if(howManyBatches > 1) {
                    removalText = 'these (' + howManyBatches + ') batches';
                }
                
                $('#removalText').html(removalText);
            });
            
            $(document).on('change', '.batchIds', function(){
                //alert that a checkbox was checked or not checked
                var status = $(this).is(":checked") ? true : false;
                var alertMessage = $(this).attr('aria-label');
                
                if(status) {
                    VAP.common.fireAriaAlert(alertMessage + ' checked.');
                } else {
                    VAP.common.fireAriaAlert(alertMessage + ' not checked.');
                }
                
                if($('input[name=batchId]:checked').length > 0) {
                    $('#removeButton').removeClass('disabled');
                    $('#removeButton').attr('aria-label', 'Remove, Active');
                    $('#removeButton').attr('title', 'Remove, Active');
                } else {
                    $('#removeButton').addClass('disabled');
                    $('#removeButton').attr('aria-label', 'Remove, Inactive');
                    $('#removeButton').attr('title', 'Remove, Inactive');
                }
            });
            
            //handle the delete
            $('#btnConfirmRemoveBatch').on('click', function(){
                //loop over and create a list of exports to delete
                var removalIds = $('input[name=batchId]:checked').map(function() { 
                    return $(this).val();
                }).get().join(',');

                //if a message is still showing, hide it
                $('#errorMessage').hide();
                
                //make the ajax call to delete them
                $.ajax({
                    url: "BatchManagerQuery.do_sec?delete=true&ids=" + removalIds, 
                    headers: { 'skipLoadingAlert' : 'true' },
                    success: function (result) {
                        var info = $.parseJSON(result);

                        //refresh the table
                        $('#tblReport .dataTables_empty').html(''); //blank out the empty table message if there is one
                        $('#tblReport').DataTable().ajax.reload();

                        //pop the success message
                        $('#messageBody').html('Successfully removed ' + $('input[name=batchId]:checked').length + ' task(s).');
                        $('#message').show().delay(5000).slideUp();
                        
                        //disable the remove button again
                        $('#removeButton').addClass('disabled');
                        $('#removeButton').attr('aria-label', 'Remove, Inactive');
                        $('#removeButton').attr('title', 'Remove, Inactive');
                    }, 
                    error: function(result) {
                       $('#errorMessage').html('There was an error removing the selected export(s). Please try again later.');
                       $('#errorMessage').show(); 
                    }
                }); 
            });
            
            $('#results').hide();
        });
    </script>
    <div class="row">
        <div class="col-xs-12 col-sm-12 col-md-12 col-lg-12">
            <div class="panel panel-default">
                <div class="panel-heading">
                    <div class="panel-title">
                        <a href="#" title="Search Criteria" aria-label="Search Criteria" aria-expanded="true" class="vap-panel-hide-show"><i class="fa fa-minus" aria-hidden="true" data-panel-body="patientDiscoverySummaryReportSearchPanel"></i></a> Search Criteria
                    </div>
                </div>    
                <div class="panel-body" id="patientDiscoverySummaryReportSearchPanel">
                    <div class="alert alert-info">
                        Fill in the fields below to narrow your search. All dates will be included if Start Date and End Date are left blank.
                    </div>
                    <div id="errorMessage" class="alert alert-warning" style="display: none;" role="alert"></div>
                    <div id="message" class="alert alert-success pad-bottom-10 pad-top-10 pad-left-10" role="alert" style="display: none;">
                        <button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>
                        <span id="messageBody"></span>
                    </div>
                    <form name="batchAnnounceForm" id="batchAnnounceForm" class="form-horizontal">
                        <div class="form-group">
                            <label for="startDate" class="col-xs-2 col-sm-2 col-md-2 col-lg-2 control-label">Start Date</label>
                            <div class="col-xs-3 col-sm-3 col-md-3 col-lg-3">
                                <input type="text" name="startDate" id="startDate" class="form-control vap-datepicker-input js-simple-tooltip" data-simpletooltip-text="<c:out value="${descriptors['manageBatchesStartDateField']}" />" placeholder="mm/dd/yyyy" value="<c:out value="${today}" />" />
                            </div>
                        </div>
                        <div class="form-group">
                            <label for="endDate" class="col-xs-2 col-sm-2 col-md-2 col-lg-2 control-label">End Date</label>
                            <div class="col-xs-3 col-sm-3 col-md-3 col-lg-3">
                                <input type="text" name="endDate" id="endDate" class="form-control vap-datepicker-input js-simple-tooltip" data-simpletooltip-text="<c:out value="${descriptors['manageBatchesStartDateField']}" />" placeholder="mm/dd/yyyy" value="<c:out value="${today}" />" />
                            </div>
                        </div>
                        <div class="form-group">
                            <label title="Query based on <c:out value="${organizationName}" /> Organization from the dropdown list" for="remoteOrganization" class="col-xs-2 col-sm-2 col-md-2 col-lg-2 control-label"><c:out value="${organizationName}" /> Organization</label>
                            <div class="col-xs-3 col-sm-3 col-md-3 col-lg-3">
                                <select id="remoteOrganization" name="remoteOrganization" data-simpletooltip-text="<c:out value="${descriptors['manageBatchesEHealthOrganizationsField']}"/>" class="form-control js-simple-tooltip">
                                    <option value="">ALL</option>
                                    <c:forEach var="r" items="${organizations}">
                                        <option value="<c:out value="${r.orgOid}" />"><c:out value="${r.orgName}" /></option>
                                    </c:forEach>
                                </select>
                            </div>
                        </div>
                        <div class="form-group">
                            <div class="col-xs-5 col-sm-5 col-md-5 col-lg-5 text-right">
                                <input name="btnSearch" id="btnSearch" title="Search" class="btn btn-primary" type="button" value="Search" />
                            </div>
                        </div>
                    </form>
                </div>
            </div>
        </div>
    </div>
    <div id="results">
        <div class="row">
            <div id="divReportContent" class="col-xs-12 col-sm-12 col-md-12 col-lg-12">
                <div class="panel panel-default" id="divReportPanel">
                    <div class="panel-heading">
                        <div class="panel-title">
                            <a href="#" title="Search Results" aria-label="Search Results" aria-expanded="true" class="vap-panel-hide-show"><i class="fa fa-minus" aria-hidden="true" data-panel-body="batchManagerQueryResultsPanel"></i></a> Search Results
                        </div>
                    </div>    
                    <div class="panel-body" id="batchManagerQueryResultsPanel">
                        <div id="deleteErrorMessage" class="alert alert-warning" style="display: none;" role="alert"></div>
                        <div class="pull-right">
                            <button class="btn btn-danger disabled" id="removeButton" title="Remove, Inactive" aria-label="Remove, Inactive">Remove <span class="fa fa-trash" aria-hidden="true"></span></button>
                        </div>
                        <table id="tblReport" class="table table-striped" width="100%" cellspacing="0" role="presentation">
                            <thead>
                                <tr role="row">
                                    <th role="columnheader" style="width: 80px;" class="js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="<c:out value="${descriptors['manageBatchesCheckBoxHeader']}"/>"><span id="selectAllHeader" data-simpletooltip-target="tableheader"><input type="checkbox" class="vertical-align-bottom" name="master" id="master" aria-label="Select All Rows" /> Select All</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="<c:out value="${descriptors['manageBatchesOrganizationHeader']}" />" style="min-width: 140px;"><span id="orgNameHeader" data-simpletooltip-target="tableheader">Organization Name</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="<c:out value="${descriptors['manageBatchesDateHeader']}" />" style="min-width: 120px;"><span id="dateHeader" data-simpletooltip-target="tableheader">Date Created (CT)</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="<c:out value="${descriptors['manageBatchesScheduledHeader']}" />"><span id="scheduledHeader" data-simpletooltip-target="tableheader">Scheduled</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="<c:out value="${descriptors['manageBatchesRequestedHeader']}" />"><span id="requestedHeader" data-simpletooltip-target="tableheader">Requests Completed</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary-left" data-simpletooltip-text="<c:out value="${descriptors['manageBatchesFailedHeader']}" />"><span id="failedHeader" data-simpletooltip-target="tableheader">Requests Failed</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary-left" data-simpletooltip-text="<c:out value="${descriptors['manageBatchesSizeHeader']}" />"><span id="batchSizeHeader" data-simpletooltip-target="tableheader">Batch Size</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary-left" data-simpletooltip-text="<c:out value="${descriptors['manageBatchesActionHeader']}" />"><span id="actionHeader" data-simpletooltip-target="tableheader">Action</span></th>
                                </tr>
                            </thead>
                            <tfoot>
                                <tr role="row">
                                    <%--Filled in by datatables, total--%>
                                    <td role="gridcell" class="text-left bold">Total:</td>
                                    <td role="gridcell"></td>
                                    <td role="gridcell"></td>
                                    <td role="gridcell" class="text-right bold"></td>
                                    <td role="gridcell" class="text-right bold"></td>
                                    <td role="gridcell" class="text-right bold"></td>
                                    <td role="gridcell" class="text-right bold"></td>
                                    <td role="gridcell"></td>
                                </tr>
                            </tfoot>
                        </table>
                    </div>
                </div>
            </div>
        </div>
    </div>
    <div class="modal fade" id="batchAnnounceDetailModal" role="dialog" data-backdrop="static" aria-labelledby="batchAnnounceDetailModalTitle">
        <div class="vertical-alignment-helper">
            <div class="modal-dialog vertical-align-center modal-lg">
                <div class="modal-content" id="batchAnnounceDetailsContent">
                    <div>
                        <div class="modal-header" id="batchAnnounceDetailModalTitle">
                            <h4 class="modal-title">Batch Announce Details</h4>
                        </div>
                        <div class="modal-body" id="batchAnnounceDetailModalBody"></div>
                    </div>
                    <div class="modal-footer">
                        <button type="button" class="btn btn-primary" data-dismiss="modal">Close</button>
                    </div>
                </div>
            </div>
        </div>
    </div>
                                        
    <div class="modal fade" id="confirmRemoveBatchModal" role="dialog" data-backdrop="static" aria-labelledby="confirmRemoveBatchModalLabel">
        <div class="vertical-alignment-helper">
            <div class="modal-dialog vertical-align-center" role="document">
                <div class="modal-content" tabindex="-1" id="confirmRemoveBatchModalContent">
                    <div class="modal-header">
                        <h4 class="modal-title" id="confirmRemoveBatchModalLabel">Confirm Remove Batch(es)</h4>
                    </div>
                    <div class="modal-body" aria-live="assertive" role="alert">
                        Are you sure you want to permanently remove <span id="removalText"></span>? This cannot be undone. 
                    </div>
                    <div class="modal-footer">
                        <button type="button" class="btn btn-default" data-dismiss="modal">Cancel</button>
                        <button type="button" id="btnConfirmRemoveBatch" class="btn btn-primary" data-dismiss="modal">Confirm</button>
                    </div>
                </div>
            </div>
        </div>
    </div>
</f:mainTemplate>