<%--
    Document   : Inbound Document Report
    Created By : David Vasquez / Robert Snelling
    Modified By: Asha Amritraj
--%>

<%@taglib tagdir="/WEB-INF/tags/formatting" prefix="f" %>
<%@taglib uri="http://java.sun.com/jsp/jstl/core" prefix="c" %>
<%@taglib uri="http://java.sun.com/jsp/jstl/functions" prefix="fn" %>
<%@taglib tagdir="/WEB-INF/tags/date" prefix="date" %>
<%@taglib tagdir="/WEB-INF/tags/cpp" prefix="cpp" %>
<%@page import="java.util.*"%>
<%@page import="java.text.SimpleDateFormat"%>

<f:mainTemplate headerText="Received ${organizationName} Documents Report">
    <script language="JavaScript">
        $(document).ready(function () {        
            $('#startDate').datepicker({
                theme: 'vap',
                inputFormat: 'MM/dd/yyyy',
                outputFormat: 'MM/dd/yyyy',
                gainFocusOnConstruction: false,
                max: '<c:out value="${today}" />',
                onUpdate: function(curValue) {
                    $('#endDate').datepicker('min', curValue);
                }
            });
            
            $('#endDate').datepicker({
                theme: 'vap',
                inputFormat: 'MM/dd/yyyy',
                outputFormat: 'MM/dd/yyyy',
                gainFocusOnConstruction: false,
                max: '<c:out value="${today}" />',
                onUpdate: function(curValue) {
                    $('#startDate').datepicker('max', curValue);
                }
            });
            
            //make sure the end date calendar is closed when the start date one opens and vice versa
            $('#startDate').on('ab.datepicker.opening', function(){
                $('#endDate').datepicker('hide');
            });

            $('#endDate').on('ab.datepicker.opening', function(){
                $('#startDate').datepicker('hide');
            });

            VAP.common.setProcessingForDataTable('reportTable', 'divReportPanel');
            
            VAP.common.reportTable = $('#reportTable').DataTable({
                deferLoading: 0,
                destroy: true,
                ajax: { 
                    url: VAP.common.constructUrl("doSearch"),
                    dataSrc: function(json){
                        $('.dataTables_empty').remove();
                        return json.data;
                    },
                    error: function(o){
                        VAP.common.handleErrorsInDataTable('reportTable', o, 14, 'divReportPanel');
                    }
                }, 
                info: <c:out value="${info}" />,
                pagingType: '<c:out value="${pagingType}" />',
                paging: true,
                dom: "<'row'<'col-sm-12'l>>" +
                        "<'row'<'col-sm-6'i><'col-sm-6'p>>" +
                        "<'row'<'col-sm-12't<'pad-bottom-20'>r>>" +
                        "<'row'<'col-sm-5'i><'col-sm-7'p>>",
                lengthMenu: [ 10, 25, 50, 100, 250, 500 ],
                pageLength: 25,
                processing: false,
                searching: false,
                serverSide: true,
                order: [[3, "desc"]],
                columnDefs: [
                    {orderable: false, targets: [11, 12, 13]},
                    {
                        targets: 13,
                        data: null,
                        render: function(data,type,full,meta){
                            return '<a href="#/" class="btn btn-primary btn-xs" onclick="documentView(\''+data[13]+'\', \''+data[14]+'\');">View</a>';
                        }
                    },
                    {
                        targets: 0,
                        data: null,
                        render: function(data,type,full,meta){
                            return '<a href="#/" onclick="linkToPatient(\''+data[12]+'\', \''+data[1]+'\', \''+data[2]+'\');">'+data[0]+'</a>';
                        }
                    },
                    {
                        targets: 12,
                        data: null,
                        render: function(data,type,full,meta){
                            return data[15];
                        }
                    },
                    {className: "vap-col-min-80", targets: 0},
                    {className: "vap-col-min-130", targets: [3,5,7]},
                    {className: "vap-col-min-200", targets: 4},
                    {className: 'text-center vertical-align-center', targets: [13]}
                ]
            });
            
            VAP.common.reportTable.on('draw', function () {
                VAP.common.addAriaToDataTable('reportTable', VAP.common.reportTable);
                VAP.common.addSrollingToDataTable('reportTable');
            });
            
            $('#searchButton').on('click', function () {
                $('#reportTable .dataTables_empty').html(''); //blank out the empty table message if there is one
                
                var startDate = $('#startDate').val();
                var endDate = $('#endDate').val();

                var errorMessage = VAP.common.validateStartAndEndDate(startDate, endDate);
                
                if (errorMessage != '')
                {
                    //we have an error so show the error, and then don't send the ajax request
                    $('#errorMessage').html(errorMessage);
                    $('#errorMessage').show();

                    return false;
                } else {
                    //no error, let's make the call to populate the report
                    $('#results').show();
                    VAP.common.reportTable.ajax.url(VAP.common.constructUrl('doSearch')).load();
                }
            });

            $('#startDate,#endDate').on('change', function () {
                //if either date changes, hide the error message if its not already hidden
                $('#errorMessage').hide();
            });
            
            $('#results').hide();
            
            //set up JSON to pass to the Advanced Scheduling modal/functionality
            $('#btnScheduleExport').on('click', function(){
                //set up array of items to view at the top of the schedule export modal
                var viewParameters = [ 
                    {name : 'SSN', value : $('#ssn').val()}, 
                    {name : 'Last Name', value : $('#lastName').val()}, 
                    {name : 'First Name', value : $('#firstName').val()},
                    {name : 'User ID', value : $('#userId').val()},
                    {name : 'Purpose of Use', value : $('#purposeOfUse option:selected').text()},
                    {name : 'Patient Preferred Facility', value : $('#facility option:selected').text()},
                    {name : 'Organization', value : $('#remoteOrganization option:selected').text()},
                    {name : 'Patient Types', value : $('#patientTypes option:selected').text()} 
                ];

                //set up array of items to pass as JSON in the schedule export modal form
                var parameters = {
                    'ssn' : $('#ssn').val(), 
                    'lastName': $('#lastName').val(), 
                    'firstName' : $("#firstName").val(),
                    'userId' : $('#userId').val(),
                    'purposeOfUse' : $('#purposeOfUse').val(),
                    'facility' : $('#facility').val(),
                    'remoteOrganization' : $('#remoteOrganization').val(),
                    'patientTypes' : $('#patientTypes').val() 
                };

                VAP.common.advancedSchedulingPopulateScheduleParameters(viewParameters, parameters, 'Received eHealth Exchange Documents Report');
            });
        });
        
        VAP.common.constructUrl = function(methodName){
            var url = 'InboundDocumentReport.do_sec' +
                    '?' + methodName + '=true' +
                    '&ssn=' + $('#ssn').val() +
                    '&firstName=' + $('#firstName').val() +
                    '&lastName=' + $('#lastName').val() +
                    '&purposeOfUse=' + $('#purposeOfUse').val() +
                    '&recordsPerPage=' + $('#recordsPerPage').val() +
                    '&currentPage=' + $('#currentPage').val() +
                    '&startDate=' + $('#startDate').val() +
                    '&endDate=' + $('#endDate').val() +
                    '&userId=' + $('#userId').val() +
                    '&remoteOrganization=' + $('#remoteOrganization').val() +
                    '&facility=' + $('#facility').val() +
                    '&facilityName=' + $('#facilityName').val() +
                    '&patientTypes=' + $('#patientTypes').val() +
                    '&sortBy=' + $('#reportTable').dataTable().fnSettings().aaSorting[0][0] +
                    '&sortOrder=' + $('#reportTable').dataTable().fnSettings().aaSorting[0][1];

            if (methodName === 'exportToCsv' || methodName === 'exportToExcel') {
                url = url + '&totalRows=' + VAP.common.reportTable.page.info().recordsTotal;
            }
            return url;
        };
        
        //Used by 'view' button in exchange final table column
        function documentView(documentIcn, documentUniqueID){
            $('#document-View-ICN').val(documentIcn);
            $('#document-View-Unique-Id').val(documentUniqueID);
            document.getElementById('documentViewForm').submit();
        }
    </script>
    
    <div class="row">
        <div class="col-xs-12 col-sm-12 col-md-12 col-lg-12">
            <div class="panel panel-default">
                <div class="panel-heading">
                    <div class="panel-title">
                        <a href="#" title="Search Criteria" aria-label="Search Criteria" aria-expanded="true" class="vap-panel-hide-show"><i class="fa fa-minus" aria-hidden="true" data-panel-body="receivedDocumentsReportSearchPanel"></i></a> Search Criteria
                    </div>
                </div>
                 <div class="panel-body" id="receivedDocumentsReportSearchPanel">
                    <div class="alert alert-info">
                        Fill in the fields below to narrow your search. All dates will be included if Start Date and End Date are left blank.
                    </div>
                    <div id="errorMessage" class="alert alert-warning pad-15 margin-bottom-10" style="display: none;" role="alert"></div>
                    <div id="userScheduledExportModalMessage" class="alert alert-dismissible" role="alert" style="display: none;">
                        <button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>
                    </div>
                    <form name="receivedDocumentsReportForm" action="InboundDocumentReport.do_sec" method="POST" class="form-horizontal" role="form">
                        <input type="hidden" id="organizationName" name="organizationName" />
                        <input type="hidden" id="facilityName" name="facilityName" />
                        <div class="row">
                            <div class="col-xs-5 col-sm-5 col-md-5 col-lg-5">
                                <div class="form-group">
                                    <label for="ssn" class="col-xs-4 col-sm-4 col-md-4 col-lg-4 control-label">SSN</label>
                                    <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6">
                                        <%-- <input title="Enter SSN. Enter numbers only no hyphens" id="ssn" name="ssn" type="text" maxlength="9" size="9" data-tooltip="<c:out value="${descriptors['ssnField']}"/>" placeholder="#########" class="form-control" /> --%>
                                        <input title="Enter SSN. Enter numbers only no hyphens" id="ssn" name="ssn" type="text" maxlength="9" size="9" data-simpletooltip-text="<c:out value="${descriptors['ssnField']}"/>" placeholder="#########" class="form-control js-simple-tooltip" />
                                    </div>
                                </div>
                                <div class="form-group">
                                    <label for="lastName" class="col-xs-4 col-sm-4 col-md-4 col-lg-4 control-label">Last Name</label>
                                    <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6">
                                        <%-- <input title="Enter Last Name" id="lastName" name="lastName" type="text" maxlength="255" size="30" data-tooltip="<c:out value="${descriptors['lastNameField']}"/>" class="form-control" /> --%>
                                        <input title="Enter Last Name" id="lastName" name="lastName" type="text" maxlength="255" size="30" data-simpletooltip-text="<c:out value="${descriptors['lastNameField']}"/>" class="form-control js-simple-tooltip" />
                                    </div>
                                </div>
                                <div class="form-group">
                                    <label for="firstName" class="col-xs-4 col-sm-4 col-md-4 col-lg-4 control-label">First Name</label>
                                    <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6">
                                        <%-- <input title="Enter First Name" id="firstName" name="firstName" type="text" maxlength="255" size="30" data-tooltip="<c:out value="${descriptors['firstNameField']}"/>" class="form-control" /> --%>
                                        <input title="Enter First Name" id="firstName" name="firstName" type="text" maxlength="255" size="30" data-simpletooltip-text="<c:out value="${descriptors['firstNameField']}"/>" class="form-control js-simple-tooltip" />
                                    </div>
                                </div>
                                <div class="form-group">
                                    <label for="userId" class="col-xs-4 col-sm-4 col-md-4 col-lg-4 control-label">User ID</label>
                                    <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6"> 
                                        <%-- <input title="Enter User ID" data-tooltip="<c:out value="${descriptors['genericUserId']}" />" id="userId" name="userId" type="text" maxlength="255" class="form-control" /> --%>
                                        <input title="Enter User ID" data-simpletooltip-text="<c:out value="${descriptors['genericUserId']}" />" id="userId" name="userId" type="text" maxlength="255" class="form-control js-simple-tooltip" />
                                    </div>
                                </div>
                                <div class="form-group">
                                    <label for="startDate" class="col-xs-4 col-sm-4 col-md-4 col-lg-4 control-label">Start Date</label>
                                    <%-- <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6" data-tooltip="<c:out value="${descriptors['startDateEndDateField']}" />">
                                        <input type="text" name="startDate" id="startDate" class="form-control" value="<c:out value="${endDateDefault}" />" /> --%>
                                    <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6">
                                        <input type="text" name="startDate" id="startDate" class="form-control js-simple-tooltip" data-simpletooltip-text="<c:out value="${descriptors['startDateEndDateField']}" />" value="<c:out value="${endDateDefault}" />" />
                                    </div>
                                </div>
                                <div class="form-group">
                                    <label for="endDate" class="col-xs-4 col-sm-4 col-md-4 col-lg-4 control-label">End Date</label>
                                    <%-- <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6" data-tooltip="<c:out value="${descriptors['startDateEndDateField']}" />">
                                        <input type="text" name="endDate" id="endDate" class="form-control" value="<c:out value="${endDateDefault}" />" /> --%>
                                    <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6">
                                        <input type="text" name="endDate" id="endDate" class="form-control js-simple-tooltip" data-simpletooltip-text="<c:out value="${descriptors['startDateEndDateField']}" />" value="<c:out value="${endDateDefault}" />" />
                                    </div>
                                </div>
                            </div>
                            <div class="col-xs-7 col-sm-7 col-md-7 col-lg-7">
                                <c:if test="${pouFlag eq true}">
                                    <div class="form-group">
                                        <label for="purposeOfUse" class="col-xs-3 col-sm-3 col-md-3 col-lg-3 control-label">Purpose of Use</label>
                                        <div class="col-xs-7 col-sm-7 col-md-7 col-lg-7">
                                            <%-- <select id="purposeOfUse" name="purposeOfUse" data-tooltip="<c:out value="${descriptors['purposeOfUseField']}"/>" class="form-control"> --%>
                                            <select id="purposeOfUse" name="purposeOfUse" data-simpletooltip-text="<c:out value="${descriptors['purposeOfUseField']}"/>" class="form-control js-simple-tooltip" data-simpletooltip-prefix-class="minimalist-left">
                                                <option value="">ALL</option>
                                                <c:forEach var="pou" items="${pous}">
                                                    <option value="<c:out value="${pou.pouValue}" />"><c:out value="${pou.pouValue}" /></option>
                                                </c:forEach>
                                            </select>
                                        </div>
                                    </div>
                                </c:if>
                                <div class="form-group">
                                    <label class="col-xs-3 col-sm-3 col-md-3 col-lg-3 control-label" for="facility">Patient Preferred Facility</label>
                                    <div class="col-xs-7 col-sm-7 col-md-7 col-lg-7">
                                        <%-- <select id="facility" name="facility" class="form-control" onchange="document.getElementById('facilityName').value = this.options[this.selectedIndex].text;" data-tooltip="<c:out value="${descriptors['patientPreferredFacilityFilter']}" />"> --%>
                                        <select id="facility" name="facility" class="form-control js-simple-tooltip" onchange="document.getElementById('facilityName').value = this.options[this.selectedIndex].text;" data-simpletooltip-text="<c:out value="${descriptors['patientPreferredFacilityFilter']}" />" data-simpletooltip-prefix-class="minimalist-left">
                                            <option value="" >ALL</option>
                                            <c:forEach var="f" items="${facilities}">
                                                <option value="<c:out value="${f.facilityStation}" />"
                                                    <c:choose>
                                                        <c:when test="${f.facilityStation eq defaultUserFacility.facilityStation}">
                                                            selected
                                                        </c:when>
                                                    </c:choose>>
                                                    <c:choose>
                                                        <c:when test="${f.facilityName != ''}">
                                                            <c:out value="${f.facilityName}" />
                                                        </c:when>
                                                        <c:otherwise>
                                                            <c:out value="${fn:split(f.facilityDns, '.')[0]}" />
                                                        </c:otherwise>
                                                    </c:choose>
                                                    (<c:out value="${f.facilityStation}" />)
                                                </option>
                                            </c:forEach>
                                        </select>
                                    </div>
                                </div>
                                <div id="organizationDiv" class="form-group">
                                    <label for="remoteOrganization" class="col-xs-3 col-sm-3 col-md-3 col-lg-3 control-label">Organization</label>
                                    <div class="col-xs-7 col-sm-7 col-md-7 col-lg-7">
                                        <%-- <select id="remoteOrganization" name="remoteOrganization" onchange="document.getElementById('organizationName').value = this.options[this.selectedIndex].text;" data-tooltip="<c:out value="${descriptors['accountingOfDisclosuresReportOrganizationField']}"/>" class="form-control"> --%>
                                        <select id="remoteOrganization" name="remoteOrganization" onchange="document.getElementById('organizationName').value = this.options[this.selectedIndex].text;" data-simpletooltip-text="<c:out value="${descriptors['accountingOfDisclosuresReportOrganizationField']}"/>" class="form-control js-simple-tooltip" data-simpletooltip-prefix-class="minimalist-left">
                                            <option value="" selected>ALL</option>
                                            <c:forEach var="r" items="${activeOrgs}">
                                                <option value="%<c:out value="${r.orgOid}" />"><c:out value="${r.orgName}" /></option>
                                            </c:forEach>
                                            <c:forEach var="r" items="${inactiveOrgs}">
                                                <option value="%<c:out value="${r.orgOid}" />">[INACTIVE] <c:out value="${r.orgName}" /></option>
                                            </c:forEach>
                                        </select>
                                    </div>
                                </div>
                                <div id="patientTypesDiv" class="form-group">
                                    <label for="patientTypes" class="col-xs-3 col-sm-3 col-md-3 col-lg-3 control-label">Patient Types</label>
                                    <div class="col-xs-7 col-sm-7 col-md-7 col-lg-7">
                                        <%-- <select id="patientTypes" name="patientTypes" data-tooltip="<c:out value="${descriptors['patientTypesField']}"/>" class="form-control"> --%>
                                        <select id="patientTypes" name="patientTypes" data-simpletooltip-text="<c:out value="${descriptors['patientTypesField']}"/>" class="form-control js-simple-tooltip" data-simpletooltip-prefix-class="minimalist-left">
                                            <option value="3">ALL</option>
                                            <option value="1" selected>Real Patients</option>
                                            <option value="2">Test Patients</option>
                                        </select>
                                    </div>
                                </div>
                            </div>
                        </div>
                    </form>
                    <div class="row">
                        <div class="col-xs-12 col-sm-12 col-md-12 col-lg-12">
                            <div class="form-group">
                                <div class="text-right">
                                    <cpp:userInRole role="VAPApplicationWebAdministrator">
                                        <button name="btnScheduleExport" id="btnScheduleExport" class="btn btn-default" data-toggle="modal" data-target="#scheduleExportModal" title="Schedule" aria-label="Schedule">Schedule</button>
                                    </cpp:userInRole>
                                    <input id="searchButton" name="search" title="Search" class="btn btn-primary" type="submit" value="Search" />
                                </div>
                            </div>
                        </div>
                    </div>
                 </div>
            </div>
        </div>
    </div>
    <div><div aria-live="assertive" role="alert" id="reportTable_info"></div></div>
    <div id="results">
        <div class="row">
            <%@include file="patientLinkForm.jsp"%>
            <div id="divReportContent" class="col-xs-12 col-sm-12 col-md-12 col-lg-12">
                <div class="panel panel-default" id="divReportPanel">
                    <div class="panel-heading">
                        <div class="panel-title">
                            <div class="pull-left">
                                <a href="#" title="Search Results" aria-label="Search Results" aria-expanded="true" class="vap-panel-hide-show"><i class="fa fa-minus" aria-hidden="true" data-panel-body="receivedDocumentsReportResultsPanel"></i></a> Search Results
                            </div>
                            <div class="pull-right">
                                <a href="#" data-toggle="modal" data-target="#confirmExportModal" data-method="exportToExcel" title="Export to Excel" class="pad-right-10"><i class="fa fa-download" aria-hidden="true"></i> Export to Excel</a>
                                <a href="#" data-toggle="modal" data-target="#confirmExportModal" data-method="exportToCsv" title="Export to CSV" class="pad-right-10"><i class="fa fa-download" aria-hidden="true"></i> Export to CSV</a>
                            </div>
                            <div class="clearfix"></div>
                        </div>
                    </div>
                    <div class="panel-body" id="receivedDocumentsReportResultsPanel">
                        <form action="InboundDocumentReport.do_sec" method="POST" target="_blank" onsubmit="return doSubmit(this)" id="documentViewForm">
                            <input type="hidden" name="view" value="true">
                            <input type="hidden" name="documentIcn"  value="" id="document-View-ICN">
                            <input type="hidden" name="documentUniqueId" value="" id="document-View-Unique-Id">
                        </form>
                        <table id="reportTable" class="table table-striped" width="100%" cellspacing="0" role="presentation">
                            <thead>
                                <tr role="row">
                                    <%-- <th role="columnheader" class="text-center"><span data-tooltip="<c:out value="${descriptors['ssnColumn']}"/>" id="ssnHeader">SSN</span></th> --%>
                                    <%-- <th role="columnheader" class="text-center"><span data-tooltip="<c:out value="${descriptors['lastNameColumn']}"/>" id="lastNameHeader">Patient Last Name</span></th> --%>
                                    <%-- <th role="columnheader" class="text-center"><span data-tooltip="<c:out value="${descriptors['firstNameColumn']}"/>" id="firstNameHeader">Patient First Name</span></th> --%>
                                    <%-- <th role="columnheader" class="text-center"><span data-tooltip="<c:out value="${descriptors['eventOccurredColumn']}"/>" id="dateReceivedHeader">Date of Disclosures (CT)</span></th> --%>
                                    <%-- <th role="columnheader" class="text-center"><span id="documentTitleHeader" data-tooltip="<c:out value="${descriptors['documentTitleColumn']}" />">Document Title</span></th> --%>
                                    <%-- <th role="columnheader" class="text-center"><span id="facilityHeader" data-tooltip="<c:out value="${descriptors['receivedEHealthExchangeDocumentsReportFacilityWithDefaultField']}" />">Patient Preferred Facility</span></th> --%>
                                    <%-- <th role="columnheader" class="text-center"><span id="facilityNumberHeader" data-tooltip="<c:out value="${descriptors['patientPreferredFacilityStationIdColumn']}" />">Patient Preferred Facility Station ID</span></th> --%>
                                    <%-- <th role="columnheader" class="text-center"><span id="organizationHeader" data-tooltip="<c:out value="${descriptors['receivedEHealthExchangeDocumentsReportOrganizationColumn']}" />"><c:out value="${organizationName}" /> Organization</span></th> --%>
                                    <%-- <th role="columnheader" class="text-center"><span id="userIdHeader" data-tooltip="<c:out value="${descriptors['receivedEHealthExchangeDocumentsReportUserIdColumn']}" />">User ID</span></th> --%>
                                    <%-- <th role="columnheader" class="text-center"><span id="userFacilityNameHeader" data-tooltip="<c:out value="${descriptors['receivedEHealthExchangeDocumentsReportUserFacilityNameColumn']}" />">User Facility</span></th> --%>
                                    <%-- <th role="columnheader" class="text-center"><span id="pouHeader" data-tooltip="<c:out value="${descriptors['purposeOfUseColumn']}" />">Purpose of Use</span></th> --%>
                                    <%-- <th role="columnheader" class="text-center"><span id="userRoleHeader" data-tooltip="<c:out value="${descriptors['receivedEHealthExchangeDocumentsReportUserRoleColumn']}" />" data-tooltip-position="left">User Role</span></th> --%>
                                    <%-- <th role="columnheader" class="text-center"><span data-tooltip="This column displays the document source system ID" data-tooltip-position="left">System ID</span></th> --%>
                                    <%-- <th role="columnheader" class="text-center"><span id="viewHeader" data-tooltip="<c:out value="${descriptors['genericViewColumn']}" />" data-tooltip-position="left">View</span></th> --%>
                                    
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-text="<c:out value="${descriptors['ssnColumn']}"/>" data-simpletooltip-prefix-class="secondary"><span id="ssnHeader" data-simpletooltip-target="tableheader">SSN</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-text="<c:out value="${descriptors['lastNameColumn']}"/>" data-simpletooltip-prefix-class="secondary"><span id="lastNameHeader" data-simpletooltip-target="tableheader">Patient Last Name</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-text="<c:out value="${descriptors['firstNameColumn']}"/>" data-simpletooltip-prefix-class="secondary"><span id="firstNameHeader" data-simpletooltip-target="tableheader">Patient First Name</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-text="<c:out value="${descriptors['eventOccurredColumn']}"/>" data-simpletooltip-prefix-class="secondary"><span id="dateReceivedHeader" data-simpletooltip-target="tableheader">Date of Disclosures (CT)</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-text="<c:out value="${descriptors['documentTitleColumn']}" />" data-simpletooltip-prefix-class="secondary"><span id="documentTitleHeader" data-simpletooltip-target="tableheader">Document Title</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-text="<c:out value="${descriptors['receivedEHealthExchangeDocumentsReportFacilityWithDefaultField']}" />" data-simpletooltip-prefix-class="secondary"><span id="facilityHeader" data-simpletooltip-target="tableheader">Patient Preferred Facility</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-text="<c:out value="${descriptors['patientPreferredFacilityStationIdColumn']}" />" data-simpletooltip-prefix-class="secondary"><span id="facilityNumberHeader" data-simpletooltip-target="tableheader">Patient Preferred Facility Station ID</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-text="<c:out value="${descriptors['receivedEHealthExchangeDocumentsReportOrganizationColumn']}" />" data-simpletooltip-prefix-class="secondary"><span id="organizationHeader" data-simpletooltip-target="tableheader"><c:out value="${organizationName}" /> Organization</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-text="<c:out value="${descriptors['receivedEHealthExchangeDocumentsReportUserIdColumn']}" />" data-simpletooltip-prefix-class="secondary"><span id="userIdHeader" data-simpletooltip-target="tableheader">User ID</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-text="<c:out value="${descriptors['receivedEHealthExchangeDocumentsReportUserFacilityNameColumn']}" />" data-simpletooltip-prefix-class="secondary"><span id="userFacilityNameHeader" data-simpletooltip-target="tableheader">User Facility</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-text="<c:out value="${descriptors['purposeOfUseColumn']}" />" data-simpletooltip-prefix-class="secondary-left"><span id="pouHeader" data-simpletooltip-target="tableheader">Purpose of Use</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-text="<c:out value="${descriptors['receivedEHealthExchangeDocumentsReportUserRoleColumn']}" />" data-simpletooltip-prefix-class="secondary-left"><span id="userRoleHeader" data-simpletooltip-target="tableheader">User Role</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-text="This column displays the document source system ID" data-simpletooltip-prefix-class="secondary-left"><span data-simpletooltip-target="tableheader">System ID</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-text="<c:out value="${descriptors['genericViewColumn']}" />" data-simpletooltip-prefix-class="secondary-left"><span id="viewHeader" data-simpletooltip-target="tableheader">View</span></th>
                                </tr>
                            </thead>
                        </table>
                    </div>
                </div>
            </div>
        </div>
    </div>
    
    <%-- adding the advanced scheduling option to this report --%>
    <cpp:advancedScheduling />
</f:mainTemplate>