<%@taglib uri="http://java.sun.com/jsp/jstl/core" prefix="c"%>
<%@taglib uri="http://java.sun.com/jsp/jstl/functions" prefix="fn"%>
<%@taglib uri="http://java.sun.com/jsp/jstl/fmt" prefix="fmt"%>
<%@taglib tagdir="/WEB-INF/tags/cpp" prefix="cpp"%>
<%@taglib tagdir="/WEB-INF/tags/date" prefix="date"%>
<%@taglib tagdir="/WEB-INF/tags/formatting" prefix="f" %>

<script type="text/javascript">
    //restrictions select boxes setup
    var opt = new OptionTransfer("modifyList1", "modifyList2");
    opt.setAutoSort(false);
    opt.setDelimiter(",");
    opt.setStaticOptionRegex(new RegExp("^.*?Defense.*?$", "i"));
    opt.saveNewLeftOptions("newLeft");
    opt.saveNewRightOptions("newRight");
        
    $(document).ready(function () { 
        $('#modifyRestrictionsSignatureDate').datepicker({
            theme: 'vap',
            inputFormat: 'MM/dd/yyyy',
            outputFormat: 'MM/dd/yyyy',
            max: '<c:out value="${today}" />',
            gainFocusOnConstruction: false
        });
        
        opt.init(document.multiSelectListForm);
        
        //$('#multiSelectListForm').validator();
        
        $('#multiSelectListForm').validator({
            custom: {
                isValidDateMMDDYYYY: function($el) { 
                    var today = moment();
                    var dateToValidate = moment($el.val(), 'MM/DD/YYYY');
                    
                    //return true to denote an error
                    return !moment($el.val(), 'MM/DD/YYYY', true).isValid() || dateToValidate.isAfter(today);
                }
            }
        });
        
        //call the calendar validation helper tag to remove the error classes for the date inputs and surrounding divs when tabbing/clicking off of the fields
        <date:calendarValidationHelper elementsToRemoveErrorClassesOn="modifyRestrictionsSignatureDateDiv,modifyRestrictionsSignatureDate" />
        
        $('#modifyList2').on('focusout', function() {
            validateList2();
        });
        
        $('.moveButtons').on('click', function () {
            validateList2();
        });
    });
    
    validateList2 = function() {
        if ($('#modifyList2')[0].children.length === 0) {
            //we don't have anything selected, so we better show this is required
            $('#modifyDivList2').addClass('has-error has-danger');
            $('#modifyDivList2Select').addClass('has-error has-danger');
        } else {
            $('#modifyDivList2').removeClass('has-error has-danger');
            $('#modifyDivList2Select').removeClass('has-error has-danger');
        }
    };
</script>

<div class="alert alert-info">
    <p>
        The Veteran has a right to request the
        Department of Veterans Affairs (VA) to restrict or limit the
        sharing of his/her electronic health information through <c:out value="${organizationName}"/> 
        by designating which non-VA healthcare provider organizations 
        he/she does NOT wish to receive his/her information. A restriction 
        request may be filed even if the Veteran does not have an authorization 
        on file permitting the disclosure of his/her health information to 
        non-VA healthcare provider organizations. Note that until an 
        authorization is on file, the restriction request will remain in an 
        <strong>inactive</strong> status. Any restriction request submitted will apply 
        ONLY to the sharing of electronic health information through 
        <strong><c:out value="${organizationName}"/></strong>.
    </p>
    <p>
        Indicate which non-VA health care provider 
        organization participating in <c:out value="${organizationName}"/> the Veteran does NOT 
        wish to receive their electronic health information. By completing the 
        Restriction on VA Form 10-0525a, the Veteran is restricting the VA 
        from sharing his/her health information for treatment purposes 
        with the non-VA health care provider organizations participating 
        in the <c:out value="${organizationName}"/> and partnering with VA.
    </p>
    <p>
        The VA will not share the Veteran health 
        information with the selected non-VA health care provider 
        organizations through <c:out value="${organizationName}"/> even if the Veteran later signs 
        an authorization.
    </p>
    <p>
        There must be at least one non-VA health care 
        provider organization -- other than Department of Defense 
        (DoD) --- in the "All Providers and Organizations <strong>(unrestricted)</strong>" 
        selection box AND at least one non-VA health care provider 
        organization in the "<c:out value="${organizationName}"/> Providers and Organizations 
        who will NOT have access to the records <strong>(restricted)</strong>" box. 
    </p>    
    <p>
        * NOTE: You cannot restrict DoD, VA patient information is always shared with the DoD.
    </p>
</div>
<div id="modifyRestrictionsErrorMessage" class="alert alert-danger" style="display: none;" role="alert"></div>
<div>
    <form name="multiSelectListForm" id="multiSelectListForm" action="PatientDetails.do_sec" method="POST" class="form-horizontal" data-toggle="validator" role="form">
        <input type="hidden" name="icn" value="<c:out value="${patientDemographics.icn}"/>" /> 
        <input type="hidden" name="requestId" value="<c:out value="${requestId}"/>" /> 
        <input type="hidden" name="newLeft" value="" /> 
        <input type="hidden" name="newRight" value="" /> 
        <input type="hidden" name="consentType" value="NwHIN Organization Restriction Modification" /> 
        <input type="hidden" name="purposeOfUse" value="TREATMENT" />
        <input type="hidden" name="optIn" value="true" /> <%-- servlet method to call on submit of this form --%>
        
        <div id="multiSelectAuthorizeForm" class="form-group margin-left-2 margin-right-2">
            <div class="col-xs-5 col-sm-5 col-md-5 col-lg-5">
                <div class="form-group" id="modifyDivList1">
                    <label for="modifyList1" class="control-label">All Providers and Organizations (unrestricted)</label>
                    <div>
                        <select class="listSelect form-control" name="modifyList1" id="modifyList1" multiple size="10" onDblClick="opt.transferRight(); validateList2();">
                            <c:if test="${!empty authorizedOrganizations}">
                                <c:forEach var="r" items="${authorizedOrganizations}" varStatus="i">
                                    <c:choose>
                                        <c:when test="${r.orgNumber eq  '200DOD'}">
                                            <option value="<c:out value="${r.orgNumber}"/>"><c:out value="${r.orgName}"/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</option>
                                        </c:when>
                                        <c:otherwise>
                                            <option value="<c:out value="${r.orgNumber}"/>"><c:out value="${r.orgName}"/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</option>
                                        </c:otherwise>
                                    </c:choose>
                                </c:forEach>
                            </c:if>
                        </select>
                    </div>
                </div>
            </div>
            <div class="col-xs-2 col-sm-2 col-md-2 col-lg-2 text-center pad-top-40">
                <%-- buttons --%>
                <input type="button" class="midButton moveButtons btn btn-primary btn-block btn-sm" name="right" value="Move All &gt;&gt;" onclick="opt.transferAllRight()" alt="Move all the organization to the restriction list" aria-label="Move all the organization to the restriction list" title="Move all the organization to the restriction list" />
                <input type="button" class="midButton moveButtons btn btn-info margin-top-10 btn-block btn-sm" name="right" value="Move Selected &gt;" onclick="opt.transferRight()" alt="Move selected organization to the restriction list" aria-label="Move selected organization to the restriction list" title="Move selected organization to the restriction list" />
                <input type="button" class="midButtonLeft moveButtons btn btn-info margin-top-10 btn-block btn-sm" name="left" value="&lt; Move Selected" onclick="opt.transferLeft()" alt="Remove selected organization from the restriction list" aria-label="Remove selected organization from the restriction list" title="Remove selected organization from the restriction list" />
                <input type="button" class="midButton moveButtons btn btn-default margin-top-10 btn-block btn-sm" name="left" value="Clear" onclick="opt.transferAllLeft()" alt="Clear all organizations from the restriction list" aria-label="Clear all organizations from the restriction list" title="Clear all organizations from the restriction list" />
            </div>
            <div class="col-xs-5 col-sm-5 col-md-5 col-lg-5" id="modifyDivList2">
                <div class="form-group">
                    <label for="modifyList2" class="control-label"><f:requiredField altText="Providers and Organizations to restrict is a required field." labelText="Providers and Organizations to restrict (restricted)" useNoWrap="false" /></label>
                    <div id="modifyDivList2Select">
                        <select name="modifyList2" class="listSelect form-control" multiple size="10" id="modifyList2" onDblClick="opt.transferLeft(); validateList2();" title="Providers and Organizations to restrict (restricted), Required" aria-label="Providers and Organizations to restrict (restricted), Required">
                            <c:if test="${!empty excludedOrganizations}">
                                <c:forEach var="r" items="${excludedOrganizations}" varStatus="i">
                                    <option value="<c:out value="${r.orgNumber}"/>"><c:out value="${r.orgName}"/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</option>
                                </c:forEach>
                            </c:if>
                        </select>
                    </div>
                </div>
            </div>
        </div>
        <div class="form-group">
            <label for="authenticatingFacility" class="col-xs-3 col-sm-3 col-md-3 col-lg-3 control-label">
                <f:requiredField altText="Authenticating Facility is a required field." labelText="Authenticating Facility" />
            </label> 
            <div class="col-xs-9 col-sm-9 col-md-9 col-lg-9">
                <select id="authenticatingFacility" name="userFacility" class="form-control" required>
                    <c:forEach var="f" items="${allowedFacilities}">
                        <option value="<c:out value="${f.facilityStation}"/>"
                            <c:choose>
                                <c:when test="${f.facilityStation eq defaultUserFacility.facilityStation}">
                                    selected
                                </c:when>
                                <c:when test="${cookie.facility.value == f.facilityStation}">
                                    selected
                                </c:when>
                            </c:choose>>
                            <c:choose>
                                <c:when test="${f.facilityName != ''}">
                                    <c:out value="${f.facilityName}"/>
                                </c:when>
                                <c:otherwise>
                                    <c:out value="${fn:split(f.facilityDns, '.')[0]}"/>
                                </c:otherwise>
                            </c:choose>
                            (<c:out value="${f.facilityStation}"/>)
                        </option>
                    </c:forEach>
                </select> 
            </div>
        </div>
        <div class="col-xs-offset-3 col-sm-offset-3 col-md-offset-3 col-lg-offset-3">
            <div class="margin-left-10 checkbox form-group pad-bottom-10">
                <label for="modiftyOption1">
                    <input type="checkbox" name="option1" value="Restriction Form Received and Validated" id="modiftyOption1" required aria-label="Form Validation for Restriction Form 10-0525a Received and Validated checkbox required"/>
                    <f:requiredField altText="Restriction Form Received and Validated is a required field." labelText="Form Validation for Restriction Form 10-0525a Received and Validated" />
                </label> 
            </div>
        </div>
        <div class="form-group" id="modifyRestrictionsSignatureDateDiv">
            <label for="modifyRestrictionsSignatureDate" class="col-xs-3 col-sm-3 col-md-3 col-lg-3 control-label"><f:requiredField altText="Patient Signature Date is a required field." labelText="Patient Signature Date" /></label> 
            <div class="col-xs-4 col-sm-4 col-md-4 col-lg-4">
                <input type="text" name="signatureDate" id="modifyRestrictionsSignatureDate" class="form-control vap-datepicker-input" data-isValidDateMMDDYYYY="isValidDateMMDDYYYY" required />
            </div>
        </div>
        <div class="form-group">
            <label for="comments" class="col-xs-3 col-sm-3 col-md-3 col-lg-3 control-label">Comments</label>
            <div class="col-xs-9 col-sm-9 col-md-9 col-lg-9">
                <textarea id="comments" name="comments" rows="5" class="form-control"></textarea>
            </div>
        </div>
    </form>
</div>