<%--
    Document   : scheduledExports
    Created on : Sep 8, 2016, 12:08:31 PM
    Author     : Zachary Tubb
--%>
<%@taglib uri="http://java.sun.com/jsp/jstl/core" prefix="c" %>
<%@taglib tagdir="/WEB-INF/tags/formatting" prefix="f" %>
<%@taglib uri="http://java.sun.com/jsp/jstl/functions" prefix="fn" %>

<f:mainTemplate headerText="Scheduled Exports">
    <script language="JavaScript">
        $(document).ready(function(){
            //init toast for IE (otherwise the first toast that we call won't alert) 
            VAP.common.initToastForInternetExplorer();
            
            VAP.common.setProcessingForDataTable('reportTable', 'divReportPanel');

            VAP.common.reportTable = $('#reportTable').DataTable( {
                destroy: true,
                searching: false,
                processing: true,
                serverSide: true,
                order: [[3, 'desc']],
                ajax: "ScheduledExports.do_sec?getUserExports=true",
                columnDefs: [
                    {orderable: false, targets: 0},
                    {className: "vertical-align-center", targets: 0}
                ],
                lengthMenu: [ 10, 25, 50, 100, 250, 500 ],
                pageLength: 25
            });
            
            $('#refreshButton').on('click', function() {
                $('#reportTable .dataTables_empty').html(''); //blank out the empty table message if there is one
                $('#reportTable').DataTable().ajax.reload();
                
                //disable the download and remove buttons again
                $('#downloadButton').addClass('disabled');
                $('#downloadButton').attr('aria-label', 'Download, Inactive');
                $('#downloadButton').attr('title', 'Download, Inactive');
                $('#removeButton').addClass('disabled');
                $('#removeButton').attr('aria-label', 'Remove, Inactive');
                $('#removeButton').attr('title', 'Remove, Inactive');
            });
            
            // DataTables overrides
            VAP.common.reportTable.on('draw', function () {
                VAP.common.addAriaToDataTable('reportTable', VAP.common.reportTable);
                VAP.common.addSrollingToDataTable('reportTable');
            });
            
            //handler for onchange of the checkboxes 
            $(document).on('change', '.scheduledExportActionCheckbox', function(){
                //alert that a checkbox was checked or not checked
                var status = $(this).is(":checked") ? true : false;
                var alertMessage = $(this).attr('aria-label');
                
                if(status) {
                    VAP.common.fireAriaAlert(alertMessage + ' checked.');
                } else {
                    VAP.common.fireAriaAlert(alertMessage + ' not checked.');
                }
                
                if($('input[name=scheduledExportAction]:checked').length > 0) {
                    $('#downloadButton').removeClass('disabled');
                    $('#downloadButton').attr('aria-label', 'Download, Active');
                    $('#downloadButton').attr('title', 'Download, Active');
                    $('#removeButton').removeClass('disabled');
                    $('#removeButton').attr('aria-label', 'Remove, Active');
                    $('#removeButton').attr('title', 'Remove, Active');
                    
                } else {
                    $('#downloadButton').addClass('disabled');
                    $('#downloadButton').attr('aria-label', 'Download, Inactive');
                    $('#downloadButton').attr('title', 'Download, Inactive');
                    $('#removeButton').addClass('disabled');
                    $('#removeButton').attr('aria-label', 'Remove, Inactive');
                    $('#removeButton').attr('title', 'Remove, Inactive');
                }
            });
            
            //handler for the select all
            $(document).on('change', '#master', function(){
                var status = $(this).is(":checked") ? true : false;
                $(".scheduledExportActionCheckbox").prop("checked",status);
                
                if(status) {
                    VAP.common.fireAriaAlert('Select All checkbox checked.');
                } else {
                    VAP.common.fireAriaAlert('Select All checkbox not checked.');
                }
                
                if($('input[name=scheduledExportAction]:checked').length > 0) {
                    $('#downloadButton').removeClass('disabled');
                    $('#downloadButton').attr('aria-label', 'Download, Active');
                    $('#downloadButton').attr('title', 'Download, Active');
                    $('#removeButton').removeClass('disabled');
                    $('#removeButton').attr('aria-label', 'Remove, Active');
                    $('#removeButton').attr('title', 'Remove, Active');
                } else {
                    $('#downloadButton').addClass('disabled');
                    $('#downloadButton').attr('aria-label', 'Download, Inactive');
                    $('#downloadButton').attr('title', 'Download, Inactive');
                    $('#removeButton').addClass('disabled');
                    $('#removeButton').attr('aria-label', 'Remove, Inactive');
                    $('#removeButton').attr('title', 'Remove, Inactive');
                }
            });
            
            $('#downloadButton').on('click', function(){
                if(!$(this).hasClass('disabled')) {
                    
                    //check if all the exports selected are in a COMPLETE status and only allow download if they ALL are. otherwise, pop a modal saying why the user cannot download. 
                    var canDownload = true;
                    $('input[name=scheduledExportAction]:checked').each(function(index, value) {
                        if(!$(value).data('iscomplete')) {
                            canDownload = false;
                        }
                    });
                    
                    if(canDownload) {
                        var urlToUse = "";
                        //looks like we are cleared to download
                        if($('input[name=scheduledExportAction]:checked').length == 1) {
                            //the user has selected only one export to download, so let's download that one directly
                            //location.href = 'ScheduledExports.do_sec?getExport=true&docId=' + $('input[name=scheduledExportAction]:checked').attr('id');
                            urlToUse = 'ScheduledExports.do_sec?getExport=true&docId=' + $('input[name=scheduledExportAction]:checked').attr('id');
                        } else {
                            //loop over the boxes that are checked and do the call to zip them up and then download them in one zip file
                             var downloadIds = $('input[name=scheduledExportAction]:checked').map(function() { 
                                return $(this).attr('id'); 
                            }).get().join(',');
                            
                            //location.href = 'ScheduledExports.do_sec?getMultipleExports=true&ids=' + downloadIds;
                            urlToUse = 'ScheduledExports.do_sec?getMultipleExports=true&ids=' + downloadIds;
                        }
                        
                        $.toast({
                            text: 'The export file has started downloading to your computer.',
                            heading : 'Download Started'
                        });
                        
                        var doDownload = getDownload(urlToUse);
                        
                        $.when(doDownload).done(function() { 
                            $.toast({
                                text: 'The export file has been downloaded to your computer.',
                                heading : 'Download Complete'
                            });
                        });
                        
                    } else {
                        //show the modal stating why the user cannot download at this time
                        $('#confirmCannotDownloadScheduledExportModal').modal('show');
                    }
                }
            });
            
            function getDownload(urlToUse) {
                return $.ajax({
                    url: urlToUse, // 'ScheduledExports.do_sec?getExport=true&docId=' + $('input[name=scheduledExportAction]:checked').attr('id'),
                    method: 'GET',
                    headers: { 'skipLoadingAlert' : 'true' },
                    xhrFields: {
                        responseType: 'blob'
                    },
                    success: function (data, status, xhr) {
                        var a = document.createElement('a');

                        if(!VAP.common.isInternetExplorer()) { //because firefox (errors out in IE if we keep this in)
                            document.body.appendChild(a); 
                        }

                        var url = window.URL.createObjectURL(data);
                        a.href = url;
                        a.download = xhr.getResponseHeader("VAP-Download-Filename");
                        a.click();

                        if(VAP.common.isInternetExplorer()) {
                            window.navigator.msSaveOrOpenBlob(data, xhr.getResponseHeader("VAP-Download-Filename"));
                        } else { //for firefox and chrome 
                            window.URL.revokeObjectURL(url);
                        }
                    }
                });
            };
            
            $('#removeButton').on('click', function(){
                if(!$(this).hasClass('disabled')) {
                   //pop the confirm removal modal since the button is NOT disabled
                   $('#confirmRemoveScheduledExportModal').modal('show');
                }
            });
            
            $('#confirmRemoveScheduledExportModal').on('show.bs.modal', function (event) {
                var removalText = 'this (1) export';
                var howManyExports = $('input[name=scheduledExportAction]:checked').length;
                
                if(howManyExports > 1) {
                    removalText = 'these (' + howManyExports + ') exports';
                }
                
                $('#removalText').html(removalText);
            });
            
            $('#btnConfirmRemoveScheduledExport').on('click', function(){
                //loop over and create a list of exports to delete
                var removalIds = $('input[name=scheduledExportAction]:checked').map(function() { 
                    return $(this).attr('id'); 
                }).get().join(',');

                //if an error message is still showing, hide it
                $('#scheduledExportErrorMessage').hide();

                //make the ajax call to delete them
                $.ajax({
                    url: "ScheduledExports.do_sec?removeExports=true&ids=" + removalIds, 
                    headers: { 'skipLoadingAlert' : 'true' },
                    success: function (result) {
                        var info = $.parseJSON(result);

                        //refresh the table
                        $('#reportTable .dataTables_empty').html(''); //blank out the empty table message if there is one
                        $('#reportTable').DataTable().ajax.reload();

                        //pop the success message
                        $('#scheduledExportSuccessMessageBody').html('Successfully removed ' + $('input[name=scheduledExportAction]:checked').length + ' export(s).');
                        $('#scheduledExportSuccessMessage').show().delay(5000).slideUp();
                        
                        //disable the download and remove buttons again
                        $('#downloadButton').addClass('disabled');
                        $('#downloadButton').attr('aria-label', 'Download, Inactive');
                        $('#downloadButton').attr('title', 'Download, Inactive');
                        $('#removeButton').addClass('disabled');
                        $('#removeButton').attr('aria-label', 'Remove, Inactive');
                        $('#removeButton').attr('title', 'Remove, Inactive');
                    }, 
                    error: function(result) {
                       $('#scheduledExportErrorMessageBody').html('There was an error removing the selected export(s). Please try again later.');
                       $('#scheduledExportErrorMessage').show(); 
                    }
                });
            });
        });
    </script>
    
    <div class="alert alert-info">
        Once an export generation task or an advanced scheduled task has been processed, the Excel and CSV exported files will appear 
        on this page for download. Utilize the check boxes, on the left, to select a file to either download or delete. Multiple files 
        can be selected at one time, if necessary. However, only exported files that reflect a status of "Completed" can be downloaded.
    </div>
    <div id="scheduledExportSuccessMessage" class="alert alert-success alert-dismissible pad-bottom-10 pad-top-10 pad-left-10" style="display: none;" role="alert">
        <button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>
        <div id="scheduledExportSuccessMessageBody"></div>
    </div>
    <div id="scheduledExportErrorMessage" class="alert alert-danger alert-dismissible pad-bottom-10 pad-top-10 pad-left-10" style="display: none;" role="alert">
        <button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>
        <div id="scheduledExportErrorMessageBody"></div>
    </div>
    <div><div aria-live="assertive" role="alert" id="reportTable_info"></div></div>
    <div id="divReportContent" class="col-xs-12 col-sm-12 col-md-12 col-lg-12">
        <div class="panel panel-default" id="divReportPanel">
            <div class="panel-body">
                <div class="pull-right">
                    <button class="btn btn-primary disabled" id="downloadButton" title="Download, Inactive" aria-label="Download, Inactive">Download <span class="fa fa-download pad-right-5" aria-hidden="true"></span></button>
                    <button class="btn btn-danger disabled" id="removeButton" title="Remove, Inactive" aria-label="Remove, Inactive">Remove <span class="fa fa-trash" aria-hidden="true"></span></button>
                    <button class="btn" id="refreshButton" title="Refresh" aria-label="Refresh">Refresh <span class="fa fa-refresh" aria-hidden="true"></span></button>
                </div>
                <table id="reportTable" class="display table table-striped" width="100%" cellspacing="0" role="presentation">
                    <thead>
                        <tr role="row">
                            <th role="columnheader" style="width: 80px;" class="js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="<c:out value="${descriptors['manageBatchesCheckBoxHeader']}"/>"><span id="selectAllHeader" data-simpletooltip-target="tableheader"><input type="checkbox" class="vertical-align-bottom" name="master" id="master" aria-label="Select All Rows" /> Select All</span></th>
                            <th role="columnheader" class="text-center js-simple-tooltip" style="min-width: 320px;" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="<c:out value="${descriptors['scheduledExportsReportColumn']}" />"><span data-simpletooltip-target="tableheader">Report</span></th>
                            <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="<c:out value="${descriptors['scheduledExportsFormatColumn']}" />"><span data-simpletooltip-target="tableheader">Format</span></th>
                            <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="<c:out value="${descriptors['scheduledExportsDateColumn']}" />"><span data-simpletooltip-target="tableheader">Date Generated (CT)</span></th>
                            <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary-left" data-simpletooltip-text="<c:out value="${descriptors['scheduledExportsStatusColumn']}" />"><span data-simpletooltip-target="tableheader">Status</span></th>
                        </tr>
                    </thead>
                </table>
            </div>
        </div>
    </div>
    
    <div class="modal fade" id="confirmRemoveScheduledExportModal" role="dialog" data-backdrop="static" aria-labelledby="confirmRemoveScheduledExportModalLabel">
        <div class="vertical-alignment-helper">
            <div class="modal-dialog vertical-align-center" role="document">
                <div class="modal-content" tabindex="-1" id="confirmRemoveScheduledExportModalContent">
                    <div class="modal-header">
                        <h4 class="modal-title" id="confirmRemoveScheduledExportModalLabel">Confirm Remove Scheduled Export(s)</h4>
                    </div>
                    <div class="modal-body" aria-live="assertive" role="alert">
                        Are you sure you want to permanently remove <span id="removalText"></span>? This cannot be undone. 
                    </div>
                    <div class="modal-footer">
                        <button type="button" class="btn btn-default" data-dismiss="modal" title="Cancel" aria-label="Cancel">Cancel</button>
                        <button type="button" id="btnConfirmRemoveScheduledExport" class="btn btn-primary" data-dismiss="modal" title="Confirm" aria-label="Confirm">Confirm</button>
                    </div>
                </div>
            </div>
        </div>
    </div>
    
    <div class="modal fade" id="confirmCannotDownloadScheduledExportModal" role="dialog" data-backdrop="static" aria-labelledby="confirmCannotDownloadScheduledExportModalLabel">
        <div class="vertical-alignment-helper">
            <div class="modal-dialog vertical-align-center" role="document">
                <div class="modal-content" tabindex="-1" id="confirmCannotDownloadScheduledExportModalContent">
                    <div class="modal-header">
                        <h4 class="modal-title" id="confirmCannotDownloadScheduledExportModalLabel">Scheduled Exports Download Alert</h4>
                    </div>
                    <div class="modal-body" aria-live="assertive" role="alert">
                        Your request to download the selected files cannot be completed. Please deselect any files that are not in a completed state.
                    </div>
                    <div class="modal-footer">
                        <button type="button" class="btn btn-default" data-dismiss="modal" title="Close" aria-label="Close">Close</button>
                    </div>
                </div>
            </div>
        </div>
    </div>
</f:mainTemplate>